@@ -18,24 +18,17 @@ extern int	clkintr_pending;
 extern u_int	i8254_freq;
 extern int	i8254_max_count;
 extern uint64_t	tsc_freq;
-extern int	tsc_is_broken;
 extern int	tsc_is_invariant;
+extern int	tsc_perf_stat;
 
 void	i8254_init(void);
 
-struct trapframe;
-
-int	hardclockintr(struct trapframe *frame);
-int	statclockintr(struct trapframe *frame);
-int	profclockintr(struct trapframe *frame);
-
 /*
  * Driver to clock driver interface.
  */
 
 void	startrtclock(void);
 void	init_TSC(void);
-void	init_TSC_tc(void);
 
 #define	HAS_TIMER_SPKR 1
 int	timer_spkr_acquire(void);
