@@ -44,6 +44,7 @@ __FBSDID("$FreeBSD$");
 #include <sys/eventhandler.h>
 #include <sys/malloc.h>
 #include <sys/cons.h>
+#include <geom/geom.h>
 #include <geom/geom_disk.h>
 #endif /* _KERNEL */
 
@@ -88,8 +89,7 @@ typedef enum {
 	DA_Q_NONE		= 0x00,
 	DA_Q_NO_SYNC_CACHE	= 0x01,
 	DA_Q_NO_6_BYTE		= 0x02,
-	DA_Q_NO_PREVENT		= 0x04,
-	DA_Q_4K			= 0x08
+	DA_Q_NO_PREVENT		= 0x04
 } da_quirks;
 
 typedef enum {
@@ -112,8 +112,6 @@ struct disk_params {
 	u_int8_t  secs_per_track;
 	u_int32_t secsize;	/* Number of bytes/sector */
 	u_int64_t sectors;	/* total number sectors */
-	u_int     stripesize;
-	u_int     stripeoffset;
 };
 
 struct da_softc {
@@ -124,7 +122,6 @@ struct da_softc {
 	da_flags flags;	
 	da_quirks quirks;
 	int	 minimum_cmd_size;
-	int	 error_inject;
 	int	 ordered_tag_count;
 	int	 outstanding_cmds;
 	struct	 disk_params params;
@@ -134,6 +131,7 @@ struct da_softc {
 	struct sysctl_ctx_list	sysctl_ctx;
 	struct sysctl_oid	*sysctl_tree;
 	struct callout		sendordered_c;
+	uint64_t wwpn;
 };
 
 struct da_quirk_entry {
@@ -566,223 +564,7 @@ static struct da_quirk_entry da_quirk_table[] =
 		 */
 		{T_DIRECT, SIP_MEDIA_REMOVABLE, "Sony", "Sony DSC", "*"},
 		/*quirks*/ DA_Q_NO_SYNC_CACHE | DA_Q_NO_PREVENT
-	},
-	/* ATA/SATA devices over SAS/USB/... */
-	{
-		/* Hitachi Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "Hitachi", "H??????????E3*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Samsung Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "SAMSUNG HD155UI*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Samsung Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "SAMSUNG", "HD155UI*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Samsung Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "SAMSUNG HD204UI*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Samsung Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "SAMSUNG", "HD204UI*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Barracuda Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST????DL*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Barracuda Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST????DL", "*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Barracuda Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST???DM*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Barracuda Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST???DM*", "*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Barracuda Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST????DM*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Barracuda Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST????DM", "*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST9500423AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST950042", "3AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST9500424AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST950042", "4AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST9640423AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST964042", "3AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST9640424AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST964042", "4AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST9750420AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST975042", "0AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST9750422AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST975042", "2AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST9750423AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST975042", "3AS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Thin Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "ST???LT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* Seagate Momentus Thin Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ST???LT*", "*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Caviar Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "WDC WD????RS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Caviar Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "WDC WD??", "??RS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Caviar Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "WDC WD????RX*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Caviar Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "WDC WD??", "??RX*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Caviar Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "WDC WD??????RS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Caviar Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "WDC WD??", "????RS*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Caviar Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "WDC WD??????RX*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Caviar Green Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "WDC WD??", "????RX*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Scorpio Black Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "WDC WD???PKT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Scorpio Black Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "WDC WD??", "?PKT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Scorpio Black Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "WDC WD?????PKT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Scorpio Black Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "WDC WD??", "???PKT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Scorpio Blue Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "WDC WD???PVT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Scorpio Blue Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "WDC WD??", "?PVT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Scorpio Blue Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "ATA", "WDC WD?????PVT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
-	{
-		/* WDC Scorpio Blue Advanced Format (4k) drives */
-		{ T_DIRECT, SIP_MEDIA_FIXED, "WDC WD??", "???PVT*", "*" },
-		/*quirks*/DA_Q_4K
-	},
+	}
 };
 
 static	disk_strategy_t	dastrategy;
@@ -803,7 +585,7 @@ static  int		daerror(union ccb *ccb, u_int32_t cam_flags,
 static void		daprevent(struct cam_periph *periph, int action);
 static int		dagetcapacity(struct cam_periph *periph);
 static void		dasetgeom(struct cam_periph *periph, uint32_t block_len,
-				  uint64_t maxsector, u_int lbppbe, u_int lalba);
+				  uint64_t maxsector);
 static timeout_t	dasendorderedtag;
 static void		dashutdown(void *arg, int howto);
 
@@ -876,7 +658,7 @@ daopen(struct disk *dp)
 	}
 
 	if (cam_periph_acquire(periph) != CAM_REQ_CMP) {
-		return (ENXIO);
+		return(ENXIO);
 	}
 
 	cam_periph_lock(periph);
@@ -905,8 +687,6 @@ daopen(struct disk *dp)
 
 		softc->disk->d_sectorsize = softc->params.secsize;
 		softc->disk->d_mediasize = softc->params.secsize * (off_t)softc->params.sectors;
-		softc->disk->d_stripesize = softc->params.stripesize;
-		softc->disk->d_stripeoffset = softc->params.stripeoffset;
 		/* XXX: these are not actually "firmware" values, so they may be wrong */
 		softc->disk->d_fwsectors = softc->params.secs_per_track;
 		softc->disk->d_fwheads = softc->params.heads;
@@ -937,18 +717,19 @@ daclose(struct disk *dp)
 
 	periph = (struct cam_periph *)dp->d_drv1;
 	if (periph == NULL)
-		return (0);	
+		return (ENXIO);	
 
 	cam_periph_lock(periph);
 	if ((error = cam_periph_hold(periph, PRIBIO)) != 0) {
 		cam_periph_unlock(periph);
 		cam_periph_release(periph);
-		return (0);
+		return (error);
 	}
 
 	softc = (struct da_softc *)periph->softc;
 
-	if ((softc->quirks & DA_Q_NO_SYNC_CACHE) == 0) {
+	if ((softc->quirks & DA_Q_NO_SYNC_CACHE) == 0
+	 && (softc->flags & DA_FLAG_PACK_INVALID) == 0) {
 		union	ccb *ccb;
 
 		ccb = cam_periph_getccb(periph, CAM_PRIORITY_NORMAL);
@@ -972,10 +753,10 @@ daclose(struct disk *dp)
 				int asc, ascq;
 				int sense_key, error_code;
 
-				scsi_extract_sense(&ccb->csio.sense_data,
-						   &error_code,
-						   &sense_key, 
-						   &asc, &ascq);
+				scsi_extract_sense_len(&ccb->csio.sense_data,
+				    ccb->csio.sense_len - ccb->csio.sense_resid,
+				    &error_code, &sense_key, &asc, &ascq,
+				    /*show_errors*/ 1);
 				if (sense_key != SSD_KEY_ILLEGAL_REQUEST)
 					scsi_sense_print(&ccb->csio);
 			} else {
@@ -1095,9 +876,8 @@ dadump(void *arg, void *virtual, vm_offset_t physical, off_t offset, size_t leng
 				/*data_ptr*/(u_int8_t *) virtual,
 				/*dxfer_len*/length,
 				/*sense_len*/SSD_FULL_SIZE,
-				da_default_timeout * 1000);
+				DA_DEFAULT_TIMEOUT * 1000);		
 		xpt_polled_action((union ccb *)&csio);
-		cam_periph_unlock(periph);
 
 		if ((csio.ccb_h.status & CAM_STATUS_MASK) != CAM_REQ_CMP) {
 			printf("Aborting dump due to I/O error.\n");
@@ -1109,6 +889,7 @@ dadump(void *arg, void *virtual, vm_offset_t physical, off_t offset, size_t leng
 				       csio.ccb_h.status, csio.scsi_status);
 			return(EIO);
 		}
+		cam_periph_unlock(periph);
 		return(0);
 	}
 		
@@ -1135,10 +916,10 @@ dadump(void *arg, void *virtual, vm_offset_t physical, off_t offset, size_t leng
 				int asc, ascq;
 				int sense_key, error_code;
 
-				scsi_extract_sense(&csio.sense_data,
-						   &error_code,
-						   &sense_key, 
-						   &asc, &ascq);
+				scsi_extract_sense_len(&csio.sense_data,
+				    csio.sense_len - csio.sense_resid,
+				    &error_code, &sense_key, &asc, &ascq,
+				    /*show_errors*/ 1);
 				if (sense_key != SSD_KEY_ILLEGAL_REQUEST)
 					scsi_sense_print(&csio);
 			} else {
@@ -1153,6 +934,25 @@ dadump(void *arg, void *virtual, vm_offset_t physical, off_t offset, size_t leng
 	return (0);
 }
 
+static int
+dagetattr(struct bio *bp)
+{
+	int ret = -1;
+	struct cam_periph *periph;
+
+	if (bp->bio_disk == NULL || bp->bio_disk->d_drv1 == NULL)
+		return ENXIO;
+	periph = (struct cam_periph *)bp->bio_disk->d_drv1;
+	if (periph->path == NULL)
+		return ENXIO;
+
+	ret = xpt_getattr(bp->bio_data, bp->bio_length, bp->bio_attribute,
+	    periph->path);
+	if (ret == 0)
+		bp->bio_completed = bp->bio_length;
+	return ret;
+}
+
 static void
 dainit(void)
 {
@@ -1198,8 +998,8 @@ daoninvalidate(struct cam_periph *periph)
 	bioq_flush(&softc->bio_queue, NULL, ENXIO);
 
 	disk_gone(softc->disk);
-	xpt_print(periph->path, "lost device - %d outstanding, %d refs\n",
-		  softc->outstanding_cmds, periph->refcount);
+	xpt_print(periph->path, "lost device - %d outstanding\n",
+		  softc->outstanding_cmds);
 }
 
 static void
@@ -1266,6 +1066,20 @@ daasync(void *callback_arg, u_int32_t code,
 		 && status != CAM_REQ_INPROG)
 			printf("daasync: Unable to attach to new device "
 				"due to status 0x%x\n", status);
+		return;
+	}
+	case AC_ADVINFO_CHANGED:
+	{
+		uintptr_t buftype;
+
+		buftype = (uintptr_t)arg;
+		if (buftype == CDAI_TYPE_PHYS_PATH) {
+			struct da_softc *softc;
+
+			softc = periph->softc;
+			disk_attr_changed(softc->disk, "GEOM::physpath",
+					  M_NOWAIT);
+		}
 		break;
 	}
 	case AC_SENT_BDR:
@@ -1282,12 +1096,12 @@ daasync(void *callback_arg, u_int32_t code,
 		softc->flags |= DA_FLAG_RETRY_UA;
 		LIST_FOREACH(ccbh, &softc->pending_ccbs, periph_links.le)
 			ccbh->ccb_state |= DA_CCB_RETRY_UA;
-		/* FALLTHROUGH*/
-	}
-	default:
-		cam_periph_async(periph, code, path, arg);
 		break;
 	}
+	default:
+		break;
+	}
+	cam_periph_async(periph, code, path, arg);
 }
 
 static void
@@ -1296,6 +1110,7 @@ dasysctlinit(void *context, int pending)
 	struct cam_periph *periph;
 	struct da_softc *softc;
 	char tmpstr[80], tmpstr2[80];
+	struct ccb_trans_settings cts;
 
 	periph = (struct cam_periph *)context;
 	/*
@@ -1322,14 +1137,38 @@ dasysctlinit(void *context, int pending)
 	}
 
 	/*
-	 * Now register the sysctl handler, so the user can the value on
+	 * Now register the sysctl handler, so the user can change the value on
 	 * the fly.
 	 */
-	SYSCTL_ADD_PROC(&softc->sysctl_ctx,SYSCTL_CHILDREN(softc->sysctl_tree),
+	SYSCTL_ADD_PROC(&softc->sysctl_ctx, SYSCTL_CHILDREN(softc->sysctl_tree),
 		OID_AUTO, "minimum_cmd_size", CTLTYPE_INT | CTLFLAG_RW,
 		&softc->minimum_cmd_size, 0, dacmdsizesysctl, "I",
 		"Minimum CDB size");
 
+	/*
+	 * Add some addressing info.
+	 */
+	memset(&cts, 0, sizeof (cts));
+	xpt_setup_ccb(&cts.ccb_h, periph->path, /*priority*/1);
+	cts.ccb_h.func_code = XPT_GET_TRAN_SETTINGS;
+	cts.type = CTS_TYPE_CURRENT_SETTINGS;
+	cam_periph_lock(periph);
+	xpt_action((union ccb *)&cts);
+	cam_periph_unlock(periph);
+	if (cts.ccb_h.status != CAM_REQ_CMP) {
+		cam_periph_release(periph);
+		return;
+	}
+	if (cts.protocol == PROTO_SCSI && cts.transport == XPORT_FC) {
+		struct ccb_trans_settings_fc *fc = &cts.xport_specific.fc;
+		if (fc->valid & CTS_FC_VALID_WWPN) {
+			softc->wwpn = fc->wwpn;
+			SYSCTL_ADD_UQUAD(&softc->sysctl_ctx,
+			    SYSCTL_CHILDREN(softc->sysctl_tree),
+			    OID_AUTO, "wwpn", CTLFLAG_RD,
+			    &softc->wwpn, "World Wide Port Name");
+		}
+	}
 	cam_periph_release(periph);
 }
 
@@ -1427,17 +1266,6 @@ daregister(struct cam_periph *periph, void *arg)
 
 	TASK_INIT(&softc->sysctl_task, 0, dasysctlinit, periph);
 
-	/*
-	 * Add async callbacks for bus reset and
-	 * bus device reset calls.  I don't bother
-	 * checking if this fails as, in most cases,
-	 * the system will function just fine without
-	 * them and the only alternative would be to
-	 * not attach the device on failure.
-	 */
-	xpt_register_async(AC_SENT_BDR | AC_BUS_RESET | AC_LOST_DEVICE,
-			   daasync, periph, periph->path);
-
 	/*
 	 * Take an exclusive refcount on the periph while dastart is called
 	 * to finish the probe.  The reference will be dropped in dadone at
@@ -1451,7 +1279,7 @@ daregister(struct cam_periph *periph, void *arg)
 	 */
 	callout_init_mtx(&softc->sendordered_c, periph->sim->mtx, 0);
 	callout_reset(&softc->sendordered_c,
-	    (da_default_timeout * hz) / DA_ORDEREDTAG_INTERVAL,
+	    (DA_DEFAULT_TIMEOUT * hz) / DA_ORDEREDTAG_INTERVAL,
 	    dasendorderedtag, softc);
 
 	mtx_unlock(periph->sim->mtx);
@@ -1498,6 +1326,7 @@ daregister(struct cam_periph *periph, void *arg)
 	softc->disk->d_close = daclose;
 	softc->disk->d_strategy = dastrategy;
 	softc->disk->d_dump = dadump;
+	softc->disk->d_getattr = dagetattr;
 	softc->disk->d_name = "da";
 	softc->disk->d_drv1 = periph;
 	if (cpi.maxio == 0)
@@ -1510,11 +1339,38 @@ daregister(struct cam_periph *periph, void *arg)
 	softc->disk->d_flags = 0;
 	if ((softc->quirks & DA_Q_NO_SYNC_CACHE) == 0)
 		softc->disk->d_flags |= DISKFLAG_CANFLUSHCACHE;
-	strlcpy(softc->disk->d_ident, cgd->serial_num,
-	    MIN(sizeof(softc->disk->d_ident), cgd->serial_num_len + 1));
+	cam_strvis(softc->disk->d_descr, cgd->inq_data.vendor,
+	    sizeof(cgd->inq_data.vendor), sizeof(softc->disk->d_descr));
+	strlcat(softc->disk->d_descr, " ", sizeof(softc->disk->d_descr));
+	cam_strvis(&softc->disk->d_descr[strlen(softc->disk->d_descr)],
+	    cgd->inq_data.product, sizeof(cgd->inq_data.product),
+	    sizeof(softc->disk->d_descr) - strlen(softc->disk->d_descr));
+	softc->disk->d_hba_vendor = cpi.hba_vendor;
+	softc->disk->d_hba_device = cpi.hba_device;
+	softc->disk->d_hba_subvendor = cpi.hba_subvendor;
+	softc->disk->d_hba_subdevice = cpi.hba_subdevice;
 	disk_create(softc->disk, DISK_VERSION);
 	mtx_lock(periph->sim->mtx);
 
+	/*
+	 * Add async callbacks for events of interest.
+	 * I don't bother checking if this fails as,
+	 * in most cases, the system will function just
+	 * fine without them and the only alternative
+	 * would be to not attach the device on failure.
+	 */
+	xpt_register_async(AC_SENT_BDR | AC_BUS_RESET
+			 | AC_LOST_DEVICE | AC_ADVINFO_CHANGED,
+			   daasync, periph, periph->path);
+
+	/*
+	 * Emit an attribute changed notification just in case 
+	 * physical path information arrived before our async
+	 * event handler was registered, but after anyone attaching
+	 * to our disk device polled it.
+	 */
+	disk_attr_changed(softc->disk, "GEOM::physpath", M_NOWAIT);
+
 	xpt_schedule(periph, CAM_PRIORITY_DEV);
 
 	return(CAM_REQ_CMP);
@@ -1745,7 +1601,7 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 		if ((done_ccb->ccb_h.status & CAM_STATUS_MASK) != CAM_REQ_CMP) {
 			int error;
 			int sf;
-			
+
 			if ((csio->ccb_h.ccb_state & DA_CCB_RETRY_UA) != 0)
 				sf = SF_RETRY_UA;
 			else
@@ -1760,8 +1616,17 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 				return;
 			}
 			if (error != 0) {
+				int queued_error;
 
-				if (error == ENXIO) {
+				/*
+				 * return all queued I/O with EIO, so that
+				 * the client can retry these I/Os in the
+				 * proper order should it attempt to recover.
+				 */
+				queued_error = EIO;
+
+				if (error == ENXIO
+				 && (softc->flags & DA_FLAG_PACK_INVALID)== 0) {
 					/*
 					 * Catastrophic error.  Mark our pack as
 					 * invalid.
@@ -1773,14 +1638,10 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 					xpt_print(periph->path,
 					    "Invalidating pack\n");
 					softc->flags |= DA_FLAG_PACK_INVALID;
+					queued_error = ENXIO;
 				}
-
-				/*
-				 * return all queued I/O with EIO, so that
-				 * the client can retry these I/Os in the
-				 * proper order should it attempt to recover.
-				 */
-				bioq_flush(&softc->bio_queue, NULL, EIO);
+				bioq_flush(&softc->bio_queue, NULL,
+					   queued_error);
 				bp->bio_error = error;
 				bp->bio_resid = bp->bio_bcount;
 				bp->bio_flags |= BIO_ERROR;
@@ -1802,13 +1663,6 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 			bp->bio_resid = csio->resid;
 			if (csio->resid > 0)
 				bp->bio_flags |= BIO_ERROR;
-			if (softc->error_inject != 0) {
-				bp->bio_error = softc->error_inject;
-				bp->bio_resid = bp->bio_bcount;
-				bp->bio_flags |= BIO_ERROR;
-				softc->error_inject = 0;
-			}
-
 		}
 
 		/*
@@ -1820,6 +1674,11 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 		if (softc->outstanding_cmds == 0)
 			softc->flags |= DA_FLAG_WENT_IDLE;
 
+		if ((softc->flags & DA_FLAG_PACK_INVALID) != 0) {
+			xpt_print(periph->path, "oustanding %d\n",
+				  softc->outstanding_cmds);
+		}
+
 		biodone(bp);
 		break;
 	}
@@ -1882,7 +1741,7 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 				announce_buf[0] = '\0';
 				cam_periph_invalidate(periph);
 			} else {
-				dasetgeom(periph, block_size, maxsector, 0, 0);
+				dasetgeom(periph, block_size, maxsector);
 				dp = &softc->params;
 				snprintf(announce_buf, sizeof(announce_buf),
 				        "%juMB (%ju %u byte sectors: %dH %dS/T "
@@ -1943,9 +1802,10 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 
 				if (have_sense) {
 					sense = &csio->sense_data;
-					scsi_extract_sense(sense, &error_code,
-							   &sense_key, 
-							   &asc, &ascq);
+					scsi_extract_sense_len(sense,
+					    csio->sense_len - csio->sense_resid,
+					    &error_code, &sense_key, &asc,
+					    &ascq, /*show_errors*/ 1);
 				}
 				/*
 				 * Attach to anything that claims to be a
@@ -1990,20 +1850,13 @@ dadone(struct cam_periph *periph, union ccb *done_ccb)
 		}
 		free(csio->data_ptr, M_SCSIDA);
 		if (announce_buf[0] != '\0') {
+			xpt_announce_periph(periph, announce_buf);
 			/*
 			 * Create our sysctl variables, now that we know
 			 * we have successfully attached.
 			 */
-			/* increase the refcount */
-			if (cam_periph_acquire(periph) == CAM_REQ_CMP) {
-				taskqueue_enqueue(taskqueue_thread,
-						  &softc->sysctl_task);
-				xpt_announce_periph(periph, announce_buf);
-			} else {
-				xpt_print(periph->path, "fatal error, "
-				    "could not acquire reference count\n");
-			}
-				
+			(void) cam_periph_acquire(periph);	/* increase the refcount */
+			taskqueue_enqueue(taskqueue_thread,&softc->sysctl_task);
 		}
 		softc->state = DA_STATE_NORMAL;	
 		/*
@@ -2213,7 +2066,7 @@ done:
 			    (uintmax_t) block_len);
 			error = EINVAL;
 		} else
-			dasetgeom(periph, block_len, maxsector, 0, 0);
+			dasetgeom(periph, block_len, maxsector);
 	}
 
 	xpt_release_ccb(ccb);
@@ -2224,8 +2077,7 @@ done:
 }
 
 static void
-dasetgeom(struct cam_periph *periph, uint32_t block_len, uint64_t maxsector,
-    u_int lbppbe, u_int lalba)
+dasetgeom(struct cam_periph *periph, uint32_t block_len, uint64_t maxsector)
 {
 	struct ccb_calc_geometry ccg;
 	struct da_softc *softc;
@@ -2236,17 +2088,6 @@ dasetgeom(struct cam_periph *periph, uint32_t block_len, uint64_t maxsector,
 	dp = &softc->params;
 	dp->secsize = block_len;
 	dp->sectors = maxsector + 1;
-	if (lbppbe > 0) {
-		dp->stripesize = block_len << lbppbe;
-		dp->stripeoffset = (dp->stripesize - block_len * lalba) %
-		    dp->stripesize;
-	} else if (softc->quirks & DA_Q_4K) {
-		dp->stripesize = 4096;
-		dp->stripeoffset = 0;
-	} else {
-		dp->stripesize = 0;
-		dp->stripeoffset = 0;
-	}
 	/*
 	 * Have the controller provide us with a geometry
 	 * for this disk.  The only time the geometry
@@ -2299,7 +2140,7 @@ dasendorderedtag(void *arg)
 	}
 	/* Queue us up again */
 	callout_reset(&softc->sendordered_c,
-	    (da_default_timeout * hz) / DA_ORDEREDTAG_INTERVAL,
+	    (DA_DEFAULT_TIMEOUT * hz) / DA_ORDEREDTAG_INTERVAL,
 	    dasendorderedtag, softc);
 }
 
