@@ -41,9 +41,13 @@
 #include <sys/cdefs.h>
 
 #ifdef __ia64__
-#define FREEBSD32_USRSTACK	((1ul << 32) - IA32_PAGE_SIZE * 2)
+#define FREEBSD32_MAXUSER	((1ul << 32) - IA32_PAGE_SIZE * 2)
+#define	FREEBSD32_SHAREDPAGE	0
+#define FREEBSD32_USRSTACK	FREEBSD32_MAXUSER
 #else
-#define FREEBSD32_USRSTACK	((1ul << 32) - IA32_PAGE_SIZE)
+#define	FREEBSD32_MAXUSER	((1ul << 32) - IA32_PAGE_SIZE)
+#define	FREEBSD32_SHAREDPAGE	(FREEBSD32_MAXUSER - IA32_PAGE_SIZE)
+#define FREEBSD32_USRSTACK	FREEBSD32_SHAREDPAGE
 #endif
 
 #define	IA32_PAGE_SIZE	4096
@@ -54,5 +58,6 @@
 struct syscall_args;
 int ia32_fetch_syscall_args(struct thread *td, struct syscall_args *sa);
 void ia32_set_syscall_retval(struct thread *, int);
+void ia32_fixlimit(struct rlimit *rl, int which);
 
 #endif
