@@ -46,7 +46,7 @@ db_print_thread(void)
 	pid = -1;
 	if (kdb_thread->td_proc != NULL)
 		pid = kdb_thread->td_proc->p_pid;
-	db_printf("[thread pid %d tid %ld ]\n", pid, (long)kdb_thread->td_tid);
+	db_printf("[ thread pid %d tid %ld ]\n", pid, (long)kdb_thread->td_tid);
 }
 
 void
@@ -94,8 +94,7 @@ db_show_threads(db_expr_t addr, boolean_t hasaddr, db_expr_t cnt, char *mod)
 
 	thr = kdb_thr_first();
 	while (!db_pager_quit && thr != NULL) {
-		db_printf("  %6ld (%p) (stack %p)  ", (long)thr->td_tid, thr,
-		    (void *)thr->td_kstack);
+		db_printf("  %6ld (%p)  ", (long)thr->td_tid, thr);
 		prev_jb = kdb_jmpbuf(jb);
 		if (setjmp(jb) == 0) {
 			if (db_trace_thread(thr, 1) != 0)
@@ -110,7 +109,7 @@ db_show_threads(db_expr_t addr, boolean_t hasaddr, db_expr_t cnt, char *mod)
  * Lookup a thread based on a db expression address.  We assume that the
  * address was parsed in hexadecimal.  We reparse the address in decimal
  * first and try to treat it as a thread ID to find an associated thread.
- * If that fails and check_pid is true, we treat the decimal value as a
+ * If that fails and check_pid is true, we terat the decimal value as a
  * PID.  If that matches a process, we return the first thread in that
  * process.  Otherwise, we treat the addr as a pointer to a thread.
  */
