@@ -270,9 +270,9 @@ ale_mediastatus(struct ifnet *ifp, struct ifmediareq *ifmr)
 	mii = device_get_softc(sc->ale_miibus);
 
 	mii_pollstat(mii);
+	ALE_UNLOCK(sc);
 	ifmr->ifm_status = mii->mii_media_status;
 	ifmr->ifm_active = mii->mii_media_active;
-	ALE_UNLOCK(sc);
 }
 
 static int
@@ -287,7 +287,7 @@ ale_mediachange(struct ifnet *ifp)
 	ALE_LOCK(sc);
 	mii = device_get_softc(sc->ale_miibus);
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 	error = mii_mediachg(mii);
 	ALE_UNLOCK(sc);
 
@@ -328,7 +328,7 @@ ale_get_macaddr(struct ale_softc *sc)
 		CSR_WRITE_4(sc, ALE_SPI_CTRL, reg);
 	}
 
-	if (pci_find_extcap(sc->ale_dev, PCIY_VPD, &vpdc) == 0) {
+	if (pci_find_cap(sc->ale_dev, PCIY_VPD, &vpdc) == 0) {
 		/*
 		 * PCI VPD capability found, let TWSI reload EEPROM.
 		 * This will set ethernet address of controller.
@@ -542,7 +542,7 @@ ale_attach(device_t dev)
 	}
 
 	/* Get DMA parameters from PCIe device control register. */
-	if (pci_find_extcap(dev, PCIY_EXPRESS, &i) == 0) {
+	if (pci_find_cap(dev, PCIY_EXPRESS, &i) == 0) {
 		sc->ale_flags |= ALE_FLAG_PCIE;
 		burst = pci_read_config(dev, i + 0x08, 2);
 		/* Max read request size. */
@@ -589,7 +589,7 @@ ale_attach(device_t dev)
 	IFQ_SET_READY(&ifp->if_snd);
 	ifp->if_capabilities = IFCAP_RXCSUM | IFCAP_TXCSUM | IFCAP_TSO4;
 	ifp->if_hwassist = ALE_CSUM_FEATURES | CSUM_TSO;
-	if (pci_find_extcap(dev, PCIY_PMG, &pmc) == 0) {
+	if (pci_find_cap(dev, PCIY_PMG, &pmc) == 0) {
 		sc->ale_flags |= ALE_FLAG_PMCAP;
 		ifp->if_capabilities |= IFCAP_WOL_MAGIC | IFCAP_WOL_MCAST;
 	}
@@ -726,7 +726,10 @@ ale_detach(device_t dev)
 #define	ALE_SYSCTL_STAT_ADD32(c, h, n, p, d)	\
 	    SYSCTL_ADD_UINT(c, h, OID_AUTO, n, CTLFLAG_RD, p, 0, d)
 
-#if __FreeBSD_version > 800000
+#if __FreeBSD_version >= 900030
+#define	ALE_SYSCTL_STAT_ADD64(c, h, n, p, d)	\
+	    SYSCTL_ADD_UQUAD(c, h, OID_AUTO, n, CTLFLAG_RD, p, d)
+#elif __FreeBSD_version > 800000
 #define	ALE_SYSCTL_STAT_ADD64(c, h, n, p, d)	\
 	    SYSCTL_ADD_QUAD(c, h, OID_AUTO, n, CTLFLAG_RD, p, d)
 #else
@@ -1464,7 +1467,7 @@ ale_setwol(struct ale_softc *sc)
 
 	ALE_LOCK_ASSERT(sc);
 
-	if (pci_find_extcap(sc->ale_dev, PCIY_PMG, &pmc) != 0) {
+	if (pci_find_cap(sc->ale_dev, PCIY_PMG, &pmc) != 0) {
 		/* Disable WOL. */
 		CSR_WRITE_4(sc, ALE_WOL_CFG, 0);
 		reg = CSR_READ_4(sc, ALE_PCIE_PHYMISC);
@@ -1543,7 +1546,7 @@ ale_resume(device_t dev)
 	sc = device_get_softc(dev);
 
 	ALE_LOCK(sc);
-	if (pci_find_extcap(sc->ale_dev, PCIY_PMG, &pmc) == 0) {
+	if (pci_find_cap(sc->ale_dev, PCIY_PMG, &pmc) == 0) {
 		/* Disable PME and clear PME status. */
 		pmstat = pci_read_config(sc->ale_dev,
 		    pmc + PCIR_POWER_STATUS, 2);
