@@ -213,10 +213,6 @@ ata_ali_ch_attach(device_t dev)
 	if (ch->dma.max_iosize > 256 * 512)
 		ch->dma.max_iosize = 256 * 512;
     }
-#ifdef ATA_CAM
-	if (ctlr->chip->cfg2 & ALI_NEW)
-		ch->flags |= ATA_NO_ATAPI_DMA;
-#endif
 
     return 0;
 }
