@@ -56,7 +56,6 @@ static int ata_cyrix_chipinit(device_t dev);
 static int ata_cyrix_ch_attach(device_t dev);
 static int ata_cyrix_setmode(device_t dev, int target, int mode);
 
-
 /*
  * Cyrix chipset support functions
  */
@@ -89,15 +88,12 @@ static int
 ata_cyrix_ch_attach(device_t dev)
 {
 	struct ata_channel *ch = device_get_softc(dev);
-	int error;
  
-	error = ata_pci_ch_attach(dev);
 	ch->dma.alignment = 16;
 	ch->dma.max_iosize = 64 * DEV_BSIZE;
-	return (error);
+	return (ata_pci_ch_attach(dev));
 }
 
-
 static int
 ata_cyrix_setmode(device_t dev, int target, int mode)
 {
