@@ -71,15 +71,12 @@
 #define	BGE_STATS_BLOCK_END		0x00000AFF
 #define	BGE_STATUS_BLOCK		0x00000B00
 #define	BGE_STATUS_BLOCK_END		0x00000B4F
-#define	BGE_SRAM_FW_MB			0x00000B50
-#define	BGE_SRAM_DATA_SIG		0x00000B54
-#define	BGE_SRAM_DATA_CFG		0x00000B58
-#define	BGE_SRAM_FW_CMD_MB		0x00000B78
-#define	BGE_SRAM_FW_CMD_LEN_MB		0x00000B7C
-#define	BGE_SRAM_FW_CMD_DATA_MB		0x00000B80
-#define	BGE_SRAM_FW_DRV_STATE_MB	0x00000C04
-#define	BGE_SRAM_MAC_ADDR_HIGH_MB	0x00000C14
-#define	BGE_SRAM_MAC_ADDR_LOW_MB	0x00000C18
+#define	BGE_SOFTWARE_GENCOMM		0x00000B50
+#define	BGE_SOFTWARE_GENCOMM_SIG	0x00000B54
+#define	BGE_SOFTWARE_GENCOMM_NICCFG	0x00000B58
+#define	BGE_SOFTWARE_GENCOMM_FW		0x00000B78
+#define	BGE_SOFTWARE_GENNCOMM_FW_LEN	0x00000B7C
+#define	BGE_SOFTWARE_GENNCOMM_FW_DATA	0x00000B80
 #define	BGE_SOFTWARE_GENCOMM_END	0x00000FFF
 #define	BGE_UNMAPPED			0x00001000
 #define	BGE_UNMAPPED_END		0x00001FFF
@@ -90,24 +87,8 @@
 #define	BGE_SEND_RING_1_TO_4_END	0x00005FFF
 
 /* Firmware interface */
-#define	BGE_SRAM_DATA_SIG_MAGIC		0x4B657654	/* 'KevT' */
-
-#define	BGE_FW_CMD_DRV_ALIVE		0x00000001
-#define	BGE_FW_CMD_PAUSE		0x00000002
-#define	BGE_FW_CMD_IPV4_ADDR_CHANGE	0x00000003
-#define	BGE_FW_CMD_IPV6_ADDR_CHANGE	0x00000004
-#define	BGE_FW_CMD_LINK_UPDATE		0x0000000C
-#define	BGE_FW_CMD_DRV_ALIVE2		0x0000000D
-#define	BGE_FW_CMD_DRV_ALIVE3		0x0000000E
-
-#define	BGE_FW_HB_TIMEOUT_SEC		3
-
-#define	BGE_FW_DRV_STATE_START		0x00000001
-#define	BGE_FW_DRV_STATE_START_DONE	0x80000001
-#define	BGE_FW_DRV_STATE_UNLOAD		0x00000002
-#define	BGE_FW_DRV_STATE_UNLOAD_DONE	0x80000002
-#define	BGE_FW_DRV_STATE_WOL		0x00000003
-#define	BGE_FW_DRV_STATE_SUSPEND	0x00000004
+#define	BGE_FW_DRV_ALIVE		0x00000001
+#define	BGE_FW_PAUSE			0x00000002
 
 /* Mappings for internal memory configuration */
 #define	BGE_STD_RX_RINGS		0x00006000
@@ -258,6 +239,15 @@
 #define	BGE_PCIMISCCTL_ASICREV_SHIFT	16
 
 #define	BGE_HIF_SWAP_OPTIONS	(BGE_PCIMISCCTL_ENDIAN_WORDSWAP)
+#if BYTE_ORDER == LITTLE_ENDIAN
+#define	BGE_DMA_SWAP_OPTIONS \
+	BGE_MODECTL_WORDSWAP_NONFRAME| \
+	BGE_MODECTL_BYTESWAP_DATA|BGE_MODECTL_WORDSWAP_DATA
+#else
+#define	BGE_DMA_SWAP_OPTIONS \
+	BGE_MODECTL_WORDSWAP_NONFRAME|BGE_MODECTL_BYTESWAP_NONFRAME| \
+	BGE_MODECTL_BYTESWAP_DATA|BGE_MODECTL_WORDSWAP_DATA
+#endif
 
 #define	BGE_INIT \
 	(BGE_HIF_SWAP_OPTIONS|BGE_PCIMISCCTL_CLEAR_INTA| \
@@ -330,7 +320,6 @@
 #define	BGE_CHIPID_BCM5717_A0		0x05717000
 #define	BGE_CHIPID_BCM5717_B0		0x05717100
 #define	BGE_CHIPID_BCM5719_A0		0x05719000
-#define	BGE_CHIPID_BCM5720_A0		0x05720000
 #define	BGE_CHIPID_BCM57765_A0		0x57785000
 #define	BGE_CHIPID_BCM57765_B0		0x57785100
 
@@ -355,7 +344,6 @@
 /* BGE_PCI_PRODID_ASICREV ASIC rev. identifiers. */
 #define	BGE_ASICREV_BCM5717		0x5717
 #define	BGE_ASICREV_BCM5719		0x5719
-#define	BGE_ASICREV_BCM5720		0x5720
 #define	BGE_ASICREV_BCM5761		0x5761
 #define	BGE_ASICREV_BCM5784		0x5784
 #define	BGE_ASICREV_BCM5785		0x5785
@@ -800,8 +788,6 @@
 #define	BGE_TXMODE_BIGBACKOFF_ENABLE	0x00000020
 #define	BGE_TXMODE_LONGPAUSE_ENABLE	0x00000040
 #define	BGE_TXMODE_MBUF_LOCKUP_FIX	0x00000100
-#define	BGE_TXMODE_JMB_FRM_LEN		0x00400000
-#define	BGE_TXMODE_CNT_DN_MODE		0x00800000
 
 /* Transmit MAC status register */
 #define	BGE_TXSTAT_RX_XOFFED		0x00000001
@@ -815,8 +801,6 @@
 #define	BGE_TXLEN_SLOTTIME		0x000000FF
 #define	BGE_TXLEN_IPG			0x00000F00
 #define	BGE_TXLEN_CRS			0x00003000
-#define	BGE_TXLEN_JMB_FRM_LEN_MSK	0x00FF0000
-#define	BGE_TXLEN_CNT_DN_VAL_MSK	0xFF000000
 
 /* Receive MAC mode register */
 #define	BGE_RXMODE_RESET		0x00000001
@@ -1274,7 +1258,6 @@
 #define	BGE_CPMU_LSPD_1000MB_CLK	0x360C
 #define	BGE_CPMU_LNK_AWARE_PWRMD	0x3610
 #define	BGE_CPMU_HST_ACC		0x361C
-#define	BGE_CPMU_CLCK_ORIDE		0x3624
 #define	BGE_CPMU_CLCK_STAT		0x3630
 #define	BGE_CPMU_MUTEX_REQ		0x365C
 #define	BGE_CPMU_MUTEX_GNT		0x3660
@@ -1302,9 +1285,6 @@
 #define	BGE_CPMU_HST_ACC_MACCLK_MASK	0x001F0000
 #define	BGE_CPMU_HST_ACC_MACCLK_6_25	0x00130000
 
-/* Clock Speed Override Policy register */
-#define	CPMU_CLCK_ORIDE_MAC_ORIDE_EN	0x80000000
-
 /* CPMU Clock Status register */
 #define	BGE_CPMU_CLCK_STAT_MAC_CLCK_MASK	0x001F0000
 #define	BGE_CPMU_CLCK_STAT_MAC_CLCK_62_5	0x00000000
@@ -1552,7 +1532,6 @@
 #define	BGE_RDMAMODE_MULT_DMA_RD_DIS	0x01000000
 #define	BGE_RDMAMODE_TSO4_ENABLE	0x08000000
 #define	BGE_RDMAMODE_TSO6_ENABLE	0x10000000
-#define	BGE_RDMAMODE_H2BNC_VLAN_DET	0x20000000
 
 /* Read DMA status register */
 #define	BGE_RDMASTAT_PCI_TGT_ABRT_ATTN	0x00000004
@@ -1573,7 +1552,6 @@
 #define	BGE_RDMA_RSRVCTRL_FIFO_HWM_MASK	0x000FF000
 #define	BGE_RDMA_RSRVCTRL_TXMRGN_MASK	0xFFE00000
 
-#define	BGE_RDMA_LSO_CRPTEN_CTRL_BLEN_BD_512	0x00020000
 #define	BGE_RDMA_LSO_CRPTEN_CTRL_BLEN_BD_4K	0x00030000
 #define	BGE_RDMA_LSO_CRPTEN_CTRL_BLEN_LSO_4K	0x000C0000
 
@@ -1897,8 +1875,7 @@
 #define	BGE_MODE_CTL			0x6800
 #define	BGE_MISC_CFG			0x6804
 #define	BGE_MISC_LOCAL_CTL		0x6808
-#define	BGE_RX_CPU_EVENT		0x6810
-#define	BGE_TX_CPU_EVENT		0x6820
+#define	BGE_CPU_EVENT			0x6810
 #define	BGE_EE_ADDR			0x6838
 #define	BGE_EE_DATA			0x683C
 #define	BGE_EE_CTL			0x6840
@@ -1906,8 +1883,6 @@
 #define	BGE_EE_DELAY			0x6848
 #define	BGE_FASTBOOT_PC			0x6894
 
-#define	BGE_RX_CPU_DRV_EVENT		0x00004000
-
 /*
  * NVRAM Control registers
  */
@@ -1964,18 +1939,14 @@
 #define	BGE_MODECTL_WORDSWAP_NONFRAME	0x00000004
 #define	BGE_MODECTL_BYTESWAP_DATA	0x00000010
 #define	BGE_MODECTL_WORDSWAP_DATA	0x00000020
-#define	BGE_MODECTL_BYTESWAP_B2HRX_DATA	0x00000040
-#define	BGE_MODECTL_WORDSWAP_B2HRX_DATA	0x00000080
 #define	BGE_MODECTL_NO_FRAME_CRACKING	0x00000200
 #define	BGE_MODECTL_NO_RX_CRC		0x00000400
 #define	BGE_MODECTL_RX_BADFRAMES	0x00000800
 #define	BGE_MODECTL_NO_TX_INTR		0x00002000
 #define	BGE_MODECTL_NO_RX_INTR		0x00004000
 #define	BGE_MODECTL_FORCE_PCI32		0x00008000
-#define	BGE_MODECTL_B2HRX_ENABLE	0x00008000
 #define	BGE_MODECTL_STACKUP		0x00010000
 #define	BGE_MODECTL_HOST_SEND_BDS	0x00020000
-#define	BGE_MODECTL_HTX2B_ENABLE	0x00040000
 #define	BGE_MODECTL_TX_NO_PHDR_CSUM	0x00100000
 #define	BGE_MODECTL_RX_NO_PHDR_CSUM	0x00800000
 #define	BGE_MODECTL_TX_ATTN_INTR	0x01000000
@@ -2081,10 +2052,10 @@
  * This magic number is written to the firmware mailbox at 0xb50
  * before a software reset is issued.  After the internal firmware
  * has completed its initialization it will write the opposite of
- * this value, ~BGE_SRAM_FW_MB_MAGIC, to the same location,
- * allowing the driver to synchronize with the firmware.
+ * this value, ~BGE_MAGIC_NUMBER, to the same location, allowing the
+ * driver to synchronize with the firmware.
  */
-#define	BGE_SRAM_FW_MB_MAGIC	0x4B657654
+#define	BGE_MAGIC_NUMBER                0x4B657654
 
 typedef struct {
 	uint32_t		bge_addr_hi;
@@ -2305,8 +2276,7 @@ struct bge_status_block {
 #define	BCOM_DEVICEID_BCM5717		0x1655
 #define	BCOM_DEVICEID_BCM5718		0x1656
 #define	BCOM_DEVICEID_BCM5719		0x1657
-#define	BCOM_DEVICEID_BCM5720_PP	0x1658	/* Not released to public. */
-#define	BCOM_DEVICEID_BCM5720		0x165F
+#define	BCOM_DEVICEID_BCM5720		0x1658
 #define	BCOM_DEVICEID_BCM5721		0x1659
 #define	BCOM_DEVICEID_BCM5722		0x165A
 #define	BCOM_DEVICEID_BCM5723		0x165B
@@ -2827,7 +2797,6 @@ struct bge_softc {
 #define	BGE_FLAG_4G_BNDRY_BUG	0x02000000
 #define	BGE_FLAG_RX_ALIGNBUG	0x04000000
 #define	BGE_FLAG_SHORT_DMA_BUG	0x08000000
-#define	BGE_FLAG_4K_RDMA_BUG	0x10000000
 	uint32_t		bge_phy_flags;
 #define	BGE_PHY_NO_WIRESPEED	0x00000001
 #define	BGE_PHY_ADC_BUG		0x00000002
@@ -2864,7 +2833,6 @@ struct bge_softc {
 	int			bge_timer;
 	int			bge_forced_collapse;
 	int			bge_forced_udpcsum;
-	int			bge_msi;
 	int			bge_csum_features;
 	struct callout		bge_stat_ch;
 	uint32_t		bge_rx_discards;
