@@ -675,14 +675,13 @@ rescan:
 #ifdef EP_LOCAL_STATS
 						sc->tx_underrun++;
 #endif
-					} else {
-						if (status & TXS_JABBER);
-						else
-							++ifp->if_collisions;
-							/* TXS_MAX_COLLISION
-							 * we shouldn't get
-							 * here
-							 */
+					}
+					if (status & TXS_MAX_COLLISION) {
+						/*
+						 * TXS_MAX_COLLISION we
+						 * shouldn't get here
+						 */
+						++ifp->if_collisions;
 					}
 					++ifp->if_oerrors;
 					CSR_WRITE_2(sc, EP_COMMAND, TX_ENABLE);
@@ -816,7 +815,7 @@ read_again:
 #endif
 		EP_FRST(sc, F_RX_FIRST);
 		status = CSR_READ_2(sc, EP_W1_RX_STATUS);
-		if (!status & ERR_RX_INCOMPLETE) {
+		if (!(status & ERR_RX_INCOMPLETE)) {
 			/*
 			 * We see if by now, the packet has completly
 			 * arrived
