@@ -79,7 +79,7 @@
 #define HME_SEB_STAT_TXEACK	0x04000000	/* error during tx DMA */
 #define HME_SEB_STAT_TXLERR	0x08000000	/* late error during tx DMA */
 #define HME_SEB_STAT_TXPERR	0x10000000	/* parity error during tx DMA */
-#define HME_SEB_STAT_TXTERR	0x20000000	/* tag error durig tx DMA */
+#define HME_SEB_STAT_TXTERR	0x20000000	/* tag error during tx DMA */
 #define HME_SEB_STAT_SLVERR	0x40000000	/* pio access error */
 #define HME_SEB_STAT_SLVPERR	0x80000000	/* pio access parity error */
 #define HME_SEB_STAT_BITS	"\177\020"				\
