@@ -37,6 +37,7 @@
 __FBSDID("$FreeBSD$");
 
 #include <sys/param.h>
+#include <sys/capability.h>
 #include <sys/file.h>
 #include <sys/kernel.h>
 #include <sys/kthread.h>
@@ -237,7 +238,7 @@ pmclog_get_buffer(struct pmc_owner *po)
 static void
 pmclog_loop(void *arg)
 {
-	int error;
+	int error, last_buffer;
 	struct pmc_owner *po;
 	struct pmclog_buffer *lb;
 	struct proc *p;
@@ -252,6 +253,7 @@ pmclog_loop(void *arg)
 	p = po->po_owner;
 	td = curthread;
 	mycred = td->td_ucred;
+	last_buffer = 0;
 
 	PROC_LOCK(p);
 	ownercred = crhold(p->p_ucred);
@@ -284,22 +286,14 @@ pmclog_loop(void *arg)
 			if ((lb = TAILQ_FIRST(&po->po_logbuffers)) == NULL) {
 				mtx_unlock_spin(&po->po_mtx);
 
-				if (po->po_flags & PMC_PO_SHUTDOWN) {
-					mtx_unlock(&pmc_kthread_mtx);
-					/*
-			 		 * Close the file to get PMCLOG_EOF
-					 * error in pmclog(3).
-					 */
-					fo_close(po->po_file, curthread);
-					mtx_lock(&pmc_kthread_mtx);
-				}
-
 				(void) msleep(po, &pmc_kthread_mtx, PWAIT,
 				    "pmcloop", 0);
 				continue;
 			}
 
 			TAILQ_REMOVE(&po->po_logbuffers, lb, plb_next);
+			if (po->po_flags & PMC_PO_SHUTDOWN)
+				last_buffer = TAILQ_EMPTY(&po->po_logbuffers);
 			mtx_unlock_spin(&po->po_mtx);
 		}
 
@@ -330,7 +324,7 @@ pmclog_loop(void *arg)
 			/* XXX some errors are recoverable */
 			/* send a SIGIO to the owner and exit */
 			PROC_LOCK(p);
-			psignal(p, SIGIO);
+			kern_psignal(p, SIGIO);
 			PROC_UNLOCK(p);
 
 			mtx_lock(&pmc_kthread_mtx);
@@ -342,6 +336,14 @@ pmclog_loop(void *arg)
 			break;
 		}
 
+		if (last_buffer) {
+			/*
+			 * Close the file to get PMCLOG_EOF error
+			 * in pmclog(3).
+			 */
+			fo_close(po->po_file, curthread);
+		}
+
 		mtx_lock(&pmc_kthread_mtx);
 
 		/* put the used buffer back into the global pool */
@@ -588,7 +590,7 @@ pmclog_configure_log(struct pmc_mdep *md, struct pmc_owner *po, int logfd)
 		po->po_file));
 
 	/* get a reference to the file state */
-	error = fget_write(curthread, logfd, &po->po_file);
+	error = fget_write(curthread, logfd, CAP_WRITE, &po->po_file);
 	if (error)
 		goto error;
 
@@ -691,7 +693,6 @@ int
 pmclog_flush(struct pmc_owner *po)
 {
 	int error;
-	struct pmclog_buffer *lb;
 
 	PMCDBG(LOG,FLS,1, "po=%p", po);
 
@@ -714,38 +715,11 @@ pmclog_flush(struct pmc_owner *po)
 	}
 
 	/*
-	 * Schedule the current buffer if any and not empty.
-	 */
-	mtx_lock_spin(&po->po_mtx);
-	lb = po->po_curbuf;
-	if (lb && lb->plb_ptr != lb->plb_base) {
-		pmclog_schedule_io(po);
-	} else
-		error = ENOBUFS;
-	mtx_unlock_spin(&po->po_mtx);
-
- error:
-	mtx_unlock(&pmc_kthread_mtx);
-
-	return (error);
-}
-
-int
-pmclog_close(struct pmc_owner *po)
-{
-
-	PMCDBG(LOG,CLO,1, "po=%p", po);
-
-	mtx_lock(&pmc_kthread_mtx);
-
-	/*
-	 * Schedule the current buffer.
+	 * Schedule the current buffer if any.
 	 */
 	mtx_lock_spin(&po->po_mtx);
 	if (po->po_curbuf)
 		pmclog_schedule_io(po);
-	else
-		wakeup_one(po);
 	mtx_unlock_spin(&po->po_mtx);
 
 	/*
@@ -754,11 +728,13 @@ pmclog_close(struct pmc_owner *po)
 	 */
 	po->po_flags |= PMC_PO_SHUTDOWN;
 
+ error:
 	mtx_unlock(&pmc_kthread_mtx);
 
-	return (0);
+	return (error);
 }
 
+
 void
 pmclog_process_callchain(struct pmc *pm, struct pmc_sample *ps)
 {
