@@ -153,7 +153,6 @@ static void     iir_action(struct cam_sim *sim, union ccb *ccb);
 static void     iir_poll(struct cam_sim *sim);
 static void     iir_shutdown(void *arg, int howto);
 static void     iir_timeout(void *arg);
-static void     iir_watchdog(void *arg);
 
 static void     gdt_eval_mapping(u_int32_t size, int *cyls, int *heads, 
                                  int *secs);
@@ -1484,40 +1483,6 @@ iir_timeout(void *arg)
     GDT_DPRINTF(GDT_D_TIMEOUT, ("iir_timeout(%p)\n", gccb));
 }
 
-static void
-iir_watchdog(void *arg)
-{
-    struct gdt_softc *gdt;
-
-    gdt = (struct gdt_softc *)arg;
-    GDT_DPRINTF(GDT_D_DEBUG, ("iir_watchdog(%p)\n", gdt));
-
-    {
-        int ccbs = 0, ucmds = 0, frees = 0, pends = 0;
-        struct gdt_ccb *p;
-        struct ccb_hdr *h;
-        struct gdt_ucmd *u;
-
-        for (h = TAILQ_FIRST(&gdt->sc_ccb_queue); h != NULL; 
-             h = TAILQ_NEXT(h, sim_links.tqe))
-            ccbs++;
-        for (u = TAILQ_FIRST(&gdt->sc_ucmd_queue); u != NULL; 
-             u = TAILQ_NEXT(u, links))
-            ucmds++;
-        for (p = SLIST_FIRST(&gdt->sc_free_gccb); p != NULL; 
-             p = SLIST_NEXT(p, sle))
-            frees++;
-        for (p = SLIST_FIRST(&gdt->sc_pending_gccb); p != NULL; 
-             p = SLIST_NEXT(p, sle))
-            pends++;
-
-        GDT_DPRINTF(GDT_D_TIMEOUT, ("ccbs %d ucmds %d frees %d pends %d\n",
-               ccbs, ucmds, frees, pends));
-    }
-
-    timeout(iir_watchdog, (caddr_t)gdt, hz * 15);
-}
-
 static void     
 iir_shutdown( void *arg, int howto )
 {
@@ -1874,13 +1839,20 @@ gdt_sync_event(struct gdt_softc *gdt, int service,
         } else {
             /* error */
             if (gccb->gc_service == GDT_CACHESERVICE) {
+                struct scsi_sense_data *sense;
+
                 ccb->ccb_h.status |= CAM_SCSI_STATUS_ERROR | CAM_AUTOSNS_VALID;
                 ccb->ccb_h.status &= ~CAM_SIM_QUEUED;
                 ccb->csio.scsi_status = SCSI_STATUS_CHECK_COND;
                 bzero(&ccb->csio.sense_data, ccb->csio.sense_len);
-                ccb->csio.sense_data.error_code =
-                    SSD_CURRENT_ERROR | SSD_ERRCODE_VALID;
-                ccb->csio.sense_data.flags = SSD_KEY_NOT_READY;
+                sense = &ccb->csio.sense_data;
+                scsi_set_sense_data(sense,
+                                    /*sense_format*/ SSD_TYPE_NONE,
+                                    /*current_error*/ 1,
+                                    /*sense_key*/ SSD_KEY_NOT_READY,
+                                    /*asc*/ 0x4,
+                                    /*ascq*/ 0x01,
+                                    SSD_ELEM_NONE);
 
                 gdt->sc_dvr.size = sizeof(gdt->sc_dvr.eu.sync);
                 gdt->sc_dvr.eu.sync.ionode  = gdt->sc_hanum;
