@@ -34,6 +34,7 @@ __FBSDID("$FreeBSD$");
 #include "opt_iscsi_initiator.h"
 
 #include <sys/param.h>
+#include <sys/capability.h>
 #include <sys/kernel.h>
 #include <sys/module.h>
 #include <sys/conf.h>
@@ -229,12 +230,9 @@ iscsi_ioctl(struct cdev *dev, u_long cmd, caddr_t arg, int mode, struct thread *
 	  error = sp->soc == NULL? ENOTCONN: i_fullfeature(dev, 1);
 	  if(error == 0) {
 	       sp->proc = td->td_proc;
-	       SYSCTL_ADD_UINT(&sp->clist,
-			       SYSCTL_CHILDREN(sp->oid),
-			       OID_AUTO,
-			       "pid",
-			       CTLFLAG_RD,
-			       &sp->proc->p_pid, sizeof(pid_t), "control process id");
+	       SYSCTL_ADD_INT(&sp->clist, SYSCTL_CHILDREN(sp->oid),
+		   OID_AUTO, "pid", CTLFLAG_RD,
+		   &sp->proc->p_pid, sizeof(pid_t), "control process id");
 	  }
 	  break;
 
@@ -390,11 +388,11 @@ i_setsoc(isc_session_t *sp, int fd, struct thread *td)
      if(sp->soc != NULL)
 	  isc_stop_receiver(sp);
 
-     error = fget(td, fd, &sp->fp);
+     error = fget(td, fd, CAP_SOCK_ALL, &sp->fp);
      if(error)
 	  return error;
 
-     if((error = fgetsock(td, fd, &sp->soc, 0)) == 0) {
+     if((error = fgetsock(td, fd, CAP_SOCK_ALL, &sp->soc, 0)) == 0) {
 	  sp->td = td;
 	  isc_start_receiver(sp);
      }
