@@ -224,7 +224,7 @@ kbd_register(keyboard_t *kbd)
 				strcpy(ki.kb_name, kbd->kb_name);
 				ki.kb_unit = kbd->kb_unit;
 
-				kbdd_ioctl(mux, KBADDKBD, (caddr_t) &ki);
+				(void)kbdd_ioctl(mux, KBADDKBD, (caddr_t) &ki);
 			}
 
 			return (index);
@@ -241,7 +241,7 @@ kbd_register(keyboard_t *kbd)
 				strcpy(ki.kb_name, kbd->kb_name);
 				ki.kb_unit = kbd->kb_unit;
 
-				kbdd_ioctl(mux, KBADDKBD, (caddr_t) &ki);
+				(void)kbdd_ioctl(mux, KBADDKBD, (caddr_t) &ki);
 			}
 
 			return (index);
@@ -728,7 +728,7 @@ genkbd_event(keyboard_t *kbd, int event, void *arg)
 	size_t len;
 	u_char *cp;
 	int mode;
-	int c;
+	u_int c;
 
 	/* assert(KBD_IS_VALID(kbd)) */
 	sc = (genkbd_softc_t *)arg;
@@ -837,13 +837,13 @@ static int fkey_change_ok(fkeytab_t *, fkeyarg_t *, struct thread *);
 int
 genkbd_commonioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 {
+	keymap_t *mapp;
+	okeymap_t *omapp;
 	keyarg_t *keyp;
 	fkeyarg_t *fkeyp;
 	int s;
-	int i;
-#ifndef KBD_DISABLE_KEYMAP_LOAD
+	int i, j;
 	int error;
-#endif
 
 	s = spltty();
 	switch (cmd) {
@@ -869,18 +869,54 @@ genkbd_commonioctl(keyboard_t *kbd, u_long cmd, caddr_t arg)
 		break;
 
 	case GIO_KEYMAP:	/* get keyboard translation table */
-		bcopy(kbd->kb_keymap, arg, sizeof(*kbd->kb_keymap));
-		break;
+		error = copyout(kbd->kb_keymap, *(void **)arg,
+		    sizeof(keymap_t));
+		splx(s);
+		return (error);
+	case OGIO_KEYMAP:	/* get keyboard translation table (compat) */
+		mapp = kbd->kb_keymap;
+		omapp = (okeymap_t *)arg;
+		omapp->n_keys = mapp->n_keys;
+		for (i = 0; i < NUM_KEYS; i++) {
+			for (j = 0; j < NUM_STATES; j++)
+				omapp->key[i].map[j] =
+				    mapp->key[i].map[j];
+			omapp->key[i].spcl = mapp->key[i].spcl;
+			omapp->key[i].flgs = mapp->key[i].flgs;
+		}
+		return (0);
 	case PIO_KEYMAP:	/* set keyboard translation table */
+	case OPIO_KEYMAP:	/* set keyboard translation table (compat) */
 #ifndef KBD_DISABLE_KEYMAP_LOAD
-		error = keymap_change_ok(kbd->kb_keymap, (keymap_t *)arg,
-		    curthread);
+		mapp = malloc(sizeof *mapp, M_TEMP, M_NOWAIT);
+		if (cmd == OPIO_KEYMAP) {
+			omapp = (okeymap_t *)arg;
+			mapp->n_keys = omapp->n_keys;
+			for (i = 0; i < NUM_KEYS; i++) {
+				for (j = 0; j < NUM_STATES; j++)
+					mapp->key[i].map[j] =
+					    omapp->key[i].map[j];
+				mapp->key[i].spcl = omapp->key[i].spcl;
+				mapp->key[i].flgs = omapp->key[i].flgs;
+			}
+		} else {
+			error = copyin(*(void **)arg, mapp, sizeof *mapp);
+			if (error != 0) {
+				splx(s);
+				free(mapp, M_TEMP);
+				return (error);
+			}
+		}
+
+		error = keymap_change_ok(kbd->kb_keymap, mapp, curthread);
 		if (error != 0) {
 			splx(s);
+			free(mapp, M_TEMP);
 			return (error);
 		}
 		bzero(kbd->kb_accentmap, sizeof(*kbd->kb_accentmap));
-		bcopy(arg, kbd->kb_keymap, sizeof(*kbd->kb_keymap));
+		bcopy(mapp, kbd->kb_keymap, sizeof(*kbd->kb_keymap));
+		free(mapp, M_TEMP);
 		break;
 #else
 		splx(s);
@@ -1136,7 +1172,7 @@ genkbd_diag(keyboard_t *kbd, int level)
 		(s) |= l ## DOWN;				\
 		(s) ^= l ## ED;					\
 		i = (s) & LOCK_MASK;				\
-		kbdd_ioctl((k), KDSETLED, (caddr_t)&i);		\
+		(void)kbdd_ioctl((k), KDSETLED, (caddr_t)&i);	\
 	}
 
 static u_int
@@ -1296,7 +1332,7 @@ genkbd_keyaction(keyboard_t *kbd, int keycode, int up, int *shiftstate,
 #else
 			state &= ~CLKED;
 			i = state & LOCK_MASK;
-			kbdd_ioctl(kbd, KDSETLED, (caddr_t)&i);
+			(void)kbdd_ioctl(kbd, KDSETLED, (caddr_t)&i);
 #endif
 			break;
 		case SLK:
@@ -1332,7 +1368,7 @@ genkbd_keyaction(keyboard_t *kbd, int keycode, int up, int *shiftstate,
 #else
 				state |= CLKED;
 				i = state & LOCK_MASK;
-				kbdd_ioctl(kbd, KDSETLED, (caddr_t)&i);
+				(void)kbdd_ioctl(kbd, KDSETLED, (caddr_t)&i);
 #endif
 				break;
 			case SLK:
