@@ -110,7 +110,7 @@ __FBSDID("$FreeBSD$");
 /*
  * Various supported device vendors/types and their names.
  */
-static const struct lge_type const lge_devs[] = {
+static struct lge_type lge_devs[] = {
 	{ LGE_VENDORID, LGE_DEVICEID, "Level 1 Gigabit Ethernet" },
 	{ 0, 0, NULL }
 };
@@ -170,12 +170,16 @@ static device_method_t lge_methods[] = {
 	DEVMETHOD(device_detach,	lge_detach),
 	DEVMETHOD(device_shutdown,	lge_shutdown),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	lge_miibus_readreg),
 	DEVMETHOD(miibus_writereg,	lge_miibus_writereg),
 	DEVMETHOD(miibus_statchg,	lge_miibus_statchg),
 
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 static driver_t lge_driver = {
@@ -438,7 +442,7 @@ static int
 lge_probe(dev)
 	device_t		dev;
 {
-	const struct lge_type	*t;
+	struct lge_type		*t;
 
 	t = lge_devs;
 
@@ -1406,7 +1410,7 @@ lge_ifmedia_upd_locked(ifp)
 	mii = device_get_softc(sc->lge_miibus);
 	sc->lge_link = 0;
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 	mii_mediachg(mii);
 }
 
@@ -1426,9 +1430,9 @@ lge_ifmedia_sts(ifp, ifmr)
 	LGE_LOCK(sc);
 	mii = device_get_softc(sc->lge_miibus);
 	mii_pollstat(mii);
+	LGE_UNLOCK(sc);
 	ifmr->ifm_active = mii->mii_media_active;
 	ifmr->ifm_status = mii->mii_media_status;
-	LGE_UNLOCK(sc);
 
 	return;
 }
