@@ -48,6 +48,7 @@ __FBSDID("$FreeBSD$");
 
 #include <net/if.h>
 #include <net/if_media.h>
+#include <net/route.h>
 
 #include <dev/mii/mii.h>
 #include <dev/mii/miivar.h>
@@ -56,20 +57,19 @@ MODULE_VERSION(miibus, 1);
 
 #include "miibus_if.h"
 
-static device_attach_t miibus_attach;
-static bus_child_location_str_t miibus_child_location_str;
-static bus_child_pnpinfo_str_t miibus_child_pnpinfo_str;
-static device_detach_t miibus_detach;
-static bus_hinted_child_t miibus_hinted_child;
-static bus_print_child_t miibus_print_child;
-static device_probe_t miibus_probe;
-static bus_read_ivar_t miibus_read_ivar;
-static miibus_readreg_t miibus_readreg;
-static miibus_statchg_t miibus_statchg;
-static miibus_writereg_t miibus_writereg;
-static miibus_linkchg_t miibus_linkchg;
-static miibus_mediainit_t miibus_mediainit;
-
+static int miibus_print_child(device_t dev, device_t child);
+static int miibus_read_ivar(device_t dev, device_t child, int which,
+    uintptr_t *result);
+static int miibus_child_location_str(device_t bus, device_t child, char *buf,
+    size_t buflen);
+static int miibus_child_pnpinfo_str(device_t bus, device_t child, char *buf,
+    size_t buflen);
+static int miibus_readreg(device_t, int, int);
+static int miibus_writereg(device_t, int, int, int);
+static void miibus_statchg(device_t);
+static void miibus_linkchg(device_t);
+static void miibus_mediainit(device_t);
+static unsigned char mii_bitreverse(unsigned char x);
 
 static device_method_t miibus_methods[] = {
 	/* device interface */
@@ -81,9 +81,9 @@ static device_method_t miibus_methods[] = {
 	/* bus interface */
 	DEVMETHOD(bus_print_child,	miibus_print_child),
 	DEVMETHOD(bus_read_ivar,	miibus_read_ivar),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
 	DEVMETHOD(bus_child_pnpinfo_str, miibus_child_pnpinfo_str),
 	DEVMETHOD(bus_child_location_str, miibus_child_location_str),
-	DEVMETHOD(bus_hinted_child,	miibus_hinted_child),
 
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	miibus_readreg),
@@ -92,7 +92,7 @@ static device_method_t miibus_methods[] = {
 	DEVMETHOD(miibus_linkchg,	miibus_linkchg),
 	DEVMETHOD(miibus_mediainit,	miibus_mediainit),
 
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 devclass_t miibus_devclass;
@@ -107,11 +107,10 @@ struct miibus_ivars {
 	struct ifnet	*ifp;
 	ifm_change_cb_t	ifmedia_upd;
 	ifm_stat_cb_t	ifmedia_sts;
-	u_int		mii_flags;
-	u_int		mii_offset;
+	int		mii_flags;
 };
 
-static int
+int
 miibus_probe(device_t dev)
 {
 
@@ -120,7 +119,7 @@ miibus_probe(device_t dev)
 	return (BUS_PROBE_SPECIFIC);
 }
 
-static int
+int
 miibus_attach(device_t dev)
 {
 	struct miibus_ivars	*ivars;
@@ -130,6 +129,7 @@ miibus_attach(device_t dev)
 	int			i, nchildren;
 
 	mii = device_get_softc(dev);
+	nchildren = 0;
 	if (device_get_children(dev, &children, &nchildren) == 0) {
 		for (i = 0; i < nchildren; i++) {
 			ma = device_get_ivars(children[i]);
@@ -152,7 +152,7 @@ miibus_attach(device_t dev)
 	return (bus_generic_attach(dev));
 }
 
-static int
+int
 miibus_detach(device_t dev)
 {
 	struct mii_data		*mii;
@@ -201,7 +201,7 @@ miibus_read_ivar(device_t dev, device_t child __unused, int which,
 }
 
 static int
-miibus_child_pnpinfo_str(device_t dev __unused, device_t child, char *buf,
+miibus_child_pnpinfo_str(device_t bus __unused, device_t child, char *buf,
     size_t buflen)
 {
 	struct mii_attach_args *ma;
@@ -214,7 +214,7 @@ miibus_child_pnpinfo_str(device_t dev __unused, device_t child, char *buf,
 }
 
 static int
-miibus_child_location_str(device_t dev __unused, device_t child, char *buf,
+miibus_child_location_str(device_t bus __unused, device_t child, char *buf,
     size_t buflen)
 {
 	struct mii_attach_args *ma;
@@ -224,60 +224,6 @@ miibus_child_location_str(device_t dev __unused, device_t child, char *buf,
 	return (0);
 }
 
-static void
-miibus_hinted_child(device_t dev, const char *name, int unit)
-{
-	struct miibus_ivars *ivars;
-	struct mii_attach_args *args, *ma;
-	device_t *children, phy;
-	int i, nchildren;
-	u_int val;
-
-	if (resource_int_value(name, unit, "phyno", &val) != 0)
-		return;
-	if (device_get_children(dev, &children, &nchildren) != 0)
-		return;
-	ma = NULL;
-	for (i = 0; i < nchildren; i++) {
-		args = device_get_ivars(children[i]);
-		if (args->mii_phyno == val) {
-			ma = args;
-			break;
-		}
-	}
-	free(children, M_TEMP);
-
-	/*
-	 * Don't add a PHY that was automatically identified by having media
-	 * in its BMSR twice, only allow to alter its attach arguments.
-	 */
-	if (ma == NULL) {
-		ma = malloc(sizeof(struct mii_attach_args), M_DEVBUF,
-		    M_NOWAIT);
-		if (ma == NULL)
-			return;
-		phy = device_add_child(dev, name, unit);
-		if (phy == NULL) {
-			free(ma, M_DEVBUF);
-			return;
-		}
-		ivars = device_get_ivars(dev);
-		ma->mii_phyno = val;
-		ma->mii_offset = ivars->mii_offset++;
-		ma->mii_id1 = 0;
-		ma->mii_id2 = 0;
-		ma->mii_capmask = BMSR_DEFCAPMASK;
-		device_set_ivars(phy, ma);
-	}
-
-	if (resource_int_value(name, unit, "id1", &val) == 0)
-		ma->mii_id1 = val;
-	if (resource_int_value(name, unit, "id2", &val) == 0)
-		ma->mii_id2 = val;
-	if (resource_int_value(name, unit, "capmask", &val) == 0)
-		ma->mii_capmask = val;
-}
-
 static int
 miibus_readreg(device_t dev, int phy, int reg)
 {
@@ -301,19 +247,12 @@ miibus_statchg(device_t dev)
 {
 	device_t		parent;
 	struct mii_data		*mii;
-	struct ifnet		*ifp;
 
 	parent = device_get_parent(dev);
 	MIIBUS_STATCHG(parent);
 
 	mii = device_get_softc(dev);
-
-	/*
-	 * Note that each NIC's softc must start with an ifnet pointer.
-	 * XXX: EVIL HACK!
-	 */
-	ifp = *(struct ifnet **)device_get_softc(parent);
-	ifp->if_baudrate = ifmedia_baudrate(mii->mii_media_active);
+	mii->mii_ifp->if_baudrate = ifmedia_baudrate(mii->mii_media_active);
 }
 
 static void
@@ -335,11 +274,7 @@ miibus_linkchg(device_t dev)
 			link_state = LINK_STATE_DOWN;
 	} else
 		link_state = LINK_STATE_UNKNOWN;
-	/*
-	 * Note that each NIC's softc must start with an ifnet pointer.
-	 * XXX: EVIL HACK!
-	 */
-	if_link_state_change(*(struct ifnet**)device_get_softc(parent), link_state);
+	if_link_state_change(mii->mii_ifp, link_state);
 }
 
 static void
@@ -372,10 +307,9 @@ mii_attach(device_t dev, device_t *miibus, struct ifnet *ifp,
     int phyloc, int offloc, int flags)
 {
 	struct miibus_ivars *ivars;
-	struct mii_attach_args *args, ma;
+	struct mii_attach_args ma, *args;
 	device_t *children, phy;
-	int bmsr, first, i, nchildren, phymax, phymin, rv;
-	uint32_t phymask;
+	int bmsr, first, i, nchildren, offset, phymax, phymin, rv;
 
 	if (phyloc != MII_PHY_ANY && offloc != MII_OFFSET_ANY) {
 		printf("%s: phyloc and offloc specified\n", __func__);
@@ -432,30 +366,27 @@ mii_attach(device_t dev, device_t *miibus, struct ifnet *ifp,
 
 	ma.mii_capmask = capmask;
 
-	if (resource_int_value(device_get_name(*miibus),
-	    device_get_unit(*miibus), "phymask", &phymask) != 0)
-		phymask = 0xffffffff;
-
-	if (device_get_children(*miibus, &children, &nchildren) != 0) {
-		children = NULL;
-		nchildren = 0;
-	}
-	ivars->mii_offset = 0;
+	phy = NULL;
+	offset = 0;
 	for (ma.mii_phyno = phymin; ma.mii_phyno <= phymax; ma.mii_phyno++) {
 		/*
 		 * Make sure we haven't already configured a PHY at this
 		 * address.  This allows mii_attach() to be called
 		 * multiple times.
 		 */
-		for (i = 0; i < nchildren; i++) {
-			args = device_get_ivars(children[i]);
-			if (args->mii_phyno == ma.mii_phyno) {
-				/*
-				 * Yes, there is already something
-				 * configured at this address.
-				 */
-				goto skip;
+		if (device_get_children(*miibus, &children, &nchildren) == 0) {
+			for (i = 0; i < nchildren; i++) {
+				args = device_get_ivars(children[i]);
+				if (args->mii_phyno == ma.mii_phyno) {
+					/*
+					 * Yes, there is already something
+					 * configured at this address.
+					 */
+					free(children, M_TEMP);
+					goto skip;
+				}
 			}
+			free(children, M_TEMP);
 		}
 
 		/*
@@ -474,23 +405,18 @@ mii_attach(device_t dev, device_t *miibus, struct ifnet *ifp,
 		 * There is a PHY at this address.  If we were given an
 		 * `offset' locator, skip this PHY if it doesn't match.
 		 */
-		if (offloc != MII_OFFSET_ANY && offloc != ivars->mii_offset)
+		if (offloc != MII_OFFSET_ANY && offloc != offset)
 			goto skip;
 
 		/*
-		 * Skip this PHY if it's not included in the phymask hint.
-		 */
-		if ((phymask & (1 << ma.mii_phyno)) == 0)
-			goto skip;
-
-		/*
-		 * Extract the IDs.  Braindead PHYs will be handled by
+		 * Extract the IDs. Braindead PHYs will be handled by
 		 * the `ukphy' driver, as we have no ID information to
 		 * match on.
 		 */
 		ma.mii_id1 = MIIBUS_READREG(dev, ma.mii_phyno, MII_PHYIDR1);
 		ma.mii_id2 = MIIBUS_READREG(dev, ma.mii_phyno, MII_PHYIDR2);
 
+		ma.mii_offset = offset;
 		args = malloc(sizeof(struct mii_attach_args), M_DEVBUF,
 		    M_NOWAIT);
 		if (args == NULL)
@@ -503,20 +429,11 @@ mii_attach(device_t dev, device_t *miibus, struct ifnet *ifp,
 		}
 		device_set_ivars(phy, args);
  skip:
-		ivars->mii_offset++;
+		offset++;
 	}
-	free(children, M_TEMP);
 
 	if (first != 0) {
-		rv = device_set_driver(*miibus, &miibus_driver);
-		if (rv != 0)
-			goto fail;
-		bus_enumerate_hinted_children(*miibus);
-		rv = device_get_children(*miibus, &children, &nchildren);
-		if (rv != 0)
-			goto fail;
-		free(children, M_TEMP);
-		if (nchildren == 0) {
+		if (phy == NULL) {
 			rv = ENXIO;
 			goto fail;
 		}
@@ -542,21 +459,6 @@ mii_attach(device_t dev, device_t *miibus, struct ifnet *ifp,
 	return (rv);
 }
 
-int
-mii_phy_probe(device_t dev, device_t *child, ifm_change_cb_t ifmedia_upd,
-    ifm_stat_cb_t ifmedia_sts)
-{
-	struct ifnet *ifp;
-
-	/*
-	 * Note that each NIC's softc must start with an ifnet pointer.
-	 * XXX: EVIL HACK!
-	 */
-	ifp = *(struct ifnet **)device_get_softc(dev);
-	return (mii_attach(dev, child, ifp, ifmedia_upd, ifmedia_sts,
-	    BMSR_DEFCAPMASK, MII_PHY_ANY, MII_OFFSET_ANY, 0));
-}
-
 /*
  * Media changed; notify all PHYs.
  */
@@ -586,7 +488,7 @@ mii_mediachg(struct mii_data *mii)
 			    BMCR_ISO);
 			continue;
 		}
-		rv = (*child->mii_service)(child, mii, MII_MEDIACHG);
+		rv = PHY_SERVICE(child, mii, MII_MEDIACHG);
 		if (rv)
 			return (rv);
 	}
@@ -609,7 +511,7 @@ mii_tick(struct mii_data *mii)
 		 */
 		if (IFM_INST(ife->ifm_media) != child->mii_inst)
 			continue;
-		(void)(*child->mii_service)(child, mii, MII_TICK);
+		(void)PHY_SERVICE(child, mii, MII_TICK);
 	}
 }
 
@@ -631,7 +533,7 @@ mii_pollstat(struct mii_data *mii)
 		 */
 		if (IFM_INST(ife->ifm_media) != child->mii_inst)
 			continue;
-		(void)(*child->mii_service)(child, mii, MII_POLLSTAT);
+		(void)PHY_SERVICE(child, mii, MII_POLLSTAT);
 	}
 }
 
@@ -646,3 +548,25 @@ mii_down(struct mii_data *mii)
 	LIST_FOREACH(child, &mii->mii_phys, mii_list)
 		mii_phy_down(child);
 }
+
+static unsigned char
+mii_bitreverse(unsigned char x)
+{
+	unsigned const char const nibbletab[16] = {
+		0, 8, 4, 12, 2, 10, 6, 14, 1, 9, 5, 13, 3, 11, 7, 15
+	};
+
+	return ((nibbletab[x & 15] << 4) | nibbletab[x >> 4]);
+}
+
+u_int
+mii_oui(u_int id1, u_int id2)
+{
+	u_int h;
+
+	h = (id1 << 6) | (id2 >> 10);
+
+	return ((mii_bitreverse(h >> 16) << 16) |
+	    (mii_bitreverse((h >> 8) & 0xff) << 8) |
+	    mii_bitreverse(h & 0xff));
+}
