@@ -223,12 +223,16 @@ static device_method_t nge_methods[] = {
 	DEVMETHOD(device_suspend,	nge_suspend),
 	DEVMETHOD(device_resume,	nge_resume),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	nge_miibus_readreg),
 	DEVMETHOD(miibus_writereg,	nge_miibus_writereg),
 	DEVMETHOD(miibus_statchg,	nge_miibus_statchg),
 
-	DEVMETHOD_END
+	{ NULL, NULL }
 };
 
 static driver_t nge_driver = {
@@ -921,7 +925,7 @@ nge_attach(device_t dev)
 	 * supply(3VAUX) to drive PME such that checking PCI power
 	 * management capability is necessary.
 	 */
-	if (pci_find_extcap(sc->nge_dev, PCIY_PMG, &i) == 0)
+	if (pci_find_cap(sc->nge_dev, PCIY_PMG, &i) == 0)
 		ifp->if_capabilities |= IFCAP_WOL;
 	ifp->if_capenable = ifp->if_capabilities;
 
@@ -2257,7 +2261,7 @@ nge_mediachange(struct ifnet *ifp)
 	NGE_LOCK(sc);
 	mii = device_get_softc(sc->nge_miibus);
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 	error = mii_mediachg(mii);
 	NGE_UNLOCK(sc);
 
@@ -2277,9 +2281,9 @@ nge_mediastatus(struct ifnet *ifp, struct ifmediareq *ifmr)
 	NGE_LOCK(sc);
 	mii = device_get_softc(sc->nge_miibus);
 	mii_pollstat(mii);
+	NGE_UNLOCK(sc);
 	ifmr->ifm_active = mii->mii_media_active;
 	ifmr->ifm_status = mii->mii_media_status;
-	NGE_UNLOCK(sc);
 }
 
 static int
@@ -2547,7 +2551,7 @@ nge_wol(struct nge_softc *sc)
 
 	NGE_LOCK_ASSERT(sc);
 
-	if (pci_find_extcap(sc->nge_dev, PCIY_PMG, &pmc) != 0)
+	if (pci_find_cap(sc->nge_dev, PCIY_PMG, &pmc) != 0)
 		return;
 
 	ifp = sc->nge_ifp;
@@ -2634,7 +2638,7 @@ nge_resume(device_t dev)
 
 	NGE_LOCK(sc);
 	ifp = sc->nge_ifp;
-	if (pci_find_extcap(sc->nge_dev, PCIY_PMG, &pmc) == 0) {
+	if (pci_find_cap(sc->nge_dev, PCIY_PMG, &pmc) == 0) {
 		/* Disable PME and clear PME status. */
 		pmstat = pci_read_config(sc->nge_dev,
 		    pmc + PCIR_POWER_STATUS, 2);
