@@ -657,6 +657,12 @@ cbb_pci_shutdown(device_t brdev)
 {
 	struct cbb_softc *sc = (struct cbb_softc *)device_get_softc(brdev);
 
+	/*
+	 * We're about to pull the rug out from the card, so mark it as
+	 * gone to prevent harm.
+         */
+        sc->cardok = 0;
+
 	/*
 	 * Place the cards in reset, turn off the interrupts and power
 	 * down the socket.
@@ -816,6 +822,7 @@ static device_method_t cbb_methods[] = {
 	DEVMETHOD(device_resume,		cbb_resume),
 
 	/* bus methods */
+	DEVMETHOD(bus_print_child,		bus_generic_print_child),
 	DEVMETHOD(bus_read_ivar,		cbb_read_ivar),
 	DEVMETHOD(bus_write_ivar,		cbb_write_ivar),
 	DEVMETHOD(bus_alloc_resource,		cbb_alloc_resource),
@@ -842,7 +849,7 @@ static device_method_t cbb_methods[] = {
 	DEVMETHOD(pcib_write_config,		cbb_write_config),
 	DEVMETHOD(pcib_route_interrupt,		cbb_route_interrupt),
 
-	DEVMETHOD_END
+	{0,0}
 };
 
 static driver_t cbb_driver = {
