@@ -173,12 +173,16 @@ static device_method_t pcn_methods[] = {
 	DEVMETHOD(device_detach,	pcn_detach),
 	DEVMETHOD(device_shutdown,	pcn_shutdown),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	pcn_miibus_readreg),
 	DEVMETHOD(miibus_writereg,	pcn_miibus_writereg),
 	DEVMETHOD(miibus_statchg,	pcn_miibus_statchg),
 
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 static driver_t pcn_driver = {
@@ -630,14 +634,13 @@ pcn_attach(dev)
 	ifp->if_snd.ifq_maxlen = PCN_TX_LIST_CNT - 1;
 
 	/*
-	 * Do MII setup.  Note that loopback support isn't implemented.
+	 * Do MII setup.
 	 * See the comment in pcn_miibus_readreg() for why we can't
 	 * universally pass MIIF_NOISOLATE here.
 	 */
 	sc->pcn_extphyaddr = -1;
 	error = mii_attach(dev, &sc->pcn_miibus, ifp, pcn_ifmedia_upd,
-	   pcn_ifmedia_sts, BMSR_DEFCAPMASK, MII_PHY_ANY, MII_OFFSET_ANY,
-	   MIIF_NOLOOP);
+	   pcn_ifmedia_sts, BMSR_DEFCAPMASK, MII_PHY_ANY, MII_OFFSET_ANY, 0);
 	if (error != 0) {
 		device_printf(dev, "attaching PHYs failed\n");
 		goto fail;
