@@ -51,6 +51,7 @@ static puc_config_f puc_config_amc;
 static puc_config_f puc_config_diva;
 static puc_config_f puc_config_exar;
 static puc_config_f puc_config_icbook;
+static puc_config_f puc_config_moxa;
 static puc_config_f puc_config_oxford_pcie;
 static puc_config_f puc_config_quatech;
 static puc_config_f puc_config_syba;
@@ -518,12 +519,25 @@ const struct puc_cfg puc_pci_devices[] = {
 	    PUC_PORT_4S, 0x18, 0, 8,
 	},
 
+	{   0x1393, 0x1042, 0xffff, 0,
+	    "Moxa Technologies, Smartio CP-104JU/PCI",
+	    DEFAULT_RCLK * 8,
+	    PUC_PORT_4S, 0x18, 0, 8,
+	},
+
 	{   0x1393, 0x1043, 0xffff, 0,
 	    "Moxa Technologies, Smartio CP-104EL/PCIe",
 	    DEFAULT_RCLK * 8,
 	    PUC_PORT_4S, 0x18, 0, 8,
 	},
 
+	{   0x1393, 0x1045, 0xffff, 0,
+	    "Moxa Technologies, Smartio CP-104EL-A/PCIe",
+	    DEFAULT_RCLK * 8,
+	    PUC_PORT_4S, 0x14, 0, -1,
+		.config_function = puc_config_moxa
+	},
+
 	{   0x1393, 0x1120, 0xffff, 0,
 	    "Moxa Technologies, CP-112UL",
 	    DEFAULT_RCLK * 8,
@@ -850,18 +864,6 @@ const struct puc_cfg puc_pci_devices[] = {
 	    PUC_PORT_2S, 0x10, 4, 0,
 	},
 
-	{   0x14d2, 0xa007, 0xffff, 0,
-	    "Titan VScom PCIex-800H",
-	    DEFAULT_RCLK * 8,
-	    PUC_PORT_4S, 0x10, 0, 8,
-	},
-
-	{   0x14d2, 0xa008, 0xffff, 0,
-	    "Titan VScom PCIex-800H",
-	    DEFAULT_RCLK * 8,
-	    PUC_PORT_4S, 0x10, 0, 8,
-	},
-
 	{   0x14db, 0x2130, 0xffff, 0,
 	    "Avlab Technology, PCI IO 2S",
 	    DEFAULT_RCLK,
@@ -1097,6 +1099,19 @@ puc_config_icbook(struct puc_softc *sc, enum puc_cfg_cmd cmd, int port,
 	return (ENXIO);
 }
 
+static int
+puc_config_moxa(struct puc_softc *sc, enum puc_cfg_cmd cmd, int port,
+    intptr_t *res)
+{
+	const struct puc_cfg *cfg = sc->sc_cfg;
+	
+	if (cmd == PUC_CFG_GET_OFS && cfg->device == 0x1045) {
+		*res = ((port == 3) ? 7 : port) * 0x200;
+		return 0;
+	}
+	return (ENXIO);
+}
+
 static int
 puc_config_quatech(struct puc_softc *sc, enum puc_cfg_cmd cmd, int port,
     intptr_t *res)
