@@ -212,12 +212,16 @@ static device_method_t sf_methods[] = {
 	DEVMETHOD(device_suspend,	sf_suspend),
 	DEVMETHOD(device_resume,	sf_resume),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	sf_miibus_readreg),
 	DEVMETHOD(miibus_writereg,	sf_miibus_writereg),
 	DEVMETHOD(miibus_statchg,	sf_miibus_statchg),
 
-	DEVMETHOD_END
+	{ NULL, NULL }
 };
 
 static driver_t sf_driver = {
@@ -532,7 +536,7 @@ sf_ifmedia_upd(struct ifnet *ifp)
 
 	mii = device_get_softc(sc->sf_miibus);
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 	error = mii_mediachg(mii);
 	SF_UNLOCK(sc);
 
