@@ -337,8 +337,8 @@
 struct sis_desc {
 	/* SiS hardware descriptor section */
 	uint32_t		sis_next;
-	uint32_t		sis_cmdsts;
-	uint32_t		sis_ptr;
+	volatile uint32_t	sis_cmdsts;
+	volatile uint32_t	sis_ptr;
 };
 
 #define SIS_CMDSTS_BUFLEN	0x00000FFF
@@ -433,26 +433,9 @@ struct sis_desc {
 struct sis_type {
 	uint16_t		sis_vid;
 	uint16_t		sis_did;
-	char			*sis_name;
+	const char		*sis_name;
 };
 
-struct sis_mii_frame {
-	uint8_t			mii_stdelim;
-	uint8_t			mii_opcode;
-	uint8_t			mii_phyaddr;
-	uint8_t			mii_regaddr;
-	uint8_t			mii_turnaround;
-	uint16_t		mii_data;
-};
-
-/*
- * MII constants
- */
-#define	SIS_MII_STARTDELIM	0x01
-#define	SIS_MII_READOP		0x02
-#define	SIS_MII_WRITEOP		0x01
-#define	SIS_MII_TURNAROUND	0x02
-
 #define SIS_TYPE_900	1
 #define SIS_TYPE_7016	2
 #define SIS_TYPE_83815	3
@@ -497,7 +480,7 @@ struct sis_softc {
 	int			sis_tx_prod;
 	int			sis_tx_cons;
 	int			sis_tx_cnt;
-	int			sis_rx_cons;;
+	int			sis_rx_cons;
 	bus_addr_t		sis_rx_paddr;
 	bus_addr_t		sis_tx_paddr;
 	struct callout		sis_stat_ch;
