@@ -50,7 +50,7 @@
 #include <dev/pci/pcireg.h>
 #include <dev/pci/pcivar.h>
 
-#include <dev/sound/pci/cs461x_dsp.h>
+#include <gnu/dev/sound/pci/csaimg.h>
 
 SND_DECLARE_FILE("$FreeBSD$");
 
@@ -95,7 +95,6 @@ static int csa_teardown_intr(device_t bus, device_t child,
 static driver_intr_t csa_intr;
 static int csa_initialize(sc_p scp);
 static int csa_downloadimage(csa_res *resp);
-static int csa_transferimage(csa_res *resp, u_int32_t *src, u_long dest, u_long len);
 
 static devclass_t csa_devclass;
 
@@ -134,7 +133,7 @@ clkrun_hack(int run)
 			if (pci_get_vendor(*childp) == 0x8086 && pci_get_device(*childp) == 0x7113) {
 				port = (pci_read_config(*childp, 0x41, 1) << 8) + 0x10;
 				/* XXX */
-				btag = I386_BUS_SPACE_IO;
+				btag = X86_BUS_SPACE_IO;
 
 				control = bus_space_read_2(btag, 0x0, port);
 				control &= ~0x2000;
@@ -861,46 +860,27 @@ csa_resetdsp(csa_res *resp)
 static int
 csa_downloadimage(csa_res *resp)
 {
-	int ret;
-	u_long ul, offset;
+	int i;
+	u_int32_t tmp, src, dst, count, data;
 
-	for (ul = 0, offset = 0 ; ul < INKY_MEMORY_COUNT ; ul++) {
-	        /*
-	         * DMA this block from host memory to the appropriate
-	         * memory on the CSDevice.
-	         */
-		ret = csa_transferimage(resp,
-		    cs461x_firmware.BA1Array + offset,
-		    cs461x_firmware.MemoryStat[ul].ulDestAddr,
-		    cs461x_firmware.MemoryStat[ul].ulSourceSize);
-		if (ret)
-			return (ret);
-		offset += cs461x_firmware.MemoryStat[ul].ulSourceSize >> 2;
+	for (i = 0; i < CLEAR__COUNT; i++) {
+		dst = ClrStat[i].BA1__DestByteOffset;
+		count = ClrStat[i].BA1__SourceSize;
+		for (tmp = 0; tmp < count; tmp += 4)
+			csa_writemem(resp, dst + tmp, 0x00000000);
 	}
-	return (0);
-}
 
-static int
-csa_transferimage(csa_res *resp, u_int32_t *src, u_long dest, u_long len)
-{
-	u_long ul;
-	
-	/*
-	 * We do not allow DMAs from host memory to host memory (although the DMA
-	 * can do it) and we do not allow DMAs which are not a multiple of 4 bytes
-	 * in size (because that DMA can not do that).  Return an error if either
-	 * of these conditions exist.
-	 */
-	if ((len & 0x3) != 0)
-		return (EINVAL);
+	for (i = 0; i < FILL__COUNT; i++) {
+		src = 0;
+		dst = FillStat[i].Offset;
+		count = FillStat[i].Size;
+		for (tmp = 0; tmp < count; tmp += 4) {
+			data = FillStat[i].pFill[src];
+			csa_writemem(resp, dst + tmp, data);
+			src++;
+		}
+	}
 
-	/* Check the destination address that it is a multiple of 4 */
-	if ((dest & 0x3) != 0)
-		return (EINVAL);
-
-	/* Write the buffer out. */
-	for (ul = 0 ; ul < len ; ul += 4)
-		csa_writemem(resp, dest + ul, src[ul >> 2]);
 	return (0);
 }
 
@@ -1096,6 +1076,7 @@ static device_method_t csa_methods[] = {
 	DEVMETHOD(device_resume,	csa_resume),
 
 	/* Bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
 	DEVMETHOD(bus_alloc_resource,	csa_alloc_resource),
 	DEVMETHOD(bus_release_resource,	csa_release_resource),
 	DEVMETHOD(bus_activate_resource, bus_generic_activate_resource),
@@ -1103,7 +1084,7 @@ static device_method_t csa_methods[] = {
 	DEVMETHOD(bus_setup_intr,	csa_setup_intr),
 	DEVMETHOD(bus_teardown_intr,	csa_teardown_intr),
 
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 static driver_t csa_driver = {
