@@ -161,12 +161,16 @@ static device_method_t ste_methods[] = {
 	DEVMETHOD(device_suspend,	ste_suspend),
 	DEVMETHOD(device_resume,	ste_resume),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	ste_miibus_readreg),
 	DEVMETHOD(miibus_writereg,	ste_miibus_writereg),
 	DEVMETHOD(miibus_statchg,	ste_miibus_statchg),
 
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 static driver_t ste_driver = {
@@ -310,7 +314,7 @@ ste_ifmedia_upd(struct ifnet *ifp)
 	STE_LOCK(sc);
 	mii = device_get_softc(sc->ste_miibus);
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 	error = mii_mediachg(mii);
 	STE_UNLOCK(sc);
 
@@ -1025,7 +1029,7 @@ ste_attach(device_t dev)
 	 */
 	ifp->if_data.ifi_hdrlen = sizeof(struct ether_vlan_header);
 	ifp->if_capabilities |= IFCAP_VLAN_MTU;
-	if (pci_find_extcap(dev, PCIY_PMG, &pmc) == 0)
+	if (pci_find_cap(dev, PCIY_PMG, &pmc) == 0)
 		ifp->if_capabilities |= IFCAP_WOL_MAGIC;
 	ifp->if_capenable = ifp->if_capabilities;
 #ifdef DEVICE_POLLING
@@ -2006,7 +2010,7 @@ ste_resume(device_t dev)
 
 	sc = device_get_softc(dev);
 	STE_LOCK(sc);
-	if (pci_find_extcap(sc->ste_dev, PCIY_PMG, &pmc) == 0) {
+	if (pci_find_cap(sc->ste_dev, PCIY_PMG, &pmc) == 0) {
 		/* Disable PME and clear PME status. */
 		pmstat = pci_read_config(sc->ste_dev,
 		    pmc + PCIR_POWER_STATUS, 2);
@@ -2029,7 +2033,7 @@ ste_resume(device_t dev)
 #define	STE_SYSCTL_STAT_ADD32(c, h, n, p, d)	\
 	    SYSCTL_ADD_UINT(c, h, OID_AUTO, n, CTLFLAG_RD, p, 0, d)
 #define	STE_SYSCTL_STAT_ADD64(c, h, n, p, d)	\
-	    SYSCTL_ADD_QUAD(c, h, OID_AUTO, n, CTLFLAG_RD, p, d)
+	    SYSCTL_ADD_UQUAD(c, h, OID_AUTO, n, CTLFLAG_RD, p, d)
 
 static void
 ste_sysctl_node(struct ste_softc *sc)
@@ -2110,7 +2114,7 @@ ste_setwol(struct ste_softc *sc)
 
 	STE_LOCK_ASSERT(sc);
 
-	if (pci_find_extcap(sc->ste_dev, PCIY_PMG, &pmc) != 0) {
+	if (pci_find_cap(sc->ste_dev, PCIY_PMG, &pmc) != 0) {
 		/* Disable WOL. */
 		CSR_READ_1(sc, STE_WAKE_EVENT);
 		CSR_WRITE_1(sc, STE_WAKE_EVENT, 0);
