@@ -15,13 +15,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the NetBSD
- *	Foundation, Inc. and its contributors.
- * 4. Neither the name of The NetBSD Foundation nor the names of its
- *    contributors may be used to endorse or promote products derived
- *    from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -221,7 +214,8 @@ static device_method_t stge_methods[] = {
 	DEVMETHOD(miibus_writereg,	stge_miibus_writereg),
 	DEVMETHOD(miibus_statchg,	stge_miibus_statchg),
 
-	DEVMETHOD_END
+	{ 0, 0 }
+
 };
 
 static driver_t stge_driver = {
@@ -602,7 +596,7 @@ stge_attach(device_t dev)
 	    (PC_PhyDuplexPolarity | PC_PhyLnkPolarity);
 
 	/* Set up MII bus. */
-	flags = MIIF_DOPAUSE | MIIF_FORCEPAUSE;
+	flags = MIIF_DOPAUSE;
 	if (sc->sc_rev >= 0x40 && sc->sc_rev <= 0x4e)
 		flags |= MIIF_MACPRIV0;
 	error = mii_attach(sc->sc_dev, &sc->sc_miibus, ifp, stge_mediachange,
