@@ -1,6 +1,6 @@
 /*-
- * Copyright (C) 2007-2008 Semihalf, Rafal Jaworowski <raj@semihalf.com>
- * Copyright (C) 2006-2007 Semihalf, Piotr Kruszynski <ppk@semihalf.com>
+ * Copyright (C) 2007-2008 Semihalf, Rafal Jaworowski
+ * Copyright (C) 2006-2007 Semihalf, Piotr Kruszynski
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -270,8 +270,8 @@ tsec_attach(struct tsec_softc *sc)
 	
 	/* Attach PHY(s) */
 	error = mii_attach(sc->dev, &sc->tsec_miibus, ifp, tsec_ifmedia_upd,
-	    tsec_ifmedia_sts, BMSR_DEFCAPMASK, device_get_unit(sc->dev),
-	    MII_OFFSET_ANY, 0);
+	    tsec_ifmedia_sts, BMSR_DEFCAPMASK, sc->phyaddr, MII_OFFSET_ANY,
+	    0);
 	if (error) {
 		device_printf(sc->dev, "attaching PHYs failed\n");
 		if_free(ifp);
