@@ -373,10 +373,9 @@ done_1:
 
 done_2:
 
-	/* reset frame number */
-	UWRITE2(sc, UHCI_FRNUM, 0);
-	/* set default SOF value */
-	UWRITE1(sc, UHCI_SOF, 0x40);
+	/* reload the configuration */
+	UWRITE2(sc, UHCI_FRNUM, sc->sc_saved_frnum);
+	UWRITE1(sc, UHCI_SOF, sc->sc_saved_sof);
 
 	USB_BUS_UNLOCK(&sc->sc_bus);
 
@@ -464,6 +463,9 @@ uhci_init(uhci_softc_t *sc)
 		uhci_dumpregs(sc);
 	}
 #endif
+	sc->sc_saved_sof = 0x40;	/* default value */
+	sc->sc_saved_frnum = 0;		/* default frame number */
+
 	/*
 	 * Setup QH's
 	 */
@@ -656,16 +658,24 @@ uhci_init(uhci_softc_t *sc)
 	return (0);
 }
 
-static void
+/* NOTE: suspend/resume is called from
+ * interrupt context and cannot sleep!
+ */
+
+void
 uhci_suspend(uhci_softc_t *sc)
 {
+	USB_BUS_LOCK(&sc->sc_bus);
+
 #ifdef USB_DEBUG
 	if (uhcidebug > 2) {
 		uhci_dumpregs(sc);
 	}
 #endif
+	/* save some state if BIOS doesn't */
 
-	USB_BUS_LOCK(&sc->sc_bus);
+	sc->sc_saved_frnum = UREAD2(sc, UHCI_FRNUM);
+	sc->sc_saved_sof = UREAD1(sc, UHCI_SOF);
 
 	/* stop the controller */
 
@@ -675,10 +685,13 @@ uhci_suspend(uhci_softc_t *sc)
 
 	UHCICMD(sc, UHCI_CMD_EGSM);
 
+	usb_pause_mtx(&sc->sc_bus.bus_mtx, 
+	    USB_MS_TO_TICKS(USB_RESUME_WAIT));
+
 	USB_BUS_UNLOCK(&sc->sc_bus);
 }
 
-static void
+void
 uhci_resume(uhci_softc_t *sc)
 {
 	USB_BUS_LOCK(&sc->sc_bus);
@@ -691,17 +704,21 @@ uhci_resume(uhci_softc_t *sc)
 
 	UHCICMD(sc, UHCI_CMD_FGR);
 
+	usb_pause_mtx(&sc->sc_bus.bus_mtx,
+	    USB_MS_TO_TICKS(USB_RESUME_DELAY));
+
 	/* and start traffic again */
 
 	uhci_start(sc);
 
-	USB_BUS_UNLOCK(&sc->sc_bus);
-
 #ifdef USB_DEBUG
-	if (uhcidebug > 2)
+	if (uhcidebug > 2) {
 		uhci_dumpregs(sc);
+	}
 #endif
 
+	USB_BUS_UNLOCK(&sc->sc_bus);
+
 	/* catch lost interrupts */
 	uhci_do_poll(&sc->sc_bus);
 }
@@ -2685,7 +2702,7 @@ uhci_roothub_exec(struct usb_device *udev,
 		break;
 	case C(UR_GET_STATUS, UT_READ_CLASS_DEVICE):
 		len = 16;
-		memset(sc->sc_hub_desc.temp, 0, 16);
+		bzero(sc->sc_hub_desc.temp, 16);
 		break;
 	case C(UR_GET_STATUS, UT_READ_CLASS_OTHER):
 		if (index == 1)
@@ -3161,24 +3178,6 @@ uhci_device_suspend(struct usb_device *udev)
 	return;
 }
 
-static void
-uhci_set_hw_power_sleep(struct usb_bus *bus, uint32_t state)
-{
-	struct uhci_softc *sc = UHCI_BUS2SC(bus);
-
-	switch (state) {
-	case USB_HW_POWER_SUSPEND:
-	case USB_HW_POWER_SHUTDOWN:
-		uhci_suspend(sc);
-		break;
-	case USB_HW_POWER_RESUME:
-		uhci_resume(sc);
-		break;
-	default:
-		break;
-	}
-}
-
 static void
 uhci_set_hw_power(struct usb_bus *bus)
 {
@@ -3226,7 +3225,6 @@ struct usb_bus_methods uhci_bus_methods =
 	.device_resume = uhci_device_resume,
 	.device_suspend = uhci_device_suspend,
 	.set_hw_power = uhci_set_hw_power,
-	.set_hw_power_sleep = uhci_set_hw_power_sleep,
 	.roothub_exec = uhci_roothub_exec,
 	.xfer_poll = uhci_do_poll,
 };
