@@ -292,7 +292,7 @@ xhci_start_controller(struct xhci_softc *sc)
 	XWRITE4(sc, oper, XHCI_USBCMD, XHCI_CMD_HCRST);
 
 	for (i = 0; i != 100; i++) {
-		usb_pause_mtx(NULL, hz / 100);
+		usb_pause_mtx(NULL, hz / 1000);
 		temp = XREAD4(sc, oper, XHCI_USBCMD) &
 		    (XHCI_CMD_HCRST | XHCI_STS_CNR);
 		if (!temp)
@@ -453,7 +453,7 @@ xhci_start_controller(struct xhci_softc *sc)
 	    XHCI_CMD_INTE | XHCI_CMD_HSEE);
 
 	for (i = 0; i != 100; i++) {
-		usb_pause_mtx(NULL, hz / 100);
+		usb_pause_mtx(NULL, hz / 1000);
 		temp = XREAD4(sc, oper, XHCI_USBSTS) & XHCI_STS_HCH;
 		if (!temp)
 			break;
@@ -487,7 +487,7 @@ xhci_halt_controller(struct xhci_softc *sc)
 	XWRITE4(sc, oper, XHCI_USBCMD, 0);
 
 	for (i = 0; i != 100; i++) {
-		usb_pause_mtx(NULL, hz / 100);
+		usb_pause_mtx(NULL, hz / 1000);
 		temp = XREAD4(sc, oper, XHCI_USBSTS) & XHCI_STS_HCH;
 		if (temp)
 			break;
@@ -550,27 +550,24 @@ xhci_uninit(struct xhci_softc *sc)
 	sx_destroy(&sc->sc_cmd_sx);
 }
 
-static void
-xhci_set_hw_power_sleep(struct usb_bus *bus, uint32_t state)
+void
+xhci_suspend(struct xhci_softc *sc)
 {
-	struct xhci_softc *sc = XHCI_BUS2SC(bus);
+	/* XXX TODO */
+}
 
-	switch (state) {
-	case USB_HW_POWER_SUSPEND:
-		DPRINTF("Stopping the XHCI\n");
-		xhci_halt_controller(sc);
-		break;
-	case USB_HW_POWER_SHUTDOWN:
-		DPRINTF("Stopping the XHCI\n");
-		xhci_halt_controller(sc);
-		break;
-	case USB_HW_POWER_RESUME:
-		DPRINTF("Starting the XHCI\n");
-		xhci_start_controller(sc);
-		break;
-	default:
-		break;
-	}
+void
+xhci_resume(struct xhci_softc *sc)
+{
+	/* XXX TODO */
+}
+
+void
+xhci_shutdown(struct xhci_softc *sc)
+{
+	DPRINTF("Stopping the XHCI\n");
+
+	xhci_halt_controller(sc);
 }
 
 static usb_error_t
@@ -1113,7 +1110,7 @@ xhci_cmd_nop(struct xhci_softc *sc)
 
 	trb.dwTrb3 = htole32(temp);
 
-	return (xhci_do_command(sc, &trb, 100 /* ms */));
+	return (xhci_do_command(sc, &trb, 50 /* ms */));
 }
 #endif
 
@@ -1130,7 +1127,7 @@ xhci_cmd_enable_slot(struct xhci_softc *sc, uint8_t *pslot)
 	trb.dwTrb2 = 0;
 	trb.dwTrb3 = htole32(XHCI_TRB_3_TYPE_SET(XHCI_TRB_TYPE_ENABLE_SLOT));
 
-	err = xhci_do_command(sc, &trb, 100 /* ms */);
+	err = xhci_do_command(sc, &trb, 50 /* ms */);
 	if (err)
 		goto done;
 
@@ -1157,7 +1154,7 @@ xhci_cmd_disable_slot(struct xhci_softc *sc, uint8_t slot_id)
 
 	trb.dwTrb3 = htole32(temp);
 
-	return (xhci_do_command(sc, &trb, 100 /* ms */));
+	return (xhci_do_command(sc, &trb, 50 /* ms */));
 }
 
 static usb_error_t
@@ -1313,7 +1310,7 @@ xhci_cmd_configure_ep(struct xhci_softc *sc, uint64_t input_ctx,
 
 	trb.dwTrb3 = htole32(temp);
 
-	return (xhci_do_command(sc, &trb, 100 /* ms */));
+	return (xhci_do_command(sc, &trb, 50 /* ms */));
 }
 
 static usb_error_t
@@ -1331,7 +1328,7 @@ xhci_cmd_evaluate_ctx(struct xhci_softc *sc, uint64_t input_ctx,
 	    XHCI_TRB_3_SLOT_SET(slot_id);
 	trb.dwTrb3 = htole32(temp);
 
-	return (xhci_do_command(sc, &trb, 100 /* ms */));
+	return (xhci_do_command(sc, &trb, 50 /* ms */));
 }
 
 static usb_error_t
@@ -1354,7 +1351,7 @@ xhci_cmd_reset_ep(struct xhci_softc *sc, uint8_t preserve,
 
 	trb.dwTrb3 = htole32(temp);
 
-	return (xhci_do_command(sc, &trb, 100 /* ms */));
+	return (xhci_do_command(sc, &trb, 50 /* ms */));
 }
 
 static usb_error_t
@@ -1376,7 +1373,7 @@ xhci_cmd_set_tr_dequeue_ptr(struct xhci_softc *sc, uint64_t dequeue_ptr,
 	    XHCI_TRB_3_EP_SET(ep_id);
 	trb.dwTrb3 = htole32(temp);
 
-	return (xhci_do_command(sc, &trb, 100 /* ms */));
+	return (xhci_do_command(sc, &trb, 50 /* ms */));
 }
 
 static usb_error_t
@@ -1399,7 +1396,7 @@ xhci_cmd_stop_ep(struct xhci_softc *sc, uint8_t suspend,
 
 	trb.dwTrb3 = htole32(temp);
 
-	return (xhci_do_command(sc, &trb, 100 /* ms */));
+	return (xhci_do_command(sc, &trb, 50 /* ms */));
 }
 
 static usb_error_t
@@ -1417,7 +1414,7 @@ xhci_cmd_reset_dev(struct xhci_softc *sc, uint8_t slot_id)
 
 	trb.dwTrb3 = htole32(temp);
 
-	return (xhci_do_command(sc, &trb, 100 /* ms */));
+	return (xhci_do_command(sc, &trb, 50 /* ms */));
 }
 
 /*------------------------------------------------------------------------*
@@ -2211,10 +2208,9 @@ xhci_configure_device(struct usb_device *udev)
 	struct usb_device *hubdev;
 	uint32_t temp;
 	uint32_t route;
-	uint32_t rh_port;
 	uint8_t is_hub;
 	uint8_t index;
-	uint8_t depth;
+	uint8_t rh_port;
 
 	index = udev->controller_slot_id;
 
@@ -2236,8 +2232,6 @@ xhci_configure_device(struct usb_device *udev)
 		if (hubdev->parent_hub == NULL)
 			break;
 
-		depth = hubdev->parent_hub->depth;
-
 		/*
 		 * NOTE: HS/FS/LS devices and the SS root HUB can have
 		 * more than 15 ports
@@ -2245,18 +2239,17 @@ xhci_configure_device(struct usb_device *udev)
 
 		rh_port = hubdev->port_no;
 
-		if (depth == 0)
+		if (hubdev->parent_hub->parent_hub == NULL)
 			break;
 
+		route *= 16;
+
 		if (rh_port > 15)
-			rh_port = 15;
-
-		if (depth < 6)
-			route |= rh_port << (4 * (depth - 1));
+			route |= 15;
+		else
+			route |= rh_port;
 	}
 
-	DPRINTF("Route=0x%08x\n", route);
-
 	temp = XHCI_SCTX_0_ROUTE_SET(route);
 
 	switch (sc->sc_hw.devs[index].state) {
@@ -2264,7 +2257,7 @@ xhci_configure_device(struct usb_device *udev)
 		temp |= XHCI_SCTX_0_CTX_NUM_SET(XHCI_MAX_ENDPOINTS - 1);
 		break;
 	default:
-		temp |= XHCI_SCTX_0_CTX_NUM_SET(1);
+		temp = XHCI_SCTX_0_CTX_NUM_SET(1);
 		break;
 	}
 
@@ -2838,7 +2831,7 @@ struct xhci_bos_desc xhci_bosd = {
 		.bLength = sizeof(xhci_bosd.usb2extd),
 		.bDescriptorType = 1,
 		.bDevCapabilityType = 2,
-		.bmAttributes[0] = 2,
+		.bmAttributes = 2,
 	},
 	.usbdcd = {
 		.bLength = sizeof(xhci_bosd.usbdcd),
@@ -2848,8 +2841,7 @@ struct xhci_bos_desc xhci_bosd = {
 		HSETW(.wSpeedsSupported, 0x000C),
 		.bFunctionalitySupport = 8,
 		.bU1DevExitLat = 255,	/* dummy - not used */
-		.wU2DevExitLat[0] = 0x00,
-		.wU2DevExitLat[1] = 0x08,
+		.bU2DevExitLat = 255,	/* dummy - not used */
 	},
 	.cidd = {
 		.bLength = sizeof(xhci_bosd.cidd),
@@ -3067,7 +3059,6 @@ xhci_roothub_exec(struct usb_device *udev,
 		case UHF_C_PORT_CONFIG_ERROR:
 			XWRITE4(sc, oper, port, v | XHCI_PS_CEC);
 			break;
-		case UHF_C_PORT_SUSPEND:
 		case UHF_C_PORT_LINK_STATE:
 			XWRITE4(sc, oper, port, v | XHCI_PS_PLC);
 			break;
@@ -3194,13 +3185,8 @@ xhci_roothub_exec(struct usb_device *udev,
 			i |= UPS_OVERCURRENT_INDICATOR;
 		if (v & XHCI_PS_PR)
 			i |= UPS_RESET;
-		if (v & XHCI_PS_PP) {
-			/*
-			 * The USB 3.0 RH is using the
-			 * USB 2.0's power bit
-			 */
+		if (v & XHCI_PS_PP)
 			i |= UPS_PORT_POWER;
-		}
 		USETW(sc->sc_hub_desc.ps.wPortStatus, i);
 
 		i = 0;
@@ -3941,5 +3927,4 @@ struct usb_bus_methods xhci_bus_methods = {
 	.set_address = xhci_set_address,
 	.clear_stall = xhci_ep_clear_stall,
 	.device_state_change = xhci_device_state_change,
-	.set_hw_power_sleep = xhci_set_hw_power_sleep,
 };
