@@ -58,24 +58,27 @@ __FBSDID("$FreeBSD$");
 #include <dev/usb/usb_pci.h>
 #include <dev/usb/controller/xhci.h>
 #include <dev/usb/controller/xhcireg.h>
-#include "usb_if.h"
 
 static device_probe_t xhci_pci_probe;
 static device_attach_t xhci_pci_attach;
 static device_detach_t xhci_pci_detach;
-static usb_take_controller_t xhci_pci_take_controller;
+static device_suspend_t xhci_pci_suspend;
+static device_resume_t xhci_pci_resume;
+static device_shutdown_t xhci_pci_shutdown;
+static void xhci_pci_takecontroller(device_t);
 
 static device_method_t xhci_device_methods[] = {
 	/* device interface */
 	DEVMETHOD(device_probe, xhci_pci_probe),
 	DEVMETHOD(device_attach, xhci_pci_attach),
 	DEVMETHOD(device_detach, xhci_pci_detach),
-	DEVMETHOD(device_suspend, bus_generic_suspend),
-	DEVMETHOD(device_resume, bus_generic_resume),
-	DEVMETHOD(device_shutdown, bus_generic_shutdown),
-	DEVMETHOD(usb_take_controller, xhci_pci_take_controller),
+	DEVMETHOD(device_suspend, xhci_pci_suspend),
+	DEVMETHOD(device_resume, xhci_pci_resume),
+	DEVMETHOD(device_shutdown, xhci_pci_shutdown),
+	/* bus interface */
+	DEVMETHOD(bus_print_child, bus_generic_print_child),
 
-	DEVMETHOD_END
+	{0, 0}
 };
 
 static driver_t xhci_driver = {
@@ -89,6 +92,45 @@ static devclass_t xhci_devclass;
 DRIVER_MODULE(xhci, pci, xhci_driver, xhci_devclass, 0, 0);
 MODULE_DEPEND(xhci, usb, 1, 1, 1);
 
+static int
+xhci_pci_suspend(device_t self)
+{
+	struct xhci_softc *sc = device_get_softc(self);
+	int err;
+
+	err = bus_generic_suspend(self);
+	if (err)
+		return (err);
+	xhci_suspend(sc);
+	return (0);
+}
+
+static int
+xhci_pci_resume(device_t self)
+{
+	struct xhci_softc *sc = device_get_softc(self);
+
+	xhci_pci_takecontroller(self);
+	xhci_resume(sc);
+
+	bus_generic_resume(self);
+
+	return (0);
+}
+
+static int
+xhci_pci_shutdown(device_t self)
+{
+	struct xhci_softc *sc = device_get_softc(self);
+	int err;
+
+	err = bus_generic_shutdown(self);
+	if (err)
+		return (err);
+	xhci_shutdown(sc);
+
+	return (0);
+}
 
 static const char *
 xhci_pci_match(device_t self)
@@ -169,7 +211,7 @@ xhci_pci_attach(device_t self)
 		sc->sc_intr_hdl = NULL;
 		goto error;
 	}
-	xhci_pci_take_controller(self);
+	xhci_pci_takecontroller(self);
 
 	err = xhci_halt_controller(sc);
 
@@ -228,8 +270,8 @@ xhci_pci_detach(device_t self)
 	return (0);
 }
 
-static int
-xhci_pci_take_controller(device_t self)
+static void
+xhci_pci_takecontroller(device_t self)
 {
 	struct xhci_softc *sc = device_get_softc(self);
 	uint32_t cparams;
@@ -272,5 +314,4 @@ xhci_pci_take_controller(device_t self)
 			usb_pause_mtx(NULL, hz / 100);	/* wait 10ms */
 		}
 	}
-	return (0);
 }
