@@ -254,12 +254,15 @@ static device_method_t aue_methods[] = {
 	DEVMETHOD(device_attach, aue_attach),
 	DEVMETHOD(device_detach, aue_detach),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child, bus_generic_print_child),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg, aue_miibus_readreg),
 	DEVMETHOD(miibus_writereg, aue_miibus_writereg),
 	DEVMETHOD(miibus_statchg, aue_miibus_statchg),
 
-	DEVMETHOD_END
+	{0, 0}
 };
 
 static driver_t aue_driver = {
@@ -1013,7 +1016,7 @@ aue_ifmedia_upd(struct ifnet *ifp)
 
         sc->sc_flags &= ~AUE_FLAG_LINK;
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 	mii_mediachg(mii);
 	return (0);
 }
@@ -1029,9 +1032,9 @@ aue_ifmedia_sts(struct ifnet *ifp, struct ifmediareq *ifmr)
 
 	AUE_LOCK(sc);
 	mii_pollstat(mii);
+	AUE_UNLOCK(sc);
 	ifmr->ifm_active = mii->mii_media_active;
 	ifmr->ifm_status = mii->mii_media_status;
-	AUE_UNLOCK(sc);
 }
 
 /*
