@@ -145,12 +145,15 @@ static device_method_t udav_methods[] = {
 	DEVMETHOD(device_attach, udav_attach),
 	DEVMETHOD(device_detach, udav_detach),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child, bus_generic_print_child),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg, udav_miibus_readreg),
 	DEVMETHOD(miibus_writereg, udav_miibus_writereg),
 	DEVMETHOD(miibus_statchg, udav_miibus_statchg),
 
-	DEVMETHOD_END
+	{0, 0}
 };
 
 static driver_t udav_driver = {
@@ -735,7 +738,7 @@ udav_ifmedia_upd(struct ifnet *ifp)
 
         sc->sc_flags &= ~UDAV_FLAG_LINK;
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 	mii_mediachg(mii);
 	return (0);
 }
@@ -748,9 +751,9 @@ udav_ifmedia_status(struct ifnet *ifp, struct ifmediareq *ifmr)
 
 	UDAV_LOCK(sc);
 	mii_pollstat(mii);
+	UDAV_UNLOCK(sc);
 	ifmr->ifm_active = mii->mii_media_active;
 	ifmr->ifm_status = mii->mii_media_status;
-	UDAV_UNLOCK(sc);
 }
 
 static void
