@@ -560,7 +560,7 @@ static int
 uftdi_set_parm_soft(struct termios *t,
     struct uftdi_param_config *cfg, uint8_t type)
 {
-	memset(cfg, 0, sizeof(*cfg));
+	bzero(cfg, sizeof(*cfg));
 
 	switch (type) {
 	case UFTDI_TYPE_SIO:
