@@ -127,11 +127,7 @@ static const STRUCT_USB_HOST_ID umodem_devs[] = {
 	/* Generic Modem class match */
 	{USB_IFACE_CLASS(UICLASS_CDC),
 		USB_IFACE_SUBCLASS(UISUBCLASS_ABSTRACT_CONTROL_MODEL),
-		USB_IFACE_PROTOCOL(UIPROTO_CDC_AT)},
-	/* Huawei Modem class match */
-	{USB_IFACE_CLASS(UICLASS_CDC),
-		USB_IFACE_SUBCLASS(UISUBCLASS_ABSTRACT_CONTROL_MODEL),
-		USB_IFACE_PROTOCOL(0xFF)},
+	USB_IFACE_PROTOCOL(UIPROTO_CDC_AT)},
 	/* Kyocera AH-K3001V */
 	{USB_VPI(USB_VENDOR_KYOCERA, USB_PRODUCT_KYOCERA_AHK3001V, 1)},
 	{USB_VPI(USB_VENDOR_SIERRA, USB_PRODUCT_SIERRA_MC5720, 1)},
@@ -540,7 +536,7 @@ umodem_cfg_param(struct ucom_softc *ucom, struct termios *t)
 
 	DPRINTF("sc=%p\n", sc);
 
-	memset(&ls, 0, sizeof(ls));
+	bzero(&ls, sizeof(ls));
 
 	USETDW(ls.dwDTERate, t->c_ospeed);
 
