@@ -311,9 +311,8 @@ uvisor_attach(device_t dev)
 	int error;
 
 	DPRINTF("sc=%p\n", sc);
-	memcpy(uvisor_config_copy, uvisor_config,
+	bcopy(uvisor_config, uvisor_config_copy,
 	    sizeof(uvisor_config_copy));
-
 	device_set_usb_desc(dev);
 
 	mtx_init(&sc->sc_mtx, "uvisor", NULL, MTX_DEF);
