@@ -80,9 +80,9 @@ void
 usbd_get_page(struct usb_page_cache *pc, usb_frlength_t offset,
     struct usb_page_search *res)
 {
-#if USB_HAVE_BUSDMA
 	struct usb_page *page;
 
+#if USB_HAVE_BUSDMA
 	if (pc->page_start) {
 
 		/* Case 1 - something has been loaded into DMA */
@@ -145,7 +145,7 @@ usbd_copy_in(struct usb_page_cache *cache, usb_frlength_t offset,
 		if (buf_res.length > len) {
 			buf_res.length = len;
 		}
-		memcpy(buf_res.buffer, ptr, buf_res.length);
+		bcopy(ptr, buf_res.buffer, buf_res.length);
 
 		offset += buf_res.length;
 		len -= buf_res.length;
@@ -267,7 +267,7 @@ usbd_copy_out(struct usb_page_cache *cache, usb_frlength_t offset,
 		if (res.length > len) {
 			res.length = len;
 		}
-		memcpy(ptr, res.buffer, res.length);
+		bcopy(res.buffer, ptr, res.length);
 
 		offset += res.length;
 		len -= res.length;
@@ -325,7 +325,7 @@ usbd_frame_zero(struct usb_page_cache *cache, usb_frlength_t offset,
 		if (res.length > len) {
 			res.length = len;
 		}
-		memset(res.buffer, 0, res.length);
+		bzero(res.buffer, res.length);
 
 		offset += res.length;
 		len -= res.length;
@@ -560,7 +560,7 @@ usb_pc_alloc_mem(struct usb_page_cache *pc, struct usb_page *pg,
 		bus_dmamem_free(utag->tag, ptr, map);
 		goto error;
 	}
-	memset(ptr, 0, size);
+	bzero(ptr, size);
 
 	usb_pc_cpu_flush(pc);
 
@@ -797,7 +797,7 @@ usb_dma_tag_setup(struct usb_dma_parent_tag *udpt,
     struct mtx *mtx, usb_dma_callback_t *func,
     uint8_t ndmabits, uint8_t nudt)
 {
-	memset(udpt, 0, sizeof(*udpt));
+	bzero(udpt, sizeof(*udpt));
 
 	/* sanity checking */
 	if ((nudt == 0) ||
@@ -818,7 +818,7 @@ usb_dma_tag_setup(struct usb_dma_parent_tag *udpt,
 	udpt->dma_bits = ndmabits;
 
 	while (nudt--) {
-		memset(udt, 0, sizeof(*udt));
+		bzero(udt, sizeof(*udt));
 		udt->tag_parent = udpt;
 		udt++;
 	}
