@@ -768,7 +768,7 @@ usb_fifo_close(struct usb_fifo *f, int fflags)
 	/* check if a thread wants SIGIO */
 	if (f->async_p != NULL) {
 		PROC_LOCK(f->async_p);
-		psignal(f->async_p, SIGIO);
+		kern_psignal(f->async_p, SIGIO);
 		PROC_UNLOCK(f->async_p);
 		f->async_p = NULL;
 	}
@@ -1582,7 +1582,7 @@ usb_fifo_wakeup(struct usb_fifo *f)
 	}
 	if (f->async_p != NULL) {
 		PROC_LOCK(f->async_p);
-		psignal(f->async_p, SIGIO);
+		kern_psignal(f->async_p, SIGIO);
 		PROC_UNLOCK(f->async_p);
 	}
 }
@@ -1809,8 +1809,8 @@ usb_fifo_free_buffer(struct usb_fifo *f)
 	}
 	/* reset queues */
 
-	memset(&f->free_q, 0, sizeof(f->free_q));
-	memset(&f->used_q, 0, sizeof(f->used_q));
+	bzero(&f->free_q, sizeof(f->free_q));
+	bzero(&f->used_q, sizeof(f->used_q));
 }
 
 void
@@ -1909,7 +1909,7 @@ usb_fifo_put_data_linear(struct usb_fifo *f, void *ptr,
 
 			io_len = MIN(len, m->cur_data_len);
 
-			memcpy(m->cur_data_ptr, ptr, io_len);
+			bcopy(ptr, m->cur_data_ptr, io_len);
 
 			m->cur_data_len = io_len;
 			ptr = USB_ADD_BYTES(ptr, io_len);
@@ -2052,7 +2052,7 @@ usb_fifo_get_data_linear(struct usb_fifo *f, void *ptr,
 
 			io_len = MIN(len, m->cur_data_len);
 
-			memcpy(ptr, m->cur_data_ptr, io_len);
+			bcopy(m->cur_data_ptr, ptr, io_len);
 
 			len -= io_len;
 			ptr = USB_ADD_BYTES(ptr, io_len);
