@@ -517,7 +517,7 @@ usbd_do_request_flags(struct usb_device *udev, struct mtx *mtx,
 				}
 			} else
 #endif
-				memcpy(data, desc, length);
+				bcopy(desc, data, length);
 		}
 		goto done;		/* success */
 	}
@@ -785,17 +785,12 @@ usbd_req_reset_port(struct usb_device *udev, struct mtx *mtx, uint8_t port)
 	struct usb_port_status ps;
 	usb_error_t err;
 	uint16_t n;
-	uint16_t status;
-	uint16_t change;
 
 #ifdef USB_DEBUG
 	uint16_t pr_poll_delay;
 	uint16_t pr_recovery_delay;
 
 #endif
-
-	DPRINTF("\n");
-
 	/* clear any leftover port reset changes first */
 	usbd_req_clear_port_feature(
 	    udev, mtx, port, UHF_C_PORT_RESET);
@@ -822,6 +817,9 @@ usbd_req_reset_port(struct usb_device *udev, struct mtx *mtx, uint8_t port)
 #endif
 	n = 0;
 	while (1) {
+		uint16_t status;
+		uint16_t change;
+
 #ifdef USB_DEBUG
 		/* wait for the device to recover from reset */
 		usb_pause_mtx(mtx, USB_MS_TO_TICKS(pr_poll_delay));
@@ -832,9 +830,9 @@ usbd_req_reset_port(struct usb_device *udev, struct mtx *mtx, uint8_t port)
 		n += USB_PORT_RESET_DELAY;
 #endif
 		err = usbd_req_get_port_status(udev, mtx, &ps, port);
-		if (err)
+		if (err) {
 			goto done;
-
+		}
 		status = UGETW(ps.wPortStatus);
 		change = UGETW(ps.wPortChange);
 
@@ -864,9 +862,9 @@ usbd_req_reset_port(struct usb_device *udev, struct mtx *mtx, uint8_t port)
 	/* clear port reset first */
 	err = usbd_req_clear_port_feature(
 	    udev, mtx, port, UHF_C_PORT_RESET);
-	if (err)
+	if (err) {
 		goto done;
-
+	}
 	/* check for timeout */
 	if (n == 0) {
 		err = USB_ERR_TIMEOUT;
@@ -900,50 +898,21 @@ done:
  *       disabled.
  *------------------------------------------------------------------------*/
 usb_error_t
-usbd_req_warm_reset_port(struct usb_device *udev, struct mtx *mtx,
-    uint8_t port)
+usbd_req_warm_reset_port(struct usb_device *udev, struct mtx *mtx, uint8_t port)
 {
 	struct usb_port_status ps;
 	usb_error_t err;
 	uint16_t n;
-	uint16_t status;
-	uint16_t change;
 
 #ifdef USB_DEBUG
 	uint16_t pr_poll_delay;
 	uint16_t pr_recovery_delay;
 
 #endif
-
-	DPRINTF("\n");
-
-	err = usbd_req_get_port_status(udev, mtx, &ps, port);
-	if (err)
+	err = usbd_req_set_port_feature(udev, mtx, port, UHF_BH_PORT_RESET);
+	if (err) {
 		goto done;
-
-	status = UGETW(ps.wPortStatus);
-
-	switch (UPS_PORT_LINK_STATE_GET(status)) {
-	case UPS_PORT_LS_U3:
-	case UPS_PORT_LS_COMP_MODE:
-	case UPS_PORT_LS_LOOPBACK:
-	case UPS_PORT_LS_SS_INA:
-		break;
-	default:
-		DPRINTF("Wrong state for warm reset\n");
-		return (0);
 	}
-
-	/* clear any leftover warm port reset changes first */
-	usbd_req_clear_port_feature(udev, mtx,
-	    port, UHF_C_BH_PORT_RESET);
-
-	/* set warm port reset */
-	err = usbd_req_set_port_feature(udev, mtx,
-	    port, UHF_BH_PORT_RESET);
-	if (err)
-		goto done;
-
 #ifdef USB_DEBUG
 	/* range check input parameters */
 	pr_poll_delay = usb_pr_poll_delay;
@@ -969,20 +938,17 @@ usbd_req_warm_reset_port(struct usb_device *udev, struct mtx *mtx,
 		n += USB_PORT_RESET_DELAY;
 #endif
 		err = usbd_req_get_port_status(udev, mtx, &ps, port);
-		if (err)
+		if (err) {
 			goto done;
-
-		status = UGETW(ps.wPortStatus);
-		change = UGETW(ps.wPortChange);
-
+		}
 		/* if the device disappeared, just give up */
-		if (!(status & UPS_CURRENT_CONNECT_STATUS))
+		if (!(UGETW(ps.wPortStatus) & UPS_CURRENT_CONNECT_STATUS)) {
 			goto done;
-
+		}
 		/* check if reset is complete */
-		if (change & UPS_C_BH_PORT_RESET)
+		if (UGETW(ps.wPortChange) & UPS_C_BH_PORT_RESET) {
 			break;
-
+		}
 		/* check for timeout */
 		if (n > 1000) {
 			n = 0;
@@ -993,9 +959,9 @@ usbd_req_warm_reset_port(struct usb_device *udev, struct mtx *mtx,
 	/* clear port reset first */
 	err = usbd_req_clear_port_feature(
 	    udev, mtx, port, UHF_C_BH_PORT_RESET);
-	if (err)
+	if (err) {
 		goto done;
-
+	}
 	/* check for timeout */
 	if (n == 0) {
 		err = USB_ERR_TIMEOUT;
@@ -2038,10 +2004,6 @@ retry:
 		}
 	}
 
-	/* Try to warm reset first */
-	if (parent_hub->speed == USB_SPEED_SUPER)
-		usbd_req_warm_reset_port(parent_hub, mtx, udev->port_no);
-
 	/* Try to reset the parent HUB port. */
 	err = usbd_req_reset_port(parent_hub, mtx, udev->port_no);
 	if (err) {
@@ -2202,27 +2164,3 @@ usbd_req_clear_tt_buffer(struct usb_device *udev, struct mtx *mtx,
 	USETW(req.wLength, 0);
 	return (usbd_do_request(udev, mtx, &req, 0));
 }
-
-/*------------------------------------------------------------------------*
- *	usbd_req_set_port_link_state
- *
- * USB 3.0 specific request
- *
- * Returns:
- *    0: Success
- * Else: Failure
- *------------------------------------------------------------------------*/
-usb_error_t
-usbd_req_set_port_link_state(struct usb_device *udev, struct mtx *mtx,
-    uint8_t port, uint8_t link_state)
-{
-	struct usb_device_request req;
-
-	req.bmRequestType = UT_WRITE_CLASS_OTHER;
-	req.bRequest = UR_SET_FEATURE;
-	USETW(req.wValue, UHF_PORT_LINK_STATE);
-	req.wIndex[0] = port;
-	req.wIndex[1] = link_state;
-	USETW(req.wLength, 0);
-	return (usbd_do_request(udev, mtx, &req, 0));
-}
