@@ -683,7 +683,7 @@ zyd_intr_read_callback(struct usb_xfer *xfer, usb_error_t error)
 				if (i != cnt)
 					continue;
 				/* copy answer into caller-supplied buffer */
-				memcpy(rqp->odata, cmd->data, rqp->olen);
+				bcopy(cmd->data, rqp->odata, rqp->olen);
 				DPRINTF(sc, ZYD_DEBUG_CMD,
 				    "command %p complete, data = %*D \n",
 				    rqp, rqp->olen, rqp->odata, ":");
@@ -783,7 +783,7 @@ zyd_cmd(struct zyd_softc *sc, uint16_t code, const void *idata, int ilen,
 		return (EINVAL);
 
 	cmd.code = htole16(code);
-	memcpy(cmd.data, idata, ilen);
+	bcopy(idata, cmd.data, ilen);
 	DPRINTF(sc, ZYD_DEBUG_CMD, "sending cmd %p = %*D\n",
 	    &rq, ilen, idata, ":");
 
