@@ -173,7 +173,6 @@ static __inline void
 static void	vge_freebufs(struct vge_softc *);
 static void	vge_ifmedia_sts(struct ifnet *, struct ifmediareq *);
 static int	vge_ifmedia_upd(struct ifnet *);
-static int	vge_ifmedia_upd_locked(struct vge_softc *);
 static void	vge_init(void *);
 static void	vge_init_locked(struct vge_softc *);
 static void	vge_intr(void *);
@@ -181,6 +180,7 @@ static void	vge_intr_holdoff(struct vge_softc *);
 static int	vge_ioctl(struct ifnet *, u_long, caddr_t);
 static void	vge_link_statchg(void *);
 static int	vge_miibus_readreg(device_t, int, int);
+static void	vge_miibus_statchg(device_t);
 static int	vge_miibus_writereg(device_t, int, int, int);
 static void	vge_miipoll_start(struct vge_softc *);
 static void	vge_miipoll_stop(struct vge_softc *);
@@ -190,7 +190,6 @@ static void	vge_reset(struct vge_softc *);
 static int	vge_rx_list_init(struct vge_softc *);
 static int	vge_rxeof(struct vge_softc *, int);
 static void	vge_rxfilter(struct vge_softc *);
-static void	vge_setmedia(struct vge_softc *);
 static void	vge_setvlan(struct vge_softc *);
 static void	vge_setwol(struct vge_softc *);
 static void	vge_start(struct ifnet *);
@@ -212,11 +211,16 @@ static device_method_t vge_methods[] = {
 	DEVMETHOD(device_resume,	vge_resume),
 	DEVMETHOD(device_shutdown,	vge_shutdown),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	vge_miibus_readreg),
 	DEVMETHOD(miibus_writereg,	vge_miibus_writereg),
+	DEVMETHOD(miibus_statchg,	vge_miibus_statchg),
 
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 static driver_t vge_driver = {
@@ -1015,12 +1019,12 @@ vge_attach(device_t dev)
 		goto fail;
 	}
 
-	if (pci_find_extcap(dev, PCIY_EXPRESS, &cap) == 0) {
+	if (pci_find_cap(dev, PCIY_EXPRESS, &cap) == 0) {
 		sc->vge_flags |= VGE_FLAG_PCIE;
 		sc->vge_expcap = cap;
 	} else
 		sc->vge_flags |= VGE_FLAG_JUMBO;
-	if (pci_find_extcap(dev, PCIY_PMG, &cap) == 0) {
+	if (pci_find_cap(dev, PCIY_PMG, &cap) == 0) {
 		sc->vge_flags |= VGE_FLAG_PMCAP;
 		sc->vge_pmcap = cap;
 	}
@@ -1095,11 +1099,10 @@ vge_attach(device_t dev)
 		goto fail;
 	}
 
-	vge_miipoll_start(sc);
 	/* Do MII setup */
 	error = mii_attach(dev, &sc->vge_miibus, ifp, vge_ifmedia_upd,
 	    vge_ifmedia_sts, BMSR_DEFCAPMASK, sc->vge_phyaddr, MII_OFFSET_ANY,
-	    MIIF_DOPAUSE);
+	    0);
 	if (error != 0) {
 		device_printf(dev, "attaching PHYs failed\n");
 		goto fail;
@@ -1657,41 +1660,30 @@ vge_link_statchg(void *xsc)
 {
 	struct vge_softc *sc;
 	struct ifnet *ifp;
-	uint8_t physts;
+	struct mii_data *mii;
 
 	sc = xsc;
 	ifp = sc->vge_ifp;
 	VGE_LOCK_ASSERT(sc);
+	mii = device_get_softc(sc->vge_miibus);
 
-	physts = CSR_READ_1(sc, VGE_PHYSTS0);
-	if ((physts & VGE_PHYSTS_RESETSTS) == 0) {
-		if ((physts & VGE_PHYSTS_LINK) == 0) {
+	mii_pollstat(mii);
+	if ((sc->vge_flags & VGE_FLAG_LINK) != 0) {
+		if (!(mii->mii_media_status & IFM_ACTIVE)) {
 			sc->vge_flags &= ~VGE_FLAG_LINK;
 			if_link_state_change(sc->vge_ifp,
 			    LINK_STATE_DOWN);
-		} else {
+		}
+	} else {
+		if (mii->mii_media_status & IFM_ACTIVE &&
+		    IFM_SUBTYPE(mii->mii_media_active) != IFM_NONE) {
 			sc->vge_flags |= VGE_FLAG_LINK;
 			if_link_state_change(sc->vge_ifp,
 			    LINK_STATE_UP);
-			CSR_WRITE_1(sc, VGE_CRC2, VGE_CR2_FDX_TXFLOWCTL_ENABLE |
-			    VGE_CR2_FDX_RXFLOWCTL_ENABLE);
-			if ((physts & VGE_PHYSTS_FDX) != 0) {
-				if ((physts & VGE_PHYSTS_TXFLOWCAP) != 0)
-					CSR_WRITE_1(sc, VGE_CRS2,
-					    VGE_CR2_FDX_TXFLOWCTL_ENABLE);
-				if ((physts & VGE_PHYSTS_RXFLOWCAP) != 0)
-					CSR_WRITE_1(sc, VGE_CRS2,
-					    VGE_CR2_FDX_RXFLOWCTL_ENABLE);
-			}
 			if (!IFQ_DRV_IS_EMPTY(&ifp->if_snd))
 				vge_start_locked(ifp);
 		}
 	}
-	/*
-	 * Restart MII auto-polling because link state change interrupt
-	 * will disable it.
-	 */
-	vge_miipoll_start(sc);
 }
 
 #ifdef DEVICE_POLLING
@@ -2036,7 +2028,6 @@ vge_init_locked(struct vge_softc *sc)
 	 */
 	vge_stop(sc);
 	vge_reset(sc);
-	vge_miipoll_start(sc);
 
 	/*
 	 * Initialize the RX and TX descriptors and mbufs.
@@ -2108,16 +2099,9 @@ vge_init_locked(struct vge_softc *sc)
 	vge_rxfilter(sc);
 	vge_setvlan(sc);
 
-	/* Initialize pause timer. */
-	CSR_WRITE_2(sc, VGE_TX_PAUSE_TIMER, 0xFFFF);
-	/*
-	 * Initialize flow control parameters.
-	 *  TX XON high threshold : 48
-	 *  TX pause low threshold : 24
-	 *  Disable hald-duplex flow control
-	 */
-	CSR_WRITE_1(sc, VGE_CRC2, 0xFF);
-	CSR_WRITE_1(sc, VGE_CRS2, VGE_CR2_XON_ENABLE | 0x0B);
+	/* Enable flow control */
+
+	CSR_WRITE_1(sc, VGE_CRS2, 0x8B);
 
 	/* Enable jumbo frame reception (if desired) */
 
@@ -2145,7 +2129,7 @@ vge_init_locked(struct vge_softc *sc)
 	CSR_WRITE_1(sc, VGE_CRS3, VGE_CR3_INT_GMSK);
 
 	sc->vge_flags &= ~VGE_FLAG_LINK;
-	vge_ifmedia_upd_locked(sc);
+	mii_mediachg(mii);
 
 	ifp->if_drv_flags |= IFF_DRV_RUNNING;
 	ifp->if_drv_flags &= ~IFF_DRV_OACTIVE;
@@ -2159,28 +2143,14 @@ static int
 vge_ifmedia_upd(struct ifnet *ifp)
 {
 	struct vge_softc *sc;
+	struct mii_data *mii;
 	int error;
 
 	sc = ifp->if_softc;
 	VGE_LOCK(sc);
-	error = vge_ifmedia_upd_locked(sc);
-	VGE_UNLOCK(sc);
-
-	return (error);
-}
-
-static int
-vge_ifmedia_upd_locked(struct vge_softc *sc)
-{
-	struct mii_data *mii;
-	struct mii_softc *miisc;
-	int error;
-
 	mii = device_get_softc(sc->vge_miibus);
-	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
-	vge_setmedia(sc);
 	error = mii_mediachg(mii);
+	VGE_UNLOCK(sc);
 
 	return (error);
 }
@@ -2203,17 +2173,19 @@ vge_ifmedia_sts(struct ifnet *ifp, struct ifmediareq *ifmr)
 		return;
 	}
 	mii_pollstat(mii);
+	VGE_UNLOCK(sc);
 	ifmr->ifm_active = mii->mii_media_active;
 	ifmr->ifm_status = mii->mii_media_status;
-	VGE_UNLOCK(sc);
 }
 
 static void
-vge_setmedia(struct vge_softc *sc)
+vge_miibus_statchg(device_t dev)
 {
+	struct vge_softc *sc;
 	struct mii_data *mii;
 	struct ifmedia_entry *ife;
 
+	sc = device_get_softc(dev);
 	mii = device_get_softc(sc->vge_miibus);
 	ife = mii->mii_media.ifm_cur;
 
@@ -2247,7 +2219,7 @@ vge_setmedia(struct vge_softc *sc)
 		}
 		break;
 	default:
-		device_printf(sc->vge_dev, "unknown media type: %x\n",
+		device_printf(dev, "unknown media type: %x\n",
 		    IFM_SUBTYPE(ife->ifm_media));
 		break;
 	}
@@ -2800,9 +2772,6 @@ vge_setlinkspeed(struct vge_softc *sc)
 			break;
 		}
 	}
-	/* Clear forced MAC speed/duplex configuration. */
-	CSR_CLRBIT_1(sc, VGE_DIAGCTL, VGE_DIAGCTL_MACFORCE);
-	CSR_CLRBIT_1(sc, VGE_DIAGCTL, VGE_DIAGCTL_FDXFORCE);
 	vge_miibus_writereg(sc->vge_dev, sc->vge_phyaddr, MII_100T2CR, 0);
 	vge_miibus_writereg(sc->vge_dev, sc->vge_phyaddr, MII_ANAR,
 	    ANAR_TX_FD | ANAR_TX | ANAR_10_FD | ANAR_10 | ANAR_CSMA);
