@@ -212,12 +212,15 @@ static device_method_t wb_methods[] = {
 	DEVMETHOD(device_detach,	wb_detach),
 	DEVMETHOD(device_shutdown,	wb_shutdown),
 
+	/* bus interface, for miibus */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	wb_miibus_readreg),
 	DEVMETHOD(miibus_writereg,	wb_miibus_writereg),
 	DEVMETHOD(miibus_statchg,	wb_miibus_statchg),
-
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 static driver_t wb_driver = {
@@ -534,7 +537,7 @@ wb_reset(sc)
 
 	mii = device_get_softc(sc->wb_miibus);
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 }
 
 static void
@@ -619,9 +622,6 @@ wb_attach(dev)
 		goto fail;
 	}
 
-	sc->wb_btag = rman_get_bustag(sc->wb_res);
-	sc->wb_bhandle = rman_get_bushandle(sc->wb_res);
-
 	/* Allocate interrupt */
 	rid = 0;
 	sc->wb_irq = bus_alloc_resource_any(dev, SYS_RES_IRQ, &rid,
