@@ -50,8 +50,6 @@ extern struct nfsclienthashhead nfsclienthash[NFSCLIENTHASHSIZE];
 extern int nfsrc_floodlevel, nfsrc_tcpsavedreplies;
 NFSV4ROOTLOCKMUTEX;
 NFSSTATESPINLOCK;
-vnode_t nfsv4root_vp = NULL;
-int nfsv4root_set = 0;
 
 int (*nfsrv3_procs0[NFS_V3NPROCS])(struct nfsrv_descript *,
     int, vnode_t , NFSPROC_T *, struct nfsexstuff *) = {
@@ -354,7 +352,7 @@ APPLESTATIC void
 nfsrvd_dorpc(struct nfsrv_descript *nd, int isdgram,
     NFSPROC_T *p)
 {
-	int error = 0;
+	int error = 0, lktype;
 	vnode_t vp;
 	mount_t mp = NULL;
 	struct nfsrvfh fh;
@@ -378,17 +376,24 @@ nfsrvd_dorpc(struct nfsrv_descript *nd, int isdgram,
 				if (error != EBADRPC)
 					printf("nfs dorpc err1=%d\n", error);
 				nd->nd_repstat = NFSERR_GARBAGE;
-				return;
+				goto out;
 			}
-			nes.nes_vfslocked = 0;
+			if (nd->nd_procnum == NFSPROC_READ ||
+			    nd->nd_procnum == NFSPROC_READDIR ||
+			    nd->nd_procnum == NFSPROC_READLINK ||
+			    nd->nd_procnum == NFSPROC_GETATTR ||
+			    nd->nd_procnum == NFSPROC_ACCESS)
+				lktype = LK_SHARED;
+			else
+				lktype = LK_EXCLUSIVE;
 			if (nd->nd_flag & ND_PUBLOOKUP)
-				nfsd_fhtovp(nd, &nfs_pubfh, &vp, &nes,
+				nfsd_fhtovp(nd, &nfs_pubfh, lktype, &vp, &nes,
 				    &mp, nfs_writerpc[nd->nd_procnum], p);
 			else
-				nfsd_fhtovp(nd, &fh, &vp, &nes,
+				nfsd_fhtovp(nd, &fh, lktype, &vp, &nes,
 				    &mp, nfs_writerpc[nd->nd_procnum], p);
 			if (nd->nd_repstat == NFSERR_PROGNOTV4)
-				return;
+				goto out;
 		}
 	}
 
@@ -409,13 +414,9 @@ nfsrvd_dorpc(struct nfsrv_descript *nd, int isdgram,
 	if (nd->nd_repstat && (nd->nd_flag & ND_NFSV2)) {
 		*nd->nd_errp = nfsd_errmap(nd);
 		NFSINCRGLOBAL(newnfsstats.srvrpccnt[nfsv3to4op[nd->nd_procnum]]);
-		if (mp != NULL) {
-			if (nfs_writerpc[nd->nd_procnum])
-				NFS_ENDWRITE(mp);
-			if (nes.nes_vfslocked)
-				nfsvno_unlockvfs(mp);
-		}
-		return;
+		if (mp != NULL && nfs_writerpc[nd->nd_procnum] != 0)
+			vn_finished_write(mp);
+		goto out;
 	}
 
 	/*
@@ -429,7 +430,7 @@ nfsrvd_dorpc(struct nfsrv_descript *nd, int isdgram,
 	} else {
 		if (nfs_retfh[nd->nd_procnum] == 1) {
 			if (vp)
-				NFSVOPUNLOCK(vp, 0, p);
+				NFSVOPUNLOCK(vp, 0);
 			error = (*(nfsrv3_procs1[nd->nd_procnum]))(nd, isdgram,
 			    vp, NULL, (fhandle_t *)fh.nfsrvfh_data, p, &nes);
 		} else if (nfs_retfh[nd->nd_procnum] == 2) {
@@ -439,12 +440,8 @@ nfsrvd_dorpc(struct nfsrv_descript *nd, int isdgram,
 			error = (*(nfsrv3_procs0[nd->nd_procnum]))(nd, isdgram,
 			    vp, p, &nes);
 		}
-		if (mp) {
-			if (nfs_writerpc[nd->nd_procnum])
-				NFS_ENDWRITE(mp);
-			if (nes.nes_vfslocked)
-				nfsvno_unlockvfs(mp);
-		}
+		if (mp != NULL && nfs_writerpc[nd->nd_procnum] != 0)
+			vn_finished_write(mp);
 		NFSINCRGLOBAL(newnfsstats.srvrpccnt[nfsv3to4op[nd->nd_procnum]]);
 	}
 	if (error) {
@@ -472,6 +469,9 @@ nfsrvd_dorpc(struct nfsrv_descript *nd, int isdgram,
 	     nd->nd_repstat == NFSERR_GRACE ||
 	     nd->nd_repstat == NFSERR_NOGRACE))
 		nd->nd_flag &= ~ND_SAVEREPLY;
+
+out:
+	NFSEXITCODE2(0, nd);
 }
 
 /*
@@ -492,9 +492,10 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 	u_char tag[NFSV4_SMALLSTR + 1], *tagstr;
 	vnode_t vp, nvp, savevp;
 	struct nfsrvfh fh;
-	mount_t mp, savemp;
+	mount_t new_mp, temp_mp = NULL;
 	struct ucred *credanon;
 	struct nfsexstuff nes, vpnes, savevpnes;
+	fsid_t cur_fsid, save_fsid;
 	static u_int64_t compref = 0;
 
 	NFSVNO_EXINIT(&vpnes);
@@ -527,10 +528,10 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 	NFSLOCKV4ROOTMUTEX();
 	if (nfsrv_stablefirst.nsf_flags & NFSNSF_NEEDLOCK)
 		igotlock = nfsv4_lock(&nfsv4rootfs_lock, 1, NULL,
-		    NFSV4ROOTLOCKMUTEXPTR);
+		    NFSV4ROOTLOCKMUTEXPTR, NULL);
 	else
 		igotlock = nfsv4_lock(&nfsv4rootfs_lock, 0, NULL,
-		    NFSV4ROOTLOCKMUTEXPTR);
+		    NFSV4ROOTLOCKMUTEXPTR, NULL);
 	NFSUNLOCKV4ROOTMUTEX();
 	if (igotlock) {
 		/*
@@ -578,7 +579,7 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 		 */
 		NFSLOCKV4ROOTMUTEX();
 		nfsv4_getref(&nfsv4rootfs_lock, NULL,
-		    NFSV4ROOTLOCKMUTEXPTR);
+		    NFSV4ROOTLOCKMUTEXPTR, NULL);
 		NFSUNLOCKV4ROOTMUTEX();
 	}
 
@@ -591,8 +592,8 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 	}
 
 	savevp = vp = NULL;
-	savevpnes.nes_vfslocked = vpnes.nes_vfslocked = 0;
-	savemp = mp = NULL;
+	save_fsid.val[0] = save_fsid.val[1] = 0;
+	cur_fsid.val[0] = cur_fsid.val[1] = 0;
 	NFSM_DISSECT(tl, u_int32_t *, NFSX_UNSIGNED);
 	taglen = fxdr_unsigned(int, *tl);
 	if (taglen < 0) {
@@ -698,62 +699,48 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 			error = nfsrv_mtofh(nd, &fh);
 			if (error)
 				goto nfsmout;
-			if (!nd->nd_repstat) {
-				nes.nes_vfslocked = vpnes.nes_vfslocked;
-				nfsd_fhtovp(nd, &fh, &nvp, &nes, &mp,
-				    0, p);
-			}
+			if (!nd->nd_repstat)
+				nfsd_fhtovp(nd, &fh, LK_SHARED, &nvp, &nes,
+				    NULL, 0, p);
 			/* For now, allow this for non-export FHs */
 			if (!nd->nd_repstat) {
 				if (vp)
 					vrele(vp);
 				vp = nvp;
-				NFSVOPUNLOCK(vp, 0, p);
+				cur_fsid = vp->v_mount->mnt_stat.f_fsid;
+				NFSVOPUNLOCK(vp, 0);
 				vpnes = nes;
 			}
 			break;
 		case NFSV4OP_PUTPUBFH:
-			if (nfs_pubfhset) {
-			    nes.nes_vfslocked = vpnes.nes_vfslocked;
-			    nfsd_fhtovp(nd, &nfs_pubfh, &nvp,
-				&nes, &mp, 0, p);
-			} else {
+			if (nfs_pubfhset)
+			    nfsd_fhtovp(nd, &nfs_pubfh, LK_SHARED, &nvp,
+				&nes, NULL, 0, p);
+			else
 			    nd->nd_repstat = NFSERR_NOFILEHANDLE;
-			}
 			if (!nd->nd_repstat) {
 				if (vp)
 					vrele(vp);
 				vp = nvp;
-				NFSVOPUNLOCK(vp, 0, p);
+				cur_fsid = vp->v_mount->mnt_stat.f_fsid;
+				NFSVOPUNLOCK(vp, 0);
 				vpnes = nes;
 			}
 			break;
 		case NFSV4OP_PUTROOTFH:
 			if (nfs_rootfhset) {
-				nes.nes_vfslocked = vpnes.nes_vfslocked;
-				nfsd_fhtovp(nd, &nfs_rootfh, &nvp,
-				    &nes, &mp, 0, p);
+				nfsd_fhtovp(nd, &nfs_rootfh, LK_SHARED, &nvp,
+				    &nes, NULL, 0, p);
 				if (!nd->nd_repstat) {
 					if (vp)
 						vrele(vp);
 					vp = nvp;
-					NFSVOPUNLOCK(vp, 0, p);
+					cur_fsid = vp->v_mount->mnt_stat.f_fsid;
+					NFSVOPUNLOCK(vp, 0);
 					vpnes = nes;
 				}
-			} else if (nfsv4root_vp && nfsv4root_set) {
-				if (vp) {
-					if (vpnes.nes_vfslocked)
-						nfsvno_unlockvfs(mp);
-					vrele(vp);
-				}
-				vp = nfsv4root_vp;
-				VREF(vp);
-				NFSVNO_SETEXRDONLY(&vpnes);
-				vpnes.nes_vfslocked = 0;
-				mp = vnode_mount(vp);
-			} else {
+			} else
 				nd->nd_repstat = NFSERR_NOFILEHANDLE;
-			}
 			break;
 		case NFSV4OP_SAVEFH:
 			if (vp && NFSVNO_EXPORTED(&vpnes)) {
@@ -765,7 +752,7 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 					VREF(vp);
 					savevp = vp;
 					savevpnes = vpnes;
-					savemp = mp;
+					save_fsid = cur_fsid;
 				}
 			} else {
 				nd->nd_repstat = NFSERR_NOFILEHANDLE;
@@ -777,23 +764,10 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 				/* If vp == savevp, a no-op */
 				if (vp != savevp) {
 					VREF(savevp);
-					if (mp == NULL || savemp == NULL)
-						panic("nfscmpmp");
-					if (!savevpnes.nes_vfslocked &&
-					    vpnes.nes_vfslocked) {
-						if (mp == savemp)
-							panic("nfscmp2");
-						nfsvno_unlockvfs(mp);
-					} else if (savevpnes.nes_vfslocked &&
-					    !vpnes.nes_vfslocked) {
-						if (mp == savemp)
-							panic("nfscmp3");
-						savevpnes.nes_vfslocked = nfsvno_lockvfs(savemp);
-					}
 					vrele(vp);
 					vp = savevp;
 					vpnes = savevpnes;
-					mp = savemp;
+					cur_fsid = save_fsid;
 				}
 			} else {
 				nd->nd_repstat = NFSERR_RESTOREFH;
@@ -815,6 +789,8 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 			    op != NFSV4OP_LOOKUP &&
 			    op != NFSV4OP_GETATTR &&
 			    op != NFSV4OP_GETFH &&
+			    op != NFSV4OP_ACCESS &&
+			    op != NFSV4OP_READLINK &&
 			    op != NFSV4OP_SECINFO)
 				nd->nd_repstat = NFSERR_NOFILEHANDLE;
 			else if (nfsvno_testexp(nd, &vpnes) &&
@@ -843,26 +819,18 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 			}
 			VREF(vp);
 			if (nfsv4_opflag[op].modifyfs)
-				NFS_STARTWRITE(NULL, &mp);
+				vn_start_write(vp, &temp_mp, V_WAIT);
 			error = (*(nfsrv4_ops1[op]))(nd, isdgram, vp,
 			    &nvp, (fhandle_t *)fh.nfsrvfh_data, p, &vpnes);
 			if (!error && !nd->nd_repstat) {
-			    if (vfs_statfs(mp)->f_fsid.val[0] !=
-				vfs_statfs(vnode_mount(nvp))->f_fsid.val[0] ||
-				vfs_statfs(mp)->f_fsid.val[1] !=
-				vfs_statfs(vnode_mount(nvp))->f_fsid.val[1]) {
-				if (vfs_statfs(vnode_mount(nvp))->f_fsid.val[0] ==
-				    NFSV4ROOT_FSID0 &&
-				    vfs_statfs(vnode_mount(nvp))->f_fsid.val[1] ==
-				    NFSV4ROOT_FSID1) {
-				    if (vpnes.nes_vfslocked) {
-					nfsvno_unlockvfs(mp);
-					vpnes.nes_vfslocked = 0;
-				    }
-				    NFSVNO_SETEXRDONLY(&vpnes);
-				    mp = vnode_mount(nvp);
-				} else {
-				    nd->nd_repstat = nfsvno_checkexp(vnode_mount(nvp),
+			    if (op == NFSV4OP_LOOKUP || op == NFSV4OP_LOOKUPP) {
+				new_mp = nvp->v_mount;
+				if (cur_fsid.val[0] !=
+				    new_mp->mnt_stat.f_fsid.val[0] ||
+				    cur_fsid.val[1] !=
+				    new_mp->mnt_stat.f_fsid.val[1]) {
+				    /* crossed a server mount point */
+				    nd->nd_repstat = nfsvno_checkexp(new_mp,
 					nd->nd_nam, &nes, &credanon);
 				    if (!nd->nd_repstat)
 					nd->nd_repstat = nfsd_excred(nd,
@@ -870,65 +838,73 @@ nfsrvd_compound(struct nfsrv_descript *nd, int isdgram,
 				    if (credanon != NULL)
 					crfree(credanon);
 				    if (!nd->nd_repstat) {
-					if (vpnes.nes_vfslocked)
-					    nfsvno_unlockvfs(mp);
-					mp = vnode_mount(nvp);
 					vpnes = nes;
-					vpnes.nes_vfslocked =
-					    nfsvno_lockvfs(mp);
+					cur_fsid = new_mp->mnt_stat.f_fsid;
 				    }
 				}
+				/* Lookup ops return a locked vnode */
+				NFSVOPUNLOCK(nvp, 0);
 			    }
 			    if (!nd->nd_repstat) {
 				    vrele(vp);
 				    vp = nvp;
-			    }
+			    } else
+				    vrele(nvp);
 			}
 			if (nfsv4_opflag[op].modifyfs)
-				NFS_ENDWRITE(mp);
+				vn_finished_write(temp_mp);
 		    } else if (nfsv4_opflag[op].retfh == 2) {
 			if (vp == NULL || savevp == NULL) {
 				nd->nd_repstat = NFSERR_NOFILEHANDLE;
 				break;
-			} else if (mp != savemp) {
+			} else if (cur_fsid.val[0] != save_fsid.val[0] ||
+			    cur_fsid.val[1] != save_fsid.val[1]) {
 				nd->nd_repstat = NFSERR_XDEV;
 				break;
 			}
-			VREF(vp);
-			VREF(savevp);
 			if (nfsv4_opflag[op].modifyfs)
-				NFS_STARTWRITE(NULL, &mp);
-			NFSVOPLOCK(savevp, LK_EXCLUSIVE | LK_RETRY, p);
-			error = (*(nfsrv4_ops2[op]))(nd, isdgram, savevp,
-			    vp, p, &savevpnes, &vpnes);
+				vn_start_write(savevp, &temp_mp, V_WAIT);
+			if (NFSVOPLOCK(savevp, LK_EXCLUSIVE) == 0) {
+				VREF(vp);
+				VREF(savevp);
+				error = (*(nfsrv4_ops2[op]))(nd, isdgram,
+				    savevp, vp, p, &savevpnes, &vpnes);
+			} else
+				nd->nd_repstat = NFSERR_PERM;
 			if (nfsv4_opflag[op].modifyfs)
-				NFS_ENDWRITE(mp);
+				vn_finished_write(temp_mp);
 		    } else {
 			if (nfsv4_opflag[op].retfh != 0)
 				panic("nfsrvd_compound");
 			if (nfsv4_opflag[op].needscfh) {
-				if (vp) {
-					VREF(vp);
+				if (vp != NULL) {
 					if (nfsv4_opflag[op].modifyfs)
-						NFS_STARTWRITE(NULL, &mp);
-					NFSVOPLOCK(vp, LK_EXCLUSIVE | LK_RETRY, p);
+						vn_start_write(vp, &temp_mp,
+						    V_WAIT);
+					if (NFSVOPLOCK(vp, nfsv4_opflag[op].lktype)
+					    == 0)
+						VREF(vp);
+					else
+						nd->nd_repstat = NFSERR_PERM;
 				} else {
 					nd->nd_repstat = NFSERR_NOFILEHANDLE;
 					if (op == NFSV4OP_SETATTR) {
-					    /*
-					     * Setattr reply requires a bitmap
-					     * even for errors like these.
-					     */
-					    NFSM_BUILD(tl, u_int32_t *,
-						NFSX_UNSIGNED);
-					    *tl = 0;
+						/*
+						 * Setattr reply requires a
+						 * bitmap even for errors like
+						 * these.
+						 */
+						NFSM_BUILD(tl, u_int32_t *,
+						    NFSX_UNSIGNED);
+						*tl = 0;
 					}
 					break;
 				}
-				error = (*(nfsrv4_ops0[op]))(nd, isdgram, vp,
-				    p, &vpnes);
+				if (nd->nd_repstat == 0)
+					error = (*(nfsrv4_ops0[op]))(nd,
+					    isdgram, vp, p, &vpnes);
 				if (nfsv4_opflag[op].modifyfs)
-					NFS_ENDWRITE(mp);
+					vn_finished_write(temp_mp);
 			} else {
 				error = (*(nfsrv4_ops0[op]))(nd, isdgram,
 				    NULL, p, &vpnes);
@@ -966,8 +942,6 @@ nfsmout:
 	} else {
 		*retopsp = txdr_unsigned(retops);
 	}
-	if (mp && vpnes.nes_vfslocked)
-		nfsvno_unlockvfs(mp);
 	if (vp)
 		vrele(vp);
 	if (savevp)
@@ -975,4 +949,6 @@ nfsmout:
 	NFSLOCKV4ROOTMUTEX();
 	nfsv4_relref(&nfsv4rootfs_lock);
 	NFSUNLOCKV4ROOTMUTEX();
+
+	NFSEXITCODE2(0, nd);
 }
