@@ -278,6 +278,7 @@ ntfs_loadntnode(
 
 		bn = ntfs_cntobn(ntmp->ntm_mftcn) +
 			ntmp->ntm_bpmftrec * ip->i_number;
+		bn *= ntmp->ntm_multiplier;
 
 		error = bread(ntmp->ntm_devvp,
 			      bn, ntfs_bntob(ntmp->ntm_bpmftrec),
@@ -581,7 +582,7 @@ ntfs_attrtontvattr(
 		memcpy(vap->va_datap, (caddr_t) rap + rap->a_r.a_dataoff,
 		       rap->a_r.a_datalen);
 	}
-	ddprintf((", len: %d", vap->va_datalen));
+	ddprintf((", len: %lld", vap->va_datalen));
 
 	if (error)
 		free(vap, M_NTFSNTVATTR);
@@ -666,18 +667,23 @@ ntfs_uastricmp(ntmp, ustr, ustrlen, astr, astrlen)
 	const char *astr;
 	size_t astrlen;
 {
-	const char *astrp = astr;
-	char tmpbuf[5];
-	int len, res;
+	int len;
 	size_t i, j, mbstrlen = astrlen;
+	int res;
+	wchar wc;
 
 	if (ntmp->ntm_ic_l2u) {
-		for (i = 0, j = 0; i < ustrlen && j < astrlen; i++) {
-			len = 4;
+		for (i = 0, j = 0; i < ustrlen && j < astrlen; i++, j++) {
+			if (j < astrlen -1) {
+				wc = (wchar)astr[j]<<8 | (astr[j+1]&0xFF);
+				len = 2;
+			} else {
+				wc = (wchar)astr[j]<<8 & 0xFF00;
+				len = 1;
+			}
 			res = ((int) NTFS_TOUPPER(ustr[i])) -
-			    ((int)NTFS_TOUPPER(NTFS_82U(astrp, &len)));
-			astrp += len;
-			j += len;
+				((int)NTFS_TOUPPER(NTFS_82U(wc, &len)));
+			j += len - 1;
 			mbstrlen -= len - 1;
 
 			if (res)
@@ -690,8 +696,7 @@ ntfs_uastricmp(ntmp, ustr, ustrlen, astr, astrlen)
 		 */
 		for (i = 0; i < ustrlen && i < astrlen; i++) {
 			res = ((int) NTFS_TOUPPER(NTFS_82U(NTFS_U28(ustr[i]), &len))) -
-				((int)NTFS_TOUPPER(NTFS_82U(astrp, &len)));
-			astrp++;
+				((int)NTFS_TOUPPER(NTFS_82U((wchar)astr[i], &len)));
 			if (res)
 				return res;
 		}
@@ -710,18 +715,23 @@ ntfs_uastrcmp(ntmp, ustr, ustrlen, astr, astrlen)
 	const char *astr;
 	size_t astrlen;
 {
-	char *c, tmpbuf[5];
+	char u, l;
 	size_t i, j, mbstrlen = astrlen;
 	int res;
+	wchar wc;
 
-	for (i = 0, j = 0; (i < ustrlen) && (j < astrlen); i++, mbstrlen++) {
-		c = NTFS_U28(ustr[i]);
-		while (*c != '\0') {
-			res = (int) (*c++ - astr[j++]);
-			if (res)
-				return res;
+	for (i = 0, j = 0; (i < ustrlen) && (j < astrlen); i++, j++) {
+		res = 0;
+		wc = NTFS_U28(ustr[i]);
+		u = (char)(wc>>8);
+		l = (char)wc;
+		if (u != '\0' && j < astrlen -1) {
+			res = (int) (u - astr[j++]);
 			mbstrlen--;
 		}
+		res = (res<<8) + (int) (l - astr[j]);
+		if (res)
+			return res;
 	}
 	return (ustrlen - mbstrlen);
 }
@@ -1482,11 +1492,13 @@ ntfs_writentvattr_plain(
 				(u_int32_t) left));
 			if ((off == 0) && (tocopy == ntfs_cntob(cl)))
 			{
-				bp = getblk(ntmp->ntm_devvp, ntfs_cntobn(cn),
+				bp = getblk(ntmp->ntm_devvp, ntfs_cntobn(cn)
+					    * ntmp->ntm_multiplier,
 					    ntfs_cntob(cl), 0, 0, 0);
 				clrbuf(bp);
 			} else {
-				error = bread(ntmp->ntm_devvp, ntfs_cntobn(cn),
+				error = bread(ntmp->ntm_devvp, ntfs_cntobn(cn)
+					      * ntmp->ntm_multiplier,
 					      ntfs_cntob(cl), NOCRED, &bp);
 				if (error) {
 					brelse(bp);
@@ -1593,7 +1605,8 @@ ntfs_readntvattr_plain(
 						(u_int32_t) tocopy, 
 						(u_int32_t) left));
 					error = bread(ntmp->ntm_devvp,
-						      ntfs_cntobn(cn),
+						      ntfs_cntobn(cn)
+						      * ntmp->ntm_multiplier,
 						      ntfs_cntob(cl),
 						      NOCRED, &bp);
 					if (error) {
@@ -1869,7 +1882,7 @@ ntfs_procfixups(
 		       fhp->fh_magic, magic);
 		return (EINVAL);
 	}
-	if ((fhp->fh_fnum - 1) * ntmp->ntm_bps != len) {
+	if ((fhp->fh_fnum - 1) * NTFS_BLOCK_SIZE != len) {
 		printf("ntfs_procfixups: " \
 		       "bad fixups number: %d for %ld bytes block\n", 
 		       fhp->fh_fnum, (long)len);	/* XXX printf kludge */
@@ -1880,7 +1893,7 @@ ntfs_procfixups(
 		return (EINVAL);
 	}
 	fxp = (u_int16_t *) (buf + fhp->fh_foff);
-	cfxp = (u_int16_t *) (buf + ntmp->ntm_bps - 2);
+	cfxp = (u_int16_t *) (buf + NTFS_BLOCK_SIZE - 2);
 	fixup = *fxp++;
 	for (i = 1; i < fhp->fh_fnum; i++, fxp++) {
 		if (*cfxp != fixup) {
@@ -1888,7 +1901,7 @@ ntfs_procfixups(
 			return (EINVAL);
 		}
 		*cfxp = *fxp;
-		cfxp = (u_int16_t *) ((caddr_t) cfxp + ntmp->ntm_bps);
+		cfxp = (u_int16_t *) ((caddr_t) cfxp + NTFS_BLOCK_SIZE);
 	}
 	return (0);
 }
@@ -2122,48 +2135,50 @@ ntfs_82u_uninit(struct ntfsmount *ntmp)
 }
 
 /*
- * maps the Unicode char to local character
+ * maps the Unicode char to 8bit equivalent
+ * XXX currently only gets lower 8bit from the Unicode char
+ * and substitutes a '_' for it if the result would be '\0';
+ * something better has to be definitely though out
  */
-char *
+wchar
 ntfs_u28(
-	char *outbuf,
 	struct ntfsmount *ntmp, 
 	wchar wc)
 {
-	char *p, *outp, inbuf[3];
+	char *p, *outp, inbuf[3], outbuf[3];
 	size_t ilen, olen;
 
-	outp = outbuf;
 	if (ntfs_iconv && ntmp->ntm_ic_u2l) {
-		ilen = 2;
-		olen = 4;
+		ilen = olen = 2;
 
 		inbuf[0] = (char)(wc>>8);
 		inbuf[1] = (char)wc;
 		inbuf[2] = '\0';
 		p = inbuf;
+		outp = outbuf;
 		ntfs_iconv->convchr(ntmp->ntm_ic_u2l, (const char **)&p, &ilen,
 				    &outp, &olen);
-		if (olen == 4)
-			*outp++ = '?';
-		*outp = '\0';
-		outp = outbuf;
-		return (outp);
+		if (olen == 1) {
+			return ((wchar)(outbuf[0]&0xFF));
+		} else if (olen == 0) {
+			return ((wchar)((outbuf[0]<<8) | (outbuf[1]&0xFF)));
+		}
+		return ('?');
 	}
 
 	p = ntmp->ntm_u28[(wc>>8)&0xFF];
-	outbuf[0] = (p == NULL) ? '_' : p[wc&0xFF] & 0xFF;
-	outbuf[1] = '\0';
-	return (outp);
+	if (p == NULL)
+		return ('_');
+	return (p[wc&0xFF]&0xFF);
 }
 
 wchar
 ntfs_82u(
 	struct ntfsmount *ntmp, 
-	const char *c,
+	wchar wc,
 	int *len)
 {
-	char *outp, outbuf[3];
+	char *p, *outp, inbuf[3], outbuf[3];
 	wchar uc;
 	size_t ilen, olen;
 
@@ -2171,8 +2186,13 @@ ntfs_82u(
 		ilen = (size_t)*len;
 		olen = 2;
 
+		inbuf[0] = (char)(wc>>8);
+		inbuf[1] = (char)wc;
+		inbuf[2] = '\0';
+		p = inbuf;
 		outp = outbuf;
-		ntfs_iconv->convchr(ntmp->ntm_ic_l2u, &c, &ilen, &outp, &olen);
+		ntfs_iconv->convchr(ntmp->ntm_ic_l2u, (const char **)&p, &ilen,
+				    &outp, &olen);
 		*len -= (int)ilen;
 		uc = (wchar)((outbuf[0]<<8) | (outbuf[1]&0xFF));
 
@@ -2180,7 +2200,7 @@ ntfs_82u(
 	}
 
 	if (ntmp->ntm_82u != NULL)
-		return (ntmp->ntm_82u[*c&0xFF]);
+		return (ntmp->ntm_82u[wc&0xFF]);
 
 	return ('?');
 }
