@@ -444,7 +444,10 @@ g_io_request(struct bio *bp, struct g_consumer *cp)
 	    ("Bio already on queue bp=%p", bp));
 	bp->bio_flags |= BIO_ONQUEUE;
 
-	binuptime(&bp->bio_t0);
+	if (g_collectstats)
+		binuptime(&bp->bio_t0);
+	else
+		getbinuptime(&bp->bio_t0);
 
 	/*
 	 * The statistics collection is lockless, as such, but we
@@ -777,19 +780,18 @@ g_print_bio(struct bio *bp)
 		return;
 	case BIO_READ:
 		cmd = "READ";
+		break;
 	case BIO_WRITE:
-		if (cmd == NULL)
-			cmd = "WRITE";
+		cmd = "WRITE";
+		break;
 	case BIO_DELETE:
-		if (cmd == NULL)
-			cmd = "DELETE";
-		printf("%s[%s(offset=%jd, length=%jd)]", pname, cmd,
-		    (intmax_t)bp->bio_offset, (intmax_t)bp->bio_length);
-		return;
+		cmd = "DELETE";
+		break;
 	default:
 		cmd = "UNKNOWN";
 		printf("%s[%s()]", pname, cmd);
 		return;
 	}
-	/* NOTREACHED */
+	printf("%s[%s(offset=%jd, length=%jd)]", pname, cmd,
+	    (intmax_t)bp->bio_offset, (intmax_t)bp->bio_length);
 }
