@@ -109,7 +109,7 @@ g_vfs_strategy(struct bufobj *bo, struct buf *bp)
 	/* G_VALID_CONSUMER(cp); We likely lack topology lock */
 
 	/*
-	 * If the the provider has orphaned us, just return EXIO.
+	 * If the provider has orphaned us, just return EXIO.
 	 */
 	if (cp->provider == NULL) {
 		bp->b_error = ENXIO;
@@ -134,12 +134,10 @@ static void
 g_vfs_orphan(struct g_consumer *cp)
 {
 	struct g_geom *gp;
-	struct bufobj *bo;
 
 	g_topology_assert();
 
 	gp = cp->geom;
-	bo = gp->softc;
 	g_trace(G_T_TOPOLOGY, "g_vfs_orphan(%p(%s))", cp, gp->name);
 	if (cp->acr > 0 || cp->acw > 0 || cp->ace > 0)
 		g_access(cp, -cp->acr, -cp->acw, -cp->ace);
@@ -173,7 +171,7 @@ g_vfs_open(struct vnode *vp, struct g_consumer **cpp, const char *fsname, int wr
 	gp = g_new_geomf(&g_vfs_class, "%s.%s", fsname, pp->name);
 	cp = g_new_consumer(gp);
 	g_attach(cp, pp);
-	error = g_access(cp, 1, wr, 1);
+	error = g_access(cp, 1, wr, wr);
 	if (error) {
 		g_wither_geom(gp, ENXIO);
 		return (error);
