@@ -40,6 +40,7 @@ __FBSDID("$FreeBSD$");
 #include <sys/queue.h>
 #include <sys/sbuf.h>
 #include <sys/systm.h>
+#include <sys/sysctl.h>
 #include <geom/geom.h>
 #include <geom/part/g_part.h>
 
@@ -50,6 +51,8 @@ __FBSDID("$FreeBSD$");
 #define	BOOT2_OFF	(BOOT1_SIZE + LABEL_SIZE)
 #define	BOOT2_SIZE	(BBSIZE - BOOT2_OFF)
 
+FEATURE(geom_part_bsd, "GEOM partitioning class for BSD disklabels");
+
 struct g_part_bsd_table {
 	struct g_part_table	base;
 	u_char			*bbarea;
