@@ -33,6 +33,11 @@
  * $FreeBSD$
  */
 
+#include <machine/_align.h>
+
+#ifndef _I386_INCLUDE_PARAM_H_
+#define	_I386_INCLUDE_PARAM_H_
+
 /*
  * Machine dependent constants for Intel 386.
  */
@@ -49,14 +54,11 @@
 #define _ALIGN(p)	(((unsigned)(p) + _ALIGNBYTES) & ~_ALIGNBYTES)
 #endif
 
-#ifndef _NO_NAMESPACE_POLLUTION
 
 #define __HAVE_ACPI
+#define	__HAVE_PIR
 #define __PCI_REROUTE_INTERRUPT
 
-#ifndef _MACHINE_PARAM_H_
-#define	_MACHINE_PARAM_H_
-
 #ifndef MACHINE
 #define MACHINE		"i386"
 #endif
@@ -66,7 +68,9 @@
 #define MID_MACHINE	MID_I386
 
 #if defined(SMP) || defined(KLD_MODULE)
+#ifndef MAXCPU
 #define MAXCPU		32
+#endif
 #else
 #define MAXCPU		1
 #endif /* SMP || KLD_MODULE */
@@ -151,5 +155,4 @@
 
 #define	pgtok(x)		((x) * (PAGE_SIZE / 1024))
 
-#endif /* !_MACHINE_PARAM_H_ */
-#endif /* !_NO_NAMESPACE_POLLUTION */
+#endif /* !_I386_INCLUDE_PARAM_H_ */
