@@ -32,13 +32,11 @@ __FBSDID("$FreeBSD$");
 #include <sys/systm.h>
 #include <machine/bootinfo.h>
 #include <machine/efi.h>
+#include <machine/md_var.h>
 #include <machine/sal.h>
 #include <vm/vm.h>
 #include <vm/pmap.h>
 
-extern uint64_t ia64_call_efi_physical(uint64_t, uint64_t, uint64_t, uint64_t,
-    uint64_t, uint64_t);
-
 static struct efi_systbl *efi_systbl;
 static struct efi_cfgtbl *efi_cfgtbl;
 static struct efi_rt *efi_runtime;
@@ -96,6 +94,7 @@ efi_boot_finish(void)
 int
 efi_boot_minimal(uint64_t systbl)
 {
+	ia64_efi_f setvirt;
 	struct efi_md *md;
 	efi_status status;
 
@@ -121,18 +120,16 @@ efi_boot_minimal(uint64_t systbl)
 	md = efi_md_first();
 	while (md != NULL) {
 		if (md->md_attr & EFI_MD_ATTR_RT) {
-			if (md->md_attr & EFI_MD_ATTR_WB)
-				md->md_virt =
-				    (void *)IA64_PHYS_TO_RR7(md->md_phys);
-			else if (md->md_attr & EFI_MD_ATTR_UC)
-				md->md_virt = pmap_mapdev(md->md_phys,
-				    md->md_pages * EFI_PAGE_SIZE);
+			md->md_virt = (md->md_attr & EFI_MD_ATTR_WB) ?
+			    (void *)IA64_PHYS_TO_RR7(md->md_phys) :
+			    (void *)IA64_PHYS_TO_RR6(md->md_phys);
 		}
 		md = efi_md_next(md);
 	}
-	status = ia64_call_efi_physical((uint64_t)efi_runtime->rt_setvirtual,
-	    bootinfo.bi_memmap_size, bootinfo.bi_memdesc_size,
-	    bootinfo.bi_memdesc_version, bootinfo.bi_memmap, 0);
+	setvirt = (void *)IA64_PHYS_TO_RR7((u_long)efi_runtime->rt_setvirtual);
+	status = ia64_efi_physical(setvirt, bootinfo->bi_memmap_size,
+	    bootinfo->bi_memdesc_size, bootinfo->bi_memdesc_version,
+	    ia64_tpa(bootinfo->bi_memmap));
 	return ((status < 0) ? EFAULT : 0);
 }
 
@@ -164,20 +161,67 @@ efi_get_time(struct efi_tm *tm)
 struct efi_md *
 efi_md_first(void)
 {
+	struct efi_md *md;
 
-	if (bootinfo.bi_memmap == 0)
+	if (bootinfo->bi_memmap == 0)
 		return (NULL);
-	return ((struct efi_md *)IA64_PHYS_TO_RR7(bootinfo.bi_memmap));
+	md = (struct efi_md *)bootinfo->bi_memmap;
+	return (md);
+}
+
+struct efi_md *
+efi_md_last(void)
+{
+	struct efi_md *md;
+
+	if (bootinfo->bi_memmap == 0)
+		return (NULL);
+	md = (struct efi_md *)(bootinfo->bi_memmap + bootinfo->bi_memmap_size -
+	    bootinfo->bi_memdesc_size);
+	return (md);
 }
 
 struct efi_md *
 efi_md_next(struct efi_md *md)
 {
-	uint64_t plim;
+	struct efi_md *lim;
 
-	plim = IA64_PHYS_TO_RR7(bootinfo.bi_memmap + bootinfo.bi_memmap_size);
-	md = (struct efi_md *)((uintptr_t)md + bootinfo.bi_memdesc_size);
-	return ((md >= (struct efi_md *)plim) ? NULL : md);
+	lim = efi_md_last();
+	md = (struct efi_md *)((uintptr_t)md + bootinfo->bi_memdesc_size);
+	return ((md > lim) ? NULL : md);
+}
+
+struct efi_md *
+efi_md_prev(struct efi_md *md)
+{
+	struct efi_md *lim;
+
+	lim = efi_md_first();
+	md = (struct efi_md *)((uintptr_t)md - bootinfo->bi_memdesc_size);
+	return ((md < lim) ? NULL : md);
+}
+
+struct efi_md *
+efi_md_find(vm_paddr_t pa)
+{
+	static struct efi_md *last = NULL;
+	struct efi_md *md, *p0, *p1;
+
+	md = (last != NULL) ? last : efi_md_first();
+	p1 = p0 = NULL;
+	while (md != NULL && md != p1) {
+		if (pa >= md->md_phys &&
+		    pa < md->md_phys + md->md_pages * EFI_PAGE_SIZE) {
+			last = md;
+			return (md);
+		}
+
+		p1 = p0;
+		p0 = md;
+		md = (pa < md->md_phys) ? efi_md_prev(md) : efi_md_next(md);
+	}
+
+	return (NULL);
 }
 
 void
