@@ -411,9 +411,10 @@ mi_switch(int flags, struct thread *newtd)
 	 */
 	if (kdb_active)
 		kdb_switch();
-	if (flags & SW_VOL)
+	if (flags & SW_VOL) {
 		td->td_ru.ru_nvcsw++;
-	else
+		td->td_swvoltick = ticks;
+	} else
 		td->td_ru.ru_nivcsw++;
 #ifdef SCHED_STATS
 	SCHED_STAT_INC(sched_switch_stats[flags & SW_TYPE_MASK]);
@@ -540,7 +541,7 @@ int
 should_yield(void)
 {
 
-	return (ticks - PCPU_GET(switchticks) >= hogticks);
+	return (ticks - curthread->td_swvoltick >= hogticks);
 }
 
 void
@@ -572,7 +573,7 @@ kern_yield(int prio)
  * General purpose yield system call.
  */
 int
-yield(struct thread *td, struct yield_args *uap)
+sys_yield(struct thread *td, struct yield_args *uap)
 {
 
 	thread_lock(td);
