@@ -32,6 +32,7 @@ __FBSDID("$FreeBSD$");
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/sysctl.h>
 #include <sys/endian.h>
 #include <sys/errno.h>
 #include <sys/mbuf.h>
@@ -40,6 +41,8 @@ __FBSDID("$FreeBSD$");
 
 #include <sys/mchain.h>
 
+FEATURE(libmchain, "mchain library");
+
 MODULE_VERSION(libmchain, 1);
 
 #define MBERROR(format, ...) printf("%s(%d): "format, __func__ , \
@@ -125,49 +128,34 @@ mb_reserve(struct mbchain *mbp, int size)
 }
 
 int
-mb_put_padbyte(struct mbchain *mbp)
-{
-	caddr_t dst;
-	char x = 0;
-
-	dst = mtod(mbp->mb_cur, caddr_t) + mbp->mb_cur->m_len;
-
-	/* only add padding if address is odd */
-	if ((unsigned long)dst & 1)
-		return mb_put_mem(mbp, (caddr_t)&x, 1, MB_MSYSTEM);
-	else
-	return 0;
-}
-
-int
-mb_put_uint8(struct mbchain *mbp, u_int8_t x)
+mb_put_uint8(struct mbchain *mbp, uint8_t x)
 {
 	return mb_put_mem(mbp, (caddr_t)&x, sizeof(x), MB_MSYSTEM);
 }
 
 int
-mb_put_uint16be(struct mbchain *mbp, u_int16_t x)
+mb_put_uint16be(struct mbchain *mbp, uint16_t x)
 {
 	x = htobe16(x);
 	return mb_put_mem(mbp, (caddr_t)&x, sizeof(x), MB_MSYSTEM);
 }
 
 int
-mb_put_uint16le(struct mbchain *mbp, u_int16_t x)
+mb_put_uint16le(struct mbchain *mbp, uint16_t x)
 {
 	x = htole16(x);
 	return mb_put_mem(mbp, (caddr_t)&x, sizeof(x), MB_MSYSTEM);
 }
 
 int
-mb_put_uint32be(struct mbchain *mbp, u_int32_t x)
+mb_put_uint32be(struct mbchain *mbp, uint32_t x)
 {
 	x = htobe32(x);
 	return mb_put_mem(mbp, (caddr_t)&x, sizeof(x), MB_MSYSTEM);
 }
 
 int
-mb_put_uint32le(struct mbchain *mbp, u_int32_t x)
+mb_put_uint32le(struct mbchain *mbp, uint32_t x)
 {
 	x = htole32(x);
 	return mb_put_mem(mbp, (caddr_t)&x, sizeof(x), MB_MSYSTEM);
@@ -367,21 +355,21 @@ md_next_record(struct mdchain *mdp)
 }
 
 int
-md_get_uint8(struct mdchain *mdp, u_int8_t *x)
+md_get_uint8(struct mdchain *mdp, uint8_t *x)
 {
 	return md_get_mem(mdp, x, 1, MB_MINLINE);
 }
 
 int
-md_get_uint16(struct mdchain *mdp, u_int16_t *x)
+md_get_uint16(struct mdchain *mdp, uint16_t *x)
 {
 	return md_get_mem(mdp, (caddr_t)x, 2, MB_MINLINE);
 }
 
 int
-md_get_uint16le(struct mdchain *mdp, u_int16_t *x)
+md_get_uint16le(struct mdchain *mdp, uint16_t *x)
 {
-	u_int16_t v;
+	uint16_t v;
 	int error = md_get_uint16(mdp, &v);
 
 	if (x != NULL)
@@ -390,8 +378,9 @@ md_get_uint16le(struct mdchain *mdp, u_int16_t *x)
 }
 
 int
-md_get_uint16be(struct mdchain *mdp, u_int16_t *x) {
-	u_int16_t v;
+md_get_uint16be(struct mdchain *mdp, uint16_t *x)
+{
+	uint16_t v;
 	int error = md_get_uint16(mdp, &v);
 
 	if (x != NULL)
@@ -400,15 +389,15 @@ md_get_uint16be(struct mdchain *mdp, u_int16_t *x) {
 }
 
 int
-md_get_uint32(struct mdchain *mdp, u_int32_t *x)
+md_get_uint32(struct mdchain *mdp, uint32_t *x)
 {
 	return md_get_mem(mdp, (caddr_t)x, 4, MB_MINLINE);
 }
 
 int
-md_get_uint32be(struct mdchain *mdp, u_int32_t *x)
+md_get_uint32be(struct mdchain *mdp, uint32_t *x)
 {
-	u_int32_t v;
+	uint32_t v;
 	int error;
 
 	error = md_get_uint32(mdp, &v);
@@ -418,9 +407,9 @@ md_get_uint32be(struct mdchain *mdp, u_int32_t *x)
 }
 
 int
-md_get_uint32le(struct mdchain *mdp, u_int32_t *x)
+md_get_uint32le(struct mdchain *mdp, uint32_t *x)
 {
-	u_int32_t v;
+	uint32_t v;
 	int error;
 
 	error = md_get_uint32(mdp, &v);
