@@ -1410,11 +1410,6 @@ m_print(const struct mbuf *m, int maxlen)
 	int pdata;
 	const struct mbuf *m2;
 
-	if (m == NULL) {
-		printf("mbuf: %p\n", m);
-		return;
-	}
-
 	if (m->m_flags & M_PKTHDR)
 		len = m->m_pkthdr.len;
 	else
@@ -2033,7 +2028,7 @@ mbprof_textify(void)
 {
 	int offset;
 	char *c;
-	u_int64_t *p;
+	uint64_t *p;
 	
 
 	p = &mbprof.wasted[0];
