@@ -118,12 +118,16 @@ static device_method_t kr_methods[] = {
 	DEVMETHOD(device_resume,	kr_resume),
 	DEVMETHOD(device_shutdown,	kr_shutdown),
 
+	/* bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
+	DEVMETHOD(bus_driver_added,	bus_generic_driver_added),
+
 	/* MII interface */
 	DEVMETHOD(miibus_readreg,	kr_miibus_readreg),
 	DEVMETHOD(miibus_writereg,	kr_miibus_writereg),
 	DEVMETHOD(miibus_statchg,	kr_miibus_statchg),
 
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 static driver_t kr_driver = {
@@ -922,7 +926,7 @@ kr_ifmedia_upd(struct ifnet *ifp)
 	KR_LOCK(sc);
 	mii = device_get_softc(sc->kr_miibus);
 	LIST_FOREACH(miisc, &mii->mii_phys, mii_list)
-		mii_phy_reset(miisc);
+		PHY_RESET(miisc);
 	error = mii_mediachg(mii);
 	KR_UNLOCK(sc);
 
@@ -941,9 +945,9 @@ kr_ifmedia_sts(struct ifnet *ifp, struct ifmediareq *ifmr)
 	mii = device_get_softc(sc->kr_miibus);
 	KR_LOCK(sc);
 	mii_pollstat(mii);
+	KR_UNLOCK(sc);
 	ifmr->ifm_active = mii->mii_media_active;
 	ifmr->ifm_status = mii->mii_media_status;
-	KR_UNLOCK(sc);
 }
 
 struct kr_dmamap_arg {
