@@ -45,7 +45,6 @@ __FBSDID("$FreeBSD$");
 #include "opt_cputype.h"
 #include "opt_ddb.h"
 #include "opt_md.h"
-#include "opt_msgbuf.h"
 
 #include <sys/param.h>
 #include <sys/proc.h>
@@ -103,9 +102,12 @@ __FBSDID("$FreeBSD$");
 char machine[] = "mips";
 SYSCTL_STRING(_hw, HW_MACHINE, machine, CTLFLAG_RD, machine, 0, "Machine class");
 
-static char cpu_model[30];
+char cpu_model[80];
 SYSCTL_STRING(_hw, HW_MODEL, model, CTLFLAG_RD, cpu_model, 0, "Machine model");
 
+char cpu_board[80];
+SYSCTL_STRING(_hw, OID_AUTO, board, CTLFLAG_RD, cpu_board, 0, "Machine board");
+
 int cold = 1;
 long realmem = 0;
 long Maxmem = 0;
@@ -136,8 +138,8 @@ char pcpu_space[MAXCPU][PAGE_SIZE * 2] \
 
 struct pcpu *pcpup = (struct pcpu *)pcpu_space;
 
-vm_offset_t phys_avail[PHYS_AVAIL_ENTRIES + 2];
-vm_offset_t physmem_desc[PHYS_AVAIL_ENTRIES + 2];
+vm_paddr_t phys_avail[PHYS_AVAIL_ENTRIES + 2];
+vm_paddr_t physmem_desc[PHYS_AVAIL_ENTRIES + 2];
 vm_paddr_t dump_avail[PHYS_AVAIL_ENTRIES + 2];
 
 #ifdef UNIMPLEMENTED
@@ -182,8 +184,8 @@ cpu_startup(void *dummy)
 	if (boothowto & RB_VERBOSE)
 		bootverbose++;
 
-	printf("real memory  = %lu (%luK bytes)\n", ptoa(realmem),
-	    ptoa(realmem) / 1024);
+	printf("real memory  = %ju (%juK bytes)\n", ptoa((uintmax_t)realmem),
+	    ptoa((uintmax_t)realmem) / 1024);
 
 	/*
 	 * Display any holes after the first chunk of extended memory.
@@ -193,20 +195,21 @@ cpu_startup(void *dummy)
 
 		printf("Physical memory chunk(s):\n");
 		for (indx = 0; phys_avail[indx + 1] != 0; indx += 2) {
-			uintptr_t size1 = phys_avail[indx + 1] - phys_avail[indx];
+			vm_paddr_t size1 = phys_avail[indx + 1] - phys_avail[indx];
 
-			printf("0x%08llx - 0x%08llx, %llu bytes (%llu pages)\n",
-			    (unsigned long long)phys_avail[indx],
-			    (unsigned long long)phys_avail[indx + 1] - 1,
-			    (unsigned long long)size1,
-			    (unsigned long long)size1 / PAGE_SIZE);
+			printf("0x%08jx - 0x%08jx, %ju bytes (%ju pages)\n",
+			    (uintmax_t)phys_avail[indx],
+			    (uintmax_t)phys_avail[indx + 1] - 1,
+			    (uintmax_t)size1,
+			    (uintmax_t)size1 / PAGE_SIZE);
 		}
 	}
 
 	vm_ksubmap_init(&kmi);
 
-	printf("avail memory = %lu (%luMB)\n", ptoa(cnt.v_free_count),
-	    ptoa(cnt.v_free_count) / 1048576);
+	printf("avail memory = %ju (%juMB)\n", 
+	    ptoa((uintmax_t)cnt.v_free_count),
+	    ptoa((uintmax_t)cnt.v_free_count) / 1048576);
 	cpu_init_interrupts();
 
 	/*
@@ -308,7 +311,9 @@ mips_proc0_init(void)
 void
 cpu_initclocks(void)
 {
+
 	platform_initclocks();
+	cpu_initclocks_bsp();
 }
 
 struct msgbuf *msgbufp=0;
@@ -333,7 +338,7 @@ mips_vector_init(void)
 	bcopy(MipsTLBMiss, (void *)MIPS_UTLB_MISS_EXC_VEC,
 	      MipsTLBMissEnd - MipsTLBMiss);
 
-#if defined(CPU_CNMIPS) || defined(CPU_RMI)
+#if defined(CPU_CNMIPS) || defined(CPU_RMI) || defined(CPU_NLM)
 /* Fake, but sufficient, for the 32-bit with 64-bit hardware addresses  */
 	bcopy(MipsTLBMiss, (void *)MIPS3_XTLB_MISS_EXC_VEC,
 	      MipsTLBMissEnd - MipsTLBMiss);
@@ -424,8 +429,10 @@ cpu_pcpu_init(struct pcpu *pcpu, int cpuid, size_t size)
 	pcpu->pc_next_asid = 1;
 	pcpu->pc_asid_generation = 1;
 #ifdef SMP
-	if ((vm_offset_t)pcpup >= VM_MIN_KERNEL_ADDRESS)
+	if ((vm_offset_t)pcpup >= VM_MIN_KERNEL_ADDRESS &&
+	    (vm_offset_t)pcpup <= VM_MAX_KERNEL_ADDRESS) {
 		mips_pcpu_tlb_init(pcpu);
+	}
 #endif
 }
 
@@ -449,11 +456,15 @@ void
 spinlock_enter(void)
 {
 	struct thread *td;
+	register_t intr;
 
 	td = curthread;
-	if (td->td_md.md_spinlock_count == 0)
-		td->td_md.md_saved_intr = intr_disable();
-	td->td_md.md_spinlock_count++;
+	if (td->td_md.md_spinlock_count == 0) {
+		intr = intr_disable();
+		td->td_md.md_spinlock_count = 1;
+		td->td_md.md_saved_intr = intr;
+	} else
+		td->td_md.md_spinlock_count++;
 	critical_enter();
 }
 
@@ -461,12 +472,14 @@ void
 spinlock_exit(void)
 {
 	struct thread *td;
+	register_t intr;
 
 	td = curthread;
 	critical_exit();
+	intr = td->td_md.md_saved_intr;
 	td->td_md.md_spinlock_count--;
 	if (td->td_md.md_spinlock_count == 0)
-		intr_restore(td->td_md.md_saved_intr);
+		intr_restore(intr);
 }
 
 /*
@@ -475,10 +488,20 @@ spinlock_exit(void)
 void
 cpu_idle(int busy)
 {
-	if (mips_rd_status() & MIPS_SR_INT_IE)
-		__asm __volatile ("wait");
-	else
-		panic("ints disabled in idleproc!");
+	KASSERT((mips_rd_status() & MIPS_SR_INT_IE) != 0,
+		("interrupts disabled in idle process."));
+	KASSERT((mips_rd_status() & MIPS_INT_MASK) != 0,
+		("all interrupts masked in idle process."));
+
+	if (!busy) {
+		critical_enter();
+		cpu_idleclock();
+	}
+	__asm __volatile ("wait");
+	if (!busy) {
+		cpu_activeclock();
+		critical_exit();
+	}
 }
 
 int
@@ -489,12 +512,12 @@ cpu_idle_wakeup(int cpu)
 }
 
 int
-is_cacheable_mem(vm_offset_t addr)
+is_cacheable_mem(vm_paddr_t pa)
 {
 	int i;
 
 	for (i = 0; physmem_desc[i + 1] != 0; i += 2) {
-		if (addr >= physmem_desc[i] && addr < physmem_desc[i + 1])
+		if (pa >= physmem_desc[i] && pa < physmem_desc[i + 1])
 			return (1);
 	}
 
