@@ -149,7 +149,8 @@ memrw(struct cdev *dev, struct uio *uio, int flags)
  * instead of going through read/write
  */
 int
-memmmap(struct cdev *dev, vm_offset_t offset, vm_paddr_t *paddr, int prot)
+memmmap(struct cdev *dev, vm_ooffset_t offset, vm_paddr_t *paddr,
+    int prot, vm_memattr_t *memattr)
 {
 	/*
 	 * /dev/mem is the only one that makes sense through this
@@ -164,8 +165,3 @@ memmmap(struct cdev *dev, vm_offset_t offset, vm_paddr_t *paddr, int prot)
 
 	return (0);
 }
-
-void
-dev_mem_md_init(void)
-{
-}
