@@ -82,6 +82,7 @@ __FBSDID("$FreeBSD$");
 #define	NSFBUFS		(512 + maxusers * 16)
 #endif
 
+#ifndef __mips_n64
 static void	sf_buf_init(void *arg);
 SYSINIT(sock_sf, SI_SUB_MBUF, SI_ORDER_ANY, sf_buf_init, NULL);
 
@@ -95,6 +96,7 @@ static struct {
 } sf_freelist;
 
 static u_int	sf_buf_alloc_want;
+#endif
 
 /*
  * Finish a fork operation, with process p2 nearly set up.
@@ -458,6 +460,7 @@ kvtop(void *addr)
 /*
  * Allocate a pool of sf_bufs (sendfile(2) or "super-fast" if you prefer. :-))
  */
+#ifndef __mips_n64
 static void
 sf_buf_init(void *arg)
 {
@@ -479,6 +482,7 @@ sf_buf_init(void *arg)
 	}
 	sf_buf_alloc_want = 0;
 }
+#endif
 
 /*
  * Get an sf_buf from the freelist.  Will block if none are available.
@@ -486,6 +490,7 @@ sf_buf_init(void *arg)
 struct sf_buf *
 sf_buf_alloc(struct vm_page *m, int flags)
 {
+#ifndef __mips_n64
 	struct sf_buf *sf;
 	int error;
 
@@ -514,6 +519,9 @@ sf_buf_alloc(struct vm_page *m, int flags)
 	}
 	mtx_unlock(&sf_freelist.sf_lock);
 	return (sf);
+#else
+	return ((struct sf_buf *)m);
+#endif
 }
 
 /*
@@ -522,14 +530,15 @@ sf_buf_alloc(struct vm_page *m, int flags)
 void
 sf_buf_free(struct sf_buf *sf)
 {
-
+#ifndef __mips_n64
 	pmap_qremove(sf->kva, 1);
 	mtx_lock(&sf_freelist.sf_lock);
 	SLIST_INSERT_HEAD(&sf_freelist.sf_head, sf, free_list);
 	nsfbufsused--;
 	if (sf_buf_alloc_want > 0)
-		wakeup_one(&sf_freelist);
+		wakeup(&sf_freelist);
 	mtx_unlock(&sf_freelist.sf_lock);
+#endif
 }
 
 /*
@@ -538,6 +547,9 @@ sf_buf_free(struct sf_buf *sf)
 void
 swi_vm(void *dummy)
 {
+
+	if (busdma_swi_pending)
+		busdma_swi();
 }
 
 int
