@@ -105,13 +105,6 @@ __FBSDID("$FreeBSD$");
         (MSI_MIPS_DATA_TRGRLVL | MSI_MIPS_DATA_DELFIXED |	       \
 	 MSI_MIPS_DATA_ASSERT | (irq))
 
-#define DEBUG
-#ifdef DEBUG
-#define dbg_devprintf	device_printf
-#else
-#define dbg_devprintf(dev, fmt, ...)
-#endif
-
 struct xlr_pcib_softc {
 	bus_dma_tag_t	sc_pci_dmat;	/* PCI DMA tag pointer */
 };
@@ -433,7 +426,7 @@ static void
 bridge_pcix_ack(int irq)
 {
 
-	xlr_read_reg(xlr_io_mmio(XLR_IO_PCIX_OFFSET), 0x140 >> 2);
+	(void)xlr_read_reg(xlr_io_mmio(XLR_IO_PCIX_OFFSET), 0x140 >> 2);
 }
 
 static void
@@ -634,6 +627,7 @@ static device_method_t xlr_pcib_methods[] = {
 	DEVMETHOD(device_attach, xlr_pcib_attach),
 
 	/* Bus interface */
+	DEVMETHOD(bus_print_child, bus_generic_print_child),
 	DEVMETHOD(bus_read_ivar, xlr_pcib_read_ivar),
 	DEVMETHOD(bus_write_ivar, xlr_pcib_write_ivar),
 	DEVMETHOD(bus_alloc_resource, xlr_pci_alloc_resource),
@@ -654,7 +648,7 @@ static device_method_t xlr_pcib_methods[] = {
 	DEVMETHOD(pcib_release_msi, xlr_release_msi),
 	DEVMETHOD(pcib_map_msi, xlr_map_msi),
 
-	DEVMETHOD_END
+	{0, 0}
 };
 
 static driver_t xlr_pcib_driver = {
