@@ -14,13 +14,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *        This product includes software developed by the NetBSD
- *        Foundation, Inc. and its contributors.
- * 4. Neither the name of The NetBSD Foundation nor the names of its
- *    contributors may be used to endorse or promote products derived
- *    from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -76,8 +69,50 @@ __FBSDID("$FreeBSD$");
 #include <dev/usb/controller/ehcireg.h>
 #include <mips/rmi/pic.h>
 
-static device_attach_t ehci_xls_attach;
-static device_detach_t ehci_xls_detach;
+static int ehci_xls_attach(device_t self);
+static int ehci_xls_detach(device_t self);
+static int ehci_xls_shutdown(device_t self);
+static int ehci_xls_suspend(device_t self);
+static int ehci_xls_resume(device_t self);
+
+static int
+ehci_xls_suspend(device_t self)
+{
+	ehci_softc_t *sc = device_get_softc(self);
+	int err;
+
+	err = bus_generic_suspend(self);
+	if (err)
+		return (err);
+	ehci_suspend(sc);
+	return (0);
+}
+
+static int
+ehci_xls_resume(device_t self)
+{
+	ehci_softc_t *sc = device_get_softc(self);
+
+	ehci_resume(sc);
+
+	bus_generic_resume(self);
+
+	return (0);
+}
+
+static int
+ehci_xls_shutdown(device_t self)
+{
+	ehci_softc_t *sc = device_get_softc(self);
+	int err;
+
+	err = bus_generic_shutdown(self);
+	if (err)
+		return (err);
+	ehci_shutdown(sc);
+
+	return (0);
+}
 
 static const char *xlr_usb_dev_desc = "RMI XLR USB 2.0 controller";
 static const char *xlr_vendor_desc = "RMI Corp";
@@ -213,17 +248,20 @@ static device_method_t ehci_methods[] = {
 	DEVMETHOD(device_probe, ehci_xls_probe),
 	DEVMETHOD(device_attach, ehci_xls_attach),
 	DEVMETHOD(device_detach, ehci_xls_detach),
-	DEVMETHOD(device_suspend, bus_generic_suspend),
-	DEVMETHOD(device_resume, bus_generic_resume),
-	DEVMETHOD(device_shutdown, bus_generic_shutdown),
+	DEVMETHOD(device_suspend, ehci_xls_suspend),
+	DEVMETHOD(device_resume, ehci_xls_resume),
+	DEVMETHOD(device_shutdown, ehci_xls_shutdown),
 
-	DEVMETHOD_END
+	/* Bus interface */
+	DEVMETHOD(bus_print_child, bus_generic_print_child),
+
+	{0, 0}
 };
 
 static driver_t ehci_driver = {
-	.name = "ehci",
-	.methods = ehci_methods,
-	.size = sizeof(struct ehci_softc),
+	"ehci",
+	ehci_methods,
+	sizeof(struct ehci_softc),
 };
 
 static devclass_t ehci_devclass;
