@@ -199,6 +199,13 @@ struct if_data {
  *   field.  IFCAP_* and CSUM_* do not match one to one and CSUM_* may be
  *   more detailed or differenciated than IFCAP_*.
  *   Hwassist features are defined CSUM_* in sys/mbuf.h
+ *
+ * Capabilities that cannot be arbitrarily changed with ifconfig/ioctl
+ * are listed in IFCAP_CANTCHANGE, similar to IFF_CANTCHANGE.
+ * This is not strictly necessary because the common code never
+ * changes capabilities, and it is left to the individual driver
+ * to do the right thing. However, having the filter here
+ * avoids replication of the same code in all individual drivers.
  */
 #define	IFCAP_RXCSUM		0x00001  /* can offload checksum on RX */
 #define	IFCAP_TXCSUM		0x00002  /* can offload checksum on TX */
@@ -227,13 +234,14 @@ struct if_data {
 #define	IFCAP_WOL	(IFCAP_WOL_UCAST | IFCAP_WOL_MCAST | IFCAP_WOL_MAGIC)
 #define	IFCAP_TOE	(IFCAP_TOE4 | IFCAP_TOE6)
 
+#define	IFCAP_CANTCHANGE	(IFCAP_NETMAP)
+
 #define	IFQ_MAXLEN	50
 #define	IFNET_SLOWHZ	1		/* granularity is 1 second */
 
 /*
  * Message format for use in obtaining information about interfaces
  * from getkerninfo and the routing socket
- * For the new, extensible interface see struct if_msghdrl below.
  */
 struct if_msghdr {
 	u_short	ifm_msglen;	/* to skip over non-understood messages */
@@ -245,35 +253,9 @@ struct if_msghdr {
 	struct	if_data ifm_data;/* statistics and other data about if */
 };
 
-/*
- * The 'l' version shall be used by new interfaces, like NET_RT_IFLISTL.  It is
- * extensible after ifm_data_off or within ifm_data.  Both the if_msghdr and
- * if_data now have a member field detailing the struct length in addition to
- * the routing message length.  Macros are provided to find the start of
- * ifm_data and the start of the socket address strucutres immediately following
- * struct if_msghdrl given a pointer to struct if_msghdrl.
- */
-#define	IF_MSGHDRL_IFM_DATA(_l) \
-    (struct if_data *)((char *)(_l) + (_l)->ifm_data_off)
-#define	IF_MSGHDRL_RTA(_l) \
-    (void *)((uintptr_t)(_l) + (_l)->ifm_len)
-struct if_msghdrl {
-	u_short	ifm_msglen;	/* to skip over non-understood messages */
-	u_char	ifm_version;	/* future binary compatibility */
-	u_char	ifm_type;	/* message type */
-	int	ifm_addrs;	/* like rtm_addrs */
-	int	ifm_flags;	/* value of if_flags */
-	u_short	ifm_index;	/* index for associated ifp */
-	u_short _ifm_spare1;	/* spare space to grow if_index, see if_var.h */
-	u_short	ifm_len;	/* length of if_msghdrl incl. if_data */
-	u_short	ifm_data_off;	/* offset of if_data from beginning */
-	struct	if_data ifm_data;/* statistics and other data about if */
-};
-
 /*
  * Message format for use in obtaining information about interface addresses
  * from getkerninfo and the routing socket
- * For the new, extensible interface see struct ifa_msghdrl below.
  */
 struct ifa_msghdr {
 	u_short	ifam_msglen;	/* to skip over non-understood messages */
@@ -285,33 +267,6 @@ struct ifa_msghdr {
 	int	ifam_metric;	/* value of ifa_metric */
 };
 
-/*
- * The 'l' version shall be used by new interfaces, like NET_RT_IFLISTL.  It is
- * extensible after ifam_metric or within ifam_data.  Both the ifa_msghdrl and
- * if_data now have a member field detailing the struct length in addition to
- * the routing message length.  Macros are provided to find the start of
- * ifm_data and the start of the socket address strucutres immediately following
- * struct ifa_msghdrl given a pointer to struct ifa_msghdrl.
- */
-#define	IFA_MSGHDRL_IFAM_DATA(_l) \
-    (struct if_data *)((char *)(_l) + (_l)->ifam_data_off)
-#define	IFA_MSGHDRL_RTA(_l) \
-    (void *)((uintptr_t)(_l) + (_l)->ifam_len)
-struct ifa_msghdrl {
-	u_short	ifam_msglen;	/* to skip over non-understood messages */
-	u_char	ifam_version;	/* future binary compatibility */
-	u_char	ifam_type;	/* message type */
-	int	ifam_addrs;	/* like rtm_addrs */
-	int	ifam_flags;	/* value of ifa_flags */
-	u_short	ifam_index;	/* index for associated ifp */
-	u_short _ifam_spare1;	/* spare space to grow if_index, see if_var.h */
-	u_short	ifam_len;	/* length of ifa_msghdrl incl. if_data */
-	u_short	ifam_data_off;	/* offset of if_data from beginning */
-	int	ifam_metric;	/* value of ifa_metric */
-	struct	if_data ifam_data;/* statistics and other data about if or
-				 * address */
-};
-
 /*
  * Message format for use in obtaining information about multicast addresses
  * from the routing socket
