@@ -147,6 +147,16 @@ typedef struct mtx acl_lock_t;
 #define	ACL_LOCK_ASSERT(_as) \
 	mtx_assert((&(_as)->as_lock), MA_OWNED)
 
+/*
+ * Scan table definitions.
+ */
+typedef struct mtx ieee80211_scan_table_lock_t;
+#define	IEEE80211_SCAN_TABLE_LOCK_INIT(_st, _name) \
+	mtx_init(&(_st)->st_lock, _name, "802.11 scan table", MTX_DEF)
+#define	IEEE80211_SCAN_TABLE_LOCK_DESTROY(_st)	mtx_destroy(&(_st)->st_lock)
+#define	IEEE80211_SCAN_TABLE_LOCK(_st)		mtx_lock(&(_st)->st_lock)
+#define	IEEE80211_SCAN_TABLE_UNLOCK(_st)	mtx_unlock(&(_st)->st_lock)
+
 /*
  * Node reference counting definitions.
  *
@@ -386,14 +396,19 @@ TEXT_SET(auth_set, name##_modevent)
 /*
  * Rate control modules provide tx rate control support.
  */
-#define	IEEE80211_RATE_MODULE(alg, version)				\
-_IEEE80211_POLICY_MODULE(rate, alg, version);				\
+#define	IEEE80211_RATECTL_MODULE(alg, version)				\
+	_IEEE80211_POLICY_MODULE(ratectl, alg, version);		\
+
+#define	IEEE80211_RATECTL_ALG(name, alg, v)				\
 static void								\
 alg##_modevent(int type)						\
 {									\
-	/* XXX nothing to do until the rate control framework arrives */\
+	if (type == MOD_LOAD)						\
+		ieee80211_ratectl_register(alg, &v);			\
+	else								\
+		ieee80211_ratectl_unregister(alg);			\
 }									\
-TEXT_SET(rate##_set, alg##_modevent)
+TEXT_SET(ratectl##_set, alg##_modevent)
 
 struct ieee80211req;
 typedef int ieee80211_ioctl_getfunc(struct ieee80211vap *,
