@@ -57,7 +57,6 @@
 
 #ifdef HAVE_KERNEL_OPTION_HEADERS
 #include "opt_netgraph.h"
-#include "opt_kdb.h"
 #endif
 
 /* debugging options */
@@ -191,7 +190,7 @@ static __inline void
 _chkhook(hook_p hook, char *file, int line)
 {
 	if (hook->hk_magic != HK_MAGIC) {
-		printf("Accessing freed ");
+		printf("Accessing freed hook ");
 		dumphook(hook, file, line);
 	}
 	hook->lastline = line;
@@ -398,7 +397,7 @@ struct ng_node {
  * Public methods for nodes.
  * If you can't do it with these you probably shouldn't be doing it.
  */
-int	ng_unref_node(node_p node); /* don't move this */
+void	ng_unref_node(node_p node); /* don't move this */
 #define _NG_NODE_NAME(node)	((node)->nd_name + 0)
 #define _NG_NODE_HAS_NAME(node)	((node)->nd_name[0] + 0)
 #define _NG_NODE_ID(node)	((node)->nd_ID + 0)
@@ -443,7 +442,7 @@ static __inline char * _ng_node_name(node_p node, char *file, int line);
 static __inline int _ng_node_has_name(node_p node, char *file, int line);
 static __inline ng_ID_t _ng_node_id(node_p node, char *file, int line);
 static __inline void _ng_node_ref(node_p node, char *file, int line);
-static __inline int _ng_node_unref(node_p node, char *file, int line);
+static __inline void _ng_node_unref(node_p node, char *file, int line);
 static __inline void _ng_node_set_private(node_p node, void * val,
 							char *file, int line);
 static __inline void * _ng_node_private(node_p node, char *file, int line);
@@ -459,7 +458,7 @@ static __inline void
 _chknode(node_p node, char *file, int line)
 {
 	if (node->nd_magic != ND_MAGIC) {
-		printf("Accessing freed ");
+		printf("Accessing freed node ");
 		dumpnode(node, file, line);
 	}
 	node->lastline = line;
@@ -494,11 +493,11 @@ _ng_node_ref(node_p node, char *file, int line)
 	_NG_NODE_REF(node);
 }
 
-static __inline int
+static __inline void
 _ng_node_unref(node_p node, char *file, int line)
 {
 	_chknode(node, file, line);
-	return (_NG_NODE_UNREF(node));
+	_NG_NODE_UNREF(node);
 }
 
 static __inline void
