@@ -75,33 +75,33 @@
 #define KEEPALIVE_SECS		10
 
 struct cisco_header {
-	uint8_t  address;
-	uint8_t  control;
-	uint16_t protocol;
-} __packed;
+	u_char  address;
+	u_char  control;
+	u_short protocol;
+};
 
 #define	CISCO_HEADER_LEN	sizeof (struct cisco_header)
 
 struct cisco_packet {
-	uint32_t type;
-	uint32_t par1;
-	uint32_t par2;
-	uint16_t rel;
-	uint16_t time0;
-	uint16_t time1;
-} __packed;
+	u_long  type;
+	u_long  par1;
+	u_long  par2;
+	u_short rel;
+	u_short time0;
+	u_short time1;
+};
 
 #define	CISCO_PACKET_LEN (sizeof(struct cisco_packet))
 
 struct protoent {
 	hook_p  hook;		/* the hook for this proto */
-	uint16_t af;		/* address family, -1 = downstream */
+	u_short af;		/* address family, -1 = downstream */
 };
 
 struct cisco_priv {
-	uint32_t local_seq;
-	uint32_t remote_seq;
-	uint32_t seqRetries;	/* how many times we've been here throwing out
+	u_long  local_seq;
+	u_long  remote_seq;
+	u_long  seqRetries;	/* how many times we've been here throwing out
 				 * the same sequence number without ack */
 	node_p  node;
 	struct callout handle;
@@ -193,9 +193,7 @@ cisco_constructor(node_p node)
 {
 	sc_p sc;
 
-	sc = malloc(sizeof(*sc), M_NETGRAPH, M_NOWAIT | M_ZERO);
-	if (sc == NULL)
-		return (ENOMEM);
+	sc = malloc(sizeof(*sc), M_NETGRAPH, M_WAITOK | M_ZERO);
 
 	ng_callout_init(&sc->handle);
 	NG_NODE_SET_PRIVATE(node, sc);
@@ -273,7 +271,7 @@ cisco_rcvmsg(node_p node, item_p item, hook_p lasthook)
 			pos = sprintf(arg,
 			  "keepalive period: %d sec; ", KEEPALIVE_SECS);
 			pos += sprintf(arg + pos,
-			  "unacknowledged keepalives: %d", sc->seqRetries);
+			  "unacknowledged keepalives: %ld", sc->seqRetries);
 			resp->header.arglen = pos + 1;
 			break;
 		    }
@@ -604,7 +602,7 @@ cisco_send(sc_p sc, int type, long par1, long par2)
 	struct cisco_packet *ch;
 	struct mbuf *m;
 	struct timeval time;
-	uint32_t t;
+	u_long  t;
 	int     error = 0;
 
 	getmicrouptime(&time);
@@ -627,8 +625,8 @@ cisco_send(sc_p sc, int type, long par1, long par2)
 	ch->par1 = htonl(par1);
 	ch->par2 = htonl(par2);
 	ch->rel = -1;
-	ch->time0 = htons((uint16_t) (t >> 16));
-	ch->time1 = htons((uint16_t) t);
+	ch->time0 = htons((u_short) (t >> 16));
+	ch->time1 = htons((u_short) t);
 
 	NG_SEND_DATA_ONLY(error, sc->downstream.hook, m);
 	return (error);
