@@ -604,9 +604,6 @@ ng_ether_rcvdata(hook_p hook, item_p item)
 	NG_FREE_ITEM(item);
 
 	panic("%s: weird hook", __func__);
-#ifdef RESTARTABLE_PANICS /* so we don't get an error msg in LINT */
-	return (0);
-#endif
 }
 
 /*
@@ -720,7 +717,6 @@ ng_ether_shutdown(node_p node)
 		(void)ifpromisc(priv->ifp, 0);
 		priv->promisc = 0;
 	}
-	priv->autoSrcAddr = 1;		/* reset auto-src-addr flag */
 	NG_NODE_REVIVE(node);		/* Signal ng_rmnode we are persisant */
 
 	return (0);
