@@ -51,7 +51,6 @@
 
 #include <sys/param.h>
 #include <sys/domain.h>
-#include <sys/hash.h>
 #include <sys/kernel.h>
 #include <sys/linker.h>
 #include <sys/lock.h>
@@ -65,6 +64,9 @@
 #include <sys/socketvar.h>
 #include <sys/syscallsubr.h>
 #include <sys/sysctl.h>
+#ifdef NOTYET
+#include <sys/vnode.h>
+#endif
 
 #include <net/vnet.h>
 
@@ -113,7 +115,6 @@ static ng_rcvmsg_t	ngs_rcvmsg;
 static ng_shutdown_t	ngs_shutdown;
 static ng_newhook_t	ngs_newhook;
 static ng_connect_t	ngs_connect;
-static ng_findhook_t	ngs_findhook;
 static ng_rcvdata_t	ngs_rcvdata;
 static ng_disconnect_t	ngs_disconnect;
 
@@ -123,6 +124,9 @@ static int	ng_attach_cntl(struct socket *so);
 static int	ng_attach_common(struct socket *so, int type);
 static void	ng_detach_common(struct ngpcb *pcbp, int type);
 static void	ng_socket_free_priv(struct ngsock *priv);
+#ifdef NOTYET
+static int	ng_internalize(struct mbuf *m, struct thread *p);
+#endif
 static int	ng_connect_data(struct sockaddr *nam, struct ngpcb *pcbp);
 static int	ng_bind(struct sockaddr *nam, struct ngpcb *pcbp);
 
@@ -139,7 +143,6 @@ static struct ng_type typestruct = {
 	.shutdown =	ngs_shutdown,
 	.newhook =	ngs_newhook,
 	.connect =	ngs_connect,
-	.findhook =	ngs_findhook,
 	.rcvdata =	ngs_rcvdata,
 	.disconnect =	ngs_disconnect,
 };
@@ -147,10 +150,10 @@ NETGRAPH_INIT_ORDERED(socket, &typestruct, SI_SUB_PROTO_DOMAIN, SI_ORDER_ANY);
 
 /* Buffer space */
 static u_long ngpdg_sendspace = 20 * 1024;	/* really max datagram size */
-SYSCTL_INT(_net_graph, OID_AUTO, maxdgram, CTLFLAG_RW,
+SYSCTL_ULONG(_net_graph, OID_AUTO, maxdgram, CTLFLAG_RW,
     &ngpdg_sendspace , 0, "Maximum outgoing Netgraph datagram size");
 static u_long ngpdg_recvspace = 20 * 1024;
-SYSCTL_INT(_net_graph, OID_AUTO, recvspace, CTLFLAG_RW,
+SYSCTL_ULONG(_net_graph, OID_AUTO, recvspace, CTLFLAG_RW,
     &ngpdg_recvspace , 0, "Maximum space for incoming Netgraph datagrams");
 
 /* List of all sockets (for netstat -f netgraph) */
@@ -206,10 +209,19 @@ ngc_send(struct socket *so, int flags, struct mbuf *m, struct sockaddr *addr,
 	int len, error = 0;
 	struct ng_apply_info apply;
 
+#ifdef	NOTYET
+	if (control && (error = ng_internalize(control, td))) {
+		if (pcbp->sockdata == NULL) {
+			error = ENOTCONN;
+			goto release;
+		}
+	}
+#else	/* NOTYET */
 	if (control) {
 		error = EINVAL;
 		goto release;
 	}
+#endif	/* NOTYET */
 
 	/* Require destination as there may be >= 1 hooks on this node. */
 	if (addr == NULL) {
@@ -527,14 +539,8 @@ ng_attach_cntl(struct socket *so)
 		return (error);
 	}
 
-	/*
-	 * Allocate node private info and hash. We start
-	 * with 16 hash entries, however we may grow later
-	 * in ngs_newhook(). We can't predict how much hooks
-	 * does this node plan to have.
-	 */
+	/* Allocate node private info */
 	priv = malloc(sizeof(*priv), M_NETGRAPH_SOCK, M_WAITOK | M_ZERO);
-	priv->hash = hashinit(16, M_NETGRAPH_SOCK, &priv->hmask);
 
 	/* Initialize mutex. */
 	mtx_init(&priv->mtx, "ng_socket", NULL, MTX_DEF);
@@ -614,7 +620,7 @@ ng_detach_common(struct ngpcb *pcbp, int which)
 			priv->datasock = NULL;
 			break;
 		default:
-			panic(__func__);
+			panic("%s", __func__);
 		}
 		pcbp->sockdata = NULL;
 
@@ -640,7 +646,6 @@ ng_socket_free_priv(struct ngsock *priv)
 
 	if (priv->refs == 0) {
 		mtx_destroy(&priv->mtx);
-		hashdestroy(priv->hash, M_NETGRAPH_SOCK, priv->hmask);
 		free(priv, M_NETGRAPH_SOCK);
 		return;
 	}
@@ -656,6 +661,69 @@ ng_socket_free_priv(struct ngsock *priv)
 		mtx_unlock(&priv->mtx);
 }
 
+#ifdef NOTYET
+/*
+ * File descriptors can be passed into an AF_NETGRAPH socket.
+ * Note, that file descriptors cannot be passed OUT.
+ * Only character device descriptors are accepted.
+ * Character devices are useful to connect a graph to a device,
+ * which after all is the purpose of this whole system.
+ */
+static int
+ng_internalize(struct mbuf *control, struct thread *td)
+{
+	const struct cmsghdr *cm = mtod(control, const struct cmsghdr *);
+	struct file *fp;
+	struct vnode *vn;
+	int oldfds;
+	int fd;
+
+	if (cm->cmsg_type != SCM_RIGHTS || cm->cmsg_level != SOL_SOCKET ||
+	    cm->cmsg_len != control->m_len) {
+		TRAP_ERROR;
+		return (EINVAL);
+	}
+
+	/* Check there is only one FD. XXX what would more than one signify? */
+	oldfds = ((caddr_t)cm + cm->cmsg_len - (caddr_t)data) / sizeof (int);
+	if (oldfds != 1) {
+		TRAP_ERROR;
+		return (EINVAL);
+	}
+
+	/* Check that the FD given is legit. and change it to a pointer to a
+	 * struct file. */
+	fd = CMSG_DATA(cm);
+	if ((error = fget(td, fd, 0, &fp)) != 0)
+		return (error);
+
+	/* Depending on what kind of resource it is, act differently. For
+	 * devices, we treat it as a file. For an AF_NETGRAPH socket,
+	 * shortcut straight to the node. */
+	switch (fp->f_type) {
+	case DTYPE_VNODE:
+		vn = fp->f_data;
+		if (vn && (vn->v_type == VCHR)) {
+			/* for a VCHR, actually reference the FILE */
+			fhold(fp);
+			/* XXX then what :) */
+			/* how to pass on to other modules? */
+		} else {
+			fdrop(fp, td);
+			TRAP_ERROR;
+			return (EINVAL);
+		}
+		break;
+	default:
+		fdrop(fp, td);
+		TRAP_ERROR;
+		return (EINVAL);
+	}
+	fdrop(fp, td);
+	return (0);
+}
+#endif	/* NOTYET */
+
 /*
  * Connect the data socket to a named control socket node.
  */
@@ -749,35 +817,6 @@ ngs_constructor(node_p nodep)
 	return (EINVAL);
 }
 
-static void
-ngs_rehash(node_p node)
-{
-	struct ngsock *priv = NG_NODE_PRIVATE(node);
-	struct ngshash *new;
-	struct hookpriv *hp;
-	hook_p hook;
-	uint32_t h;
-	u_long hmask;
-
-	new = hashinit_flags((priv->hmask + 1) * 2, M_NETGRAPH_SOCK, &hmask,
-	    HASH_NOWAIT);
-	if (new == NULL)
-		return;
-
-	LIST_FOREACH(hook, &node->nd_hooks, hk_hooks) {
-		hp = NG_HOOK_PRIVATE(hook);
-#ifdef INVARIANTS
-		LIST_REMOVE(hp, next);
-#endif
-		h = hash32_str(NG_HOOK_NAME(hook), HASHINIT) & hmask;
-		LIST_INSERT_HEAD(&new[h], hp, next);
-	}
-
-	hashdestroy(priv->hash, M_NETGRAPH_SOCK, priv->hmask);
-	priv->hash = new;
-	priv->hmask = hmask;
-}
-
 /*
  * We allow any hook to be connected to the node.
  * There is no per-hook private information though.
@@ -785,20 +824,7 @@ ngs_rehash(node_p node)
 static int
 ngs_newhook(node_p node, hook_p hook, const char *name)
 {
-	struct ngsock *const priv = NG_NODE_PRIVATE(node);
-	struct hookpriv *hp;
-	uint32_t h;
-
-	hp = malloc(sizeof(*hp), M_NETGRAPH_SOCK, M_NOWAIT);
-	if (hp == NULL)
-		return (ENOMEM);
-	if (node->nd_numhooks * 2 > priv->hmask)
-		ngs_rehash(node);
-	hp->hook = hook;
-	h = hash32_str(name, HASHINIT) & priv->hmask;
-	LIST_INSERT_HEAD(&priv->hash[h], hp, next);
-	NG_HOOK_SET_PRIVATE(hook, hp);
-
+	NG_HOOK_SET_PRIVATE(hook, NG_NODE_PRIVATE(node));
 	return (0);
 }
 
@@ -820,38 +846,6 @@ ngs_connect(hook_p hook)
 	return (0);
 }
 
-/* Look up hook by name */
-static hook_p
-ngs_findhook(node_p node, const char *name)
-{
-	struct ngsock *priv = NG_NODE_PRIVATE(node);
-	struct hookpriv *hp;
-	uint32_t h;
-
-	/*
-	 * Microoptimisation for an ng_socket with
-	 * a single hook, which is a common case.
-	 */
-	if (node->nd_numhooks == 1) {
-		hook_p hook;
-
-		hook = LIST_FIRST(&node->nd_hooks);
-
-		if (strcmp(NG_HOOK_NAME(hook), name) == 0)
-			return (hook);
-		else
-			return (NULL);
-	}
-
-	h = hash32_str(name, HASHINIT) & priv->hmask;
-
-	LIST_FOREACH(hp, &priv->hash[h], next)
-		if (strcmp(NG_HOOK_NAME(hp->hook), name) == 0)
-			return (hp->hook);
-
-	return (NULL);
-}
-
 /*
  * Incoming messages get passed up to the control socket.
  * Unless they are for us specifically (socket_type)
@@ -1019,10 +1013,6 @@ ngs_disconnect(hook_p hook)
 {
 	node_p node = NG_HOOK_NODE(hook);
 	struct ngsock *const priv = NG_NODE_PRIVATE(node);
-	struct hookpriv *hp = NG_HOOK_PRIVATE(hook);
-
-	LIST_REMOVE(hp, next);
-	free(hp, M_NETGRAPH_SOCK);
 
 	if ((priv->datasock) && (priv->datasock->ng_socket)) {
 		if (NG_NODE_NUMHOOKS(node) == 1)
