@@ -10,12 +10,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Boris Popov.
- * 4. Neither the name of the author nor the names of any co-contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -248,10 +242,6 @@ struct smb_vc {
 	void *		vc_toupper;	/* local charset */
 	void *		vc_toserver;	/* local charset to server one */
 	void *		vc_tolocal;	/* server charset to local one */
-	void *		vc_cp_toserver;	/* local charset to server one (using CodePage) */
-	void *		vc_cp_tolocal;	/* server charset to local one (using CodePage) */
-	void *		vc_ucs_toserver; /* local charset to server one (using UCS-2) */
-	void *		vc_ucs_tolocal;	/* server charset to local one (using UCS-2) */
 	int		vc_number;	/* number of this VC from the client side */
 	int		vc_genid;
 	uid_t		vc_uid;		/* user id of connection */
@@ -282,8 +272,6 @@ struct smb_vc {
 
 #define SMB_UNICODE_STRINGS(vcp)	((vcp)->vc_hflags2 & SMB_FLAGS2_UNICODE)
 
-#define	SMB_UNICODE_NAME	"UCS-2LE"
-
 /*
  * smb_share structure describes connection to the given SMB share (tree).
  * Connection to share is always built on top of the VC.
