@@ -28,17 +28,17 @@ __FBSDID("$FreeBSD$");
 
 #include <sys/param.h>
 #include <sys/systm.h>
+#include <sys/bus.h>
+#include <sys/cpuset.h>
 #include <sys/kernel.h>
 #include <sys/module.h>
-#include <sys/bus.h>
 #include <sys/rman.h>
-#include <sys/bus.h>
 
 #include <machine/bus.h>
 #include <machine/intr_machdep.h>
 #include <machine/pio.h>
 
-#include <powerpc/mpc85xx/ocpbus.h>
+#include <powerpc/mpc85xx/mpc85xx.h>
 
 #include <dev/ic/i8259.h>
 
@@ -134,7 +134,7 @@ static void
 atpic_intr(void *arg)
 {
 
-	atpic_dispatch(pic8259, arg);
+	atpic_dispatch(arg, NULL);
 }
 
 static void
@@ -151,7 +151,7 @@ atpic_isa_identify(driver_t *drv, device_t parent)
 	bus_set_resource(child, SYS_RES_IOPORT, ATPIC_SLAVE, IO_ICU2, 2);
 
 	/* ISA interrupts are routed through external interrupt 0. */
-	bus_set_resource(child, SYS_RES_IRQ, 0, PIC_IRQ_EXT(0), 1);
+	bus_set_resource(child, SYS_RES_IRQ, 0, 16, 1);
 }
 
 static int
@@ -211,14 +211,14 @@ atpic_isa_attach(device_t dev)
 		goto fail;
 
 	error = bus_setup_intr(dev, sc->sc_ires, INTR_TYPE_MISC | INTR_MPSAFE,
-	    NULL, atpic_intr, NULL, &sc->sc_icookie);
+	    NULL, atpic_intr, dev, &sc->sc_icookie);
 	if (error)
 		goto fail;
 
 	atpic_init(sc, ATPIC_SLAVE);
 	atpic_init(sc, ATPIC_MASTER);
 
-	powerpc_register_8259(dev);
+	powerpc_register_pic(dev, 0, 16, 0, TRUE);
 	return (0);
 
  fail:
@@ -305,12 +305,10 @@ atpic_mask(device_t dev, u_int irq)
 	if (irq > 7) {
 		sc->sc_mask[ATPIC_SLAVE] |= 1 << (irq - 8);
 		atpic_write(sc, ATPIC_SLAVE, 1, sc->sc_mask[ATPIC_SLAVE]);
-		atpic_write(sc, ATPIC_SLAVE, 0, OCW2_EOI);
 	} else {
 		sc->sc_mask[ATPIC_MASTER] |= 1 << irq;
 		atpic_write(sc, ATPIC_MASTER, 1, sc->sc_mask[ATPIC_MASTER]);
 	}
-	atpic_write(sc, ATPIC_MASTER, 0, OCW2_EOI);
 }
 
 static void
