@@ -21,17 +21,18 @@
  * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
  * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
  * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $FreeBSD$
  */
 
-#include <sys/cdefs.h>
-__FBSDID("$FreeBSD$");
-
 #include <sys/param.h>
 #include <sys/systm.h>
 #include <sys/module.h>
 #include <sys/bus.h>
 #include <sys/conf.h>
 #include <sys/kernel.h>
+#include <sys/pciio.h>
+#include <sys/rman.h>
 
 #include <dev/ofw/openfirm.h>
 #include <dev/ofw/ofw_pci.h>
@@ -46,8 +47,6 @@ __FBSDID("$FreeBSD$");
 #include <machine/pio.h>
 #include <machine/resource.h>
 
-#include <sys/rman.h>
-
 #include <dev/ofw/ofw_bus.h>
 #include <dev/ofw/ofw_bus_subr.h>
 
@@ -90,6 +89,16 @@ static void		cpcht_write_config(device_t, u_int, u_int, u_int,
 			    u_int, u_int32_t, int);
 static int		cpcht_route_interrupt(device_t bus, device_t dev,
 			    int pin);
+static int		cpcht_alloc_msi(device_t dev, device_t child,
+			    int count, int maxcount, int *irqs);
+static int		cpcht_release_msi(device_t dev, device_t child,
+			    int count, int *irqs);
+static int		cpcht_alloc_msix(device_t dev, device_t child,
+			    int *irq);
+static int		cpcht_release_msix(device_t dev, device_t child,
+			    int irq);
+static int		cpcht_map_msi(device_t dev, device_t child,
+			    int irq, uint64_t *addr, uint32_t *data);
 
 /*
  * ofw_bus interface
@@ -106,6 +115,7 @@ static device_method_t	cpcht_methods[] = {
 	DEVMETHOD(device_attach,	cpcht_attach),
 
 	/* Bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
 	DEVMETHOD(bus_read_ivar,	cpcht_read_ivar),
 	DEVMETHOD(bus_setup_intr,	bus_generic_setup_intr),
 	DEVMETHOD(bus_teardown_intr,	bus_generic_teardown_intr),
@@ -119,14 +129,22 @@ static device_method_t	cpcht_methods[] = {
 	DEVMETHOD(pcib_read_config,	cpcht_read_config),
 	DEVMETHOD(pcib_write_config,	cpcht_write_config),
 	DEVMETHOD(pcib_route_interrupt,	cpcht_route_interrupt),
+	DEVMETHOD(pcib_alloc_msi,	cpcht_alloc_msi),
+	DEVMETHOD(pcib_release_msi,	cpcht_release_msi),
+	DEVMETHOD(pcib_alloc_msix,	cpcht_alloc_msix),
+	DEVMETHOD(pcib_release_msix,	cpcht_release_msix),
+	DEVMETHOD(pcib_map_msi,		cpcht_map_msi),
 
 	/* ofw_bus interface */
 	DEVMETHOD(ofw_bus_get_node,     cpcht_get_node),
-
-	DEVMETHOD_END
+	{ 0, 0 }
 };
 
 struct cpcht_irq {
+	enum {
+	    IRQ_NONE, IRQ_HT, IRQ_MSI, IRQ_INTERNAL
+	}		irq_type; 
+
 	int		ht_source;
 
 	vm_offset_t	ht_base;
@@ -136,6 +154,7 @@ struct cpcht_irq {
 };
 
 static struct cpcht_irq *cpcht_irqmap = NULL;
+uint32_t cpcht_msipic = 0;
 
 struct cpcht_softc {
 	device_t		sc_dev;
@@ -146,6 +165,7 @@ struct cpcht_softc {
 	struct			rman sc_io_rman;
 
 	struct cpcht_irq	htirq_map[128];
+	struct mtx		htirq_mtx;
 };
 
 static driver_t	cpcht_driver = {
@@ -204,7 +224,7 @@ cpcht_attach(device_t dev)
 	struct		cpcht_softc *sc;
 	phandle_t	node, child;
 	u_int32_t	reg[3];
-	int		error;
+	int		i, error;
 
 	node = ofw_bus_get_node(dev);
 	sc = device_get_softc(dev);
@@ -243,6 +263,9 @@ cpcht_attach(device_t dev)
 	rman_manage_region(&sc->sc_io_rman, 0, CPCHT_IOPORT_SIZE - 1);
 
 	bzero(sc->htirq_map, sizeof(sc->htirq_map));
+	mtx_init(&sc->htirq_mtx, "cpcht irq", NULL, MTX_DEF);
+	for (i = 0; i < 8; i++)
+		sc->htirq_map[i].irq_type = IRQ_INTERNAL;
 	for (child = OF_child(node); child != 0; child = OF_peer(child))
 		cpcht_configure_htbridge(dev, child);
 
@@ -259,7 +282,7 @@ cpcht_configure_htbridge(device_t dev, phandle_t child)
 {
 	struct cpcht_softc *sc;
 	struct ofw_pci_register pcir;
-	struct cpcht_range ranges[6], *rp;
+	struct cpcht_range ranges[7], *rp;
 	int nranges, ptr, nextptr;
 	uint32_t vend, val;
 	int i, nirq, irq;
@@ -283,9 +306,10 @@ cpcht_configure_htbridge(device_t dev, phandle_t child)
 	 */
 	bzero(ranges, sizeof(ranges));
 	nranges = OF_getprop(child, "ranges", ranges, sizeof(ranges));
+	nranges /= sizeof(ranges[0]);
 	
 	ranges[6].pci_hi = 0;
-	for (rp = ranges; rp->pci_hi != 0; rp++) {
+	for (rp = ranges; rp < ranges + nranges && rp->pci_hi != 0; rp++) {
 		switch (rp->pci_hi & OFW_PCI_PHYS_HI_SPACEMASK) {
 		case OFW_PCI_PHYS_HI_SPACE_CONFIG:
 			break;
@@ -354,6 +378,7 @@ cpcht_configure_htbridge(device_t dev, phandle_t child)
 			    irq | HTAPIC_MASK, 4);
 			irq = (irq >> 16) & 0xff;
 
+			sc->htirq_map[irq].irq_type = IRQ_HT;
 			sc->htirq_map[irq].ht_source = i;
 			sc->htirq_map[irq].ht_base = sc->sc_data + 
 			    (((((s & 0x1f) << 3) | (f & 0x07)) << 8) | (ptr));
@@ -450,10 +475,6 @@ cpcht_write_config(device_t dev, u_int bus, u_int slot, u_int func,
 static int
 cpcht_read_ivar(device_t dev, device_t child, int which, uintptr_t *result)
 {
-	struct	cpcht_softc *sc;
-
-	sc = device_get_softc(dev);
-
 	switch (which) {
 	case PCIB_IVAR_DOMAIN:
 		*result = device_get_unit(dev);
@@ -489,13 +510,12 @@ cpcht_alloc_resource(device_t bus, device_t child, int type, int *rid,
 	struct			cpcht_softc *sc;
 	struct			resource *rv;
 	struct			rman *rm;
-	int			needactivate, err;
+	int			needactivate;
 
 	needactivate = flags & RF_ACTIVE;
 	flags &= ~RF_ACTIVE;
 
 	sc = device_get_softc(bus);
-	err = 0;
 
 	switch (type) {
 	case SYS_RES_IOPORT:
@@ -544,9 +564,6 @@ cpcht_activate_resource(device_t bus, device_t child, int type, int rid,
     struct resource *res)
 {
 	void	*p;
-	struct	cpcht_softc *sc;
-
-	sc = device_get_softc(bus);
 
 	if (type == SYS_RES_IRQ)
 		return (bus_activate_resource(bus, type, rid, res));
@@ -606,6 +623,129 @@ cpcht_deactivate_resource(device_t bus, device_t child, int type, int rid,
 	return (rman_deactivate_resource(res));
 }
 
+static int
+cpcht_alloc_msi(device_t dev, device_t child, int count, int maxcount,
+    int *irqs)
+{
+	struct cpcht_softc *sc;
+	int i, j;
+
+	sc = device_get_softc(dev);
+	j = 0;
+
+	/* Bail if no MSI PIC yet */
+	if (cpcht_msipic == 0)
+		return (ENXIO);
+
+	mtx_lock(&sc->htirq_mtx);
+	for (i = 8; i < 124 - count; i++) {
+		for (j = 0; j < count; j++) {
+			if (sc->htirq_map[i+j].irq_type != IRQ_NONE)
+				break;
+		}
+		if (j == count)
+			break;
+
+		i += j; /* We know there isn't a large enough run */
+	}
+
+	if (j != count) {
+		mtx_unlock(&sc->htirq_mtx);
+		return (ENXIO);
+	}
+
+	for (j = 0; j < count; j++) {
+		irqs[j] = MAP_IRQ(cpcht_msipic, i+j);
+		sc->htirq_map[i+j].irq_type = IRQ_MSI;
+	}
+	mtx_unlock(&sc->htirq_mtx);
+
+	return (0);
+}
+
+static int
+cpcht_release_msi(device_t dev, device_t child, int count, int *irqs)
+{
+	struct cpcht_softc *sc;
+	int i;
+
+	sc = device_get_softc(dev);
+
+	mtx_lock(&sc->htirq_mtx);
+	for (i = 0; i < count; i++)
+		sc->htirq_map[irqs[i] & 0xff].irq_type = IRQ_NONE;
+	mtx_unlock(&sc->htirq_mtx);
+
+	return (0);
+}
+
+static int
+cpcht_alloc_msix(device_t dev, device_t child, int *irq)
+{
+	struct cpcht_softc *sc;
+	int i;
+
+	sc = device_get_softc(dev);
+
+	/* Bail if no MSI PIC yet */
+	if (cpcht_msipic == 0)
+		return (ENXIO);
+
+	mtx_lock(&sc->htirq_mtx);
+	for (i = 8; i < 124; i++) {
+		if (sc->htirq_map[i].irq_type == IRQ_NONE) {
+			sc->htirq_map[i].irq_type = IRQ_MSI;
+			*irq = MAP_IRQ(cpcht_msipic, i);
+
+			mtx_unlock(&sc->htirq_mtx);
+			return (0);
+		}
+	}
+	mtx_unlock(&sc->htirq_mtx);
+
+	return (ENXIO);
+}
+	
+static int
+cpcht_release_msix(device_t dev, device_t child, int irq)
+{
+	struct cpcht_softc *sc;
+
+	sc = device_get_softc(dev);
+
+	mtx_lock(&sc->htirq_mtx);
+	sc->htirq_map[irq & 0xff].irq_type = IRQ_NONE;
+	mtx_unlock(&sc->htirq_mtx);
+
+	return (0);
+}
+
+static int
+cpcht_map_msi(device_t dev, device_t child, int irq, uint64_t *addr,
+    uint32_t *data)
+{
+	device_t pcib;
+	struct pci_devinfo *dinfo;
+	struct pcicfg_ht *ht = NULL;
+
+	for (pcib = child; pcib != dev; pcib =
+	    device_get_parent(device_get_parent(pcib))) {
+		dinfo = device_get_ivars(pcib);
+		ht = &dinfo->cfg.ht;
+
+		if (ht == NULL)
+			continue;
+	}
+
+	if (ht == NULL)
+		return (ENXIO);
+
+	*addr = ht->ht_msiaddr;
+	*data = irq & 0xff;
+
+	return (0);
+}
+
 /*
  * Driver for the integrated MPIC on U3/U4 (CPC925/CPC945)
  */
@@ -624,6 +764,7 @@ static device_method_t  openpic_cpcht_methods[] = {
 	DEVMETHOD(device_attach,	openpic_cpcht_attach),
 
 	/* PIC interface */
+	DEVMETHOD(pic_bind,		openpic_bind),
 	DEVMETHOD(pic_config,		openpic_cpcht_config),
 	DEVMETHOD(pic_dispatch,		openpic_dispatch),
 	DEVMETHOD(pic_enable,		openpic_cpcht_enable),
@@ -665,9 +806,11 @@ static int
 openpic_cpcht_attach(device_t dev)
 {
 	struct openpic_cpcht_softc *sc;
+	phandle_t node;
 	int err, irq;
 
-	err = openpic_attach(dev);
+	node = ofw_bus_get_node(dev);
+	err = openpic_common_attach(dev, node);
 	if (err != 0)
 		return (err);
 
@@ -691,6 +834,14 @@ openpic_cpcht_attach(device_t dev)
 	for (irq = 4; irq < 124; irq++)
 		openpic_config(dev, irq, INTR_TRIGGER_EDGE, INTR_POLARITY_LOW);
 
+	/*
+	 * Use this PIC for MSI only if it is the root PIC. This may not
+	 * be necessary, but Linux does it, and I cannot find any U3 machines
+	 * with MSI devices to test.
+	 */
+	if (dev == root_pic)
+		cpcht_msipic = node;
+
 	return (0);
 }
 
@@ -829,4 +980,3 @@ openpic_cpcht_eoi(device_t dev, u_int irq)
 
 	openpic_eoi(dev, irq);
 }
-
