@@ -115,6 +115,8 @@ static device_method_t unin_chip_methods[] = {
 	DEVMETHOD(bus_deactivate_resource, unin_chip_deactivate_resource),
 	DEVMETHOD(bus_get_resource_list, unin_chip_get_resource_list),
 
+	DEVMETHOD(bus_child_pnpinfo_str, ofw_bus_gen_child_pnpinfo_str),
+
         /* ofw_bus interface */
 	DEVMETHOD(ofw_bus_get_devinfo,	unin_chip_get_devinfo),
 	DEVMETHOD(ofw_bus_get_compat,	ofw_bus_gen_get_compat),
@@ -142,9 +144,9 @@ DRIVER_MODULE(unin, nexus, unin_chip_driver, unin_chip_devclass, 0, 0);
 static void
 unin_chip_add_intr(phandle_t devnode, struct unin_chip_devinfo *dinfo)
 {
+	phandle_t iparent;
 	int	*intr;
 	int	i, nintr;
-	phandle_t iparent;
 	int 	icells;
 
 	if (dinfo->udi_ninterrupts >= 6) {
@@ -173,10 +175,18 @@ unin_chip_add_intr(phandle_t devnode, struct unin_chip_devinfo *dinfo)
 		icells = 1;
 
 	for (i = 0; i < nintr; i+=icells) {
-		resource_list_add(&dinfo->udi_resources, SYS_RES_IRQ,
-		    dinfo->udi_ninterrupts, intr[i], intr[i], 1);
+		u_int irq = MAP_IRQ(iparent, intr[i]);
 
-		dinfo->udi_interrupts[dinfo->udi_ninterrupts] = intr[i];
+		resource_list_add(&dinfo->udi_resources, SYS_RES_IRQ,
+		    dinfo->udi_ninterrupts, irq, irq, 1);
+
+		if (icells > 1) {
+			powerpc_config_intr(irq,
+			    (intr[i+1] & 1) ? INTR_TRIGGER_LEVEL :
+			    INTR_TRIGGER_EDGE, INTR_POLARITY_LOW);
+		}
+
+		dinfo->udi_interrupts[dinfo->udi_ninterrupts] = irq;
 		dinfo->udi_ninterrupts++;
 	}
 }
@@ -520,11 +530,8 @@ static int
 unin_chip_activate_resource(device_t bus, device_t child, int type, int rid,
 			    struct resource *res)
 {
-	struct unin_chip_softc *sc;
 	void    *p;
 
-	sc = device_get_softc(bus);
-
 	if (type == SYS_RES_IRQ)
                 return (bus_activate_resource(bus, type, rid, res));
 
