@@ -75,27 +75,27 @@ SDT_PROVIDER_DECLARE(mac_framework);	/* Entry points to MAC. */
 
 #define	MAC_CHECK_PROBE_DEFINE4(name, arg0, arg1, arg2, arg3)		\
 	SDT_PROBE_DEFINE5(mac_framework, kernel, name, mac_check_err,	\
-	    "int", arg0, arg1, arg2, arg3);				\
+	    mac-check-ok, "int", arg0, arg1, arg2, arg3);		\
 	SDT_PROBE_DEFINE5(mac_framework, kernel, name, mac_check_ok,	\
-	    "int", arg0, arg1, arg2, arg3);
+	    mac-check-ok, "int", arg0, arg1, arg2, arg3);
 
 #define	MAC_CHECK_PROBE_DEFINE3(name, arg0, arg1, arg2)			\
 	SDT_PROBE_DEFINE4(mac_framework, kernel, name, mac_check_err,	\
-	    "int", arg0, arg1, arg2);					\
+	    mac-check-err, "int", arg0, arg1, arg2);			\
 	SDT_PROBE_DEFINE4(mac_framework, kernel, name, mac_check_ok,	\
-	    "int", arg0, arg1, arg2);
+	    mac-check-ok, "int", arg0, arg1, arg2);
 
 #define	MAC_CHECK_PROBE_DEFINE2(name, arg0, arg1)			\
 	SDT_PROBE_DEFINE3(mac_framework, kernel, name, mac_check_err,	\
-	    "int", arg0, arg1);						\
+	    mac-check-err, "int", arg0, arg1);				\
 	SDT_PROBE_DEFINE3(mac_framework, kernel, name, mac_check_ok,	\
-	    "int", arg0, arg1);
+	    mac-check-ok, "int", arg0, arg1);
 
 #define	MAC_CHECK_PROBE_DEFINE1(name, arg0)				\
 	SDT_PROBE_DEFINE2(mac_framework, kernel, name, mac_check_err,	\
-	    "int", arg0);						\
+	    mac-check-err, "int", arg0);				\
 	SDT_PROBE_DEFINE2(mac_framework, kernel, name, mac_check_ok,	\
-	    "int", arg0);
+	    mac-check-ok, "int", arg0);
 
 #define	MAC_CHECK_PROBE4(name, error, arg0, arg1, arg2, arg3)	do {	\
 	if (error) {							\
@@ -117,9 +117,9 @@ SDT_PROVIDER_DECLARE(mac_framework);	/* Entry points to MAC. */
 
 #define	MAC_GRANT_PROBE_DEFINE2(name, arg0, arg1)			\
 	SDT_PROBE_DEFINE3(mac_framework, kernel, name, mac_grant_err,	\
-	    "int", arg0, arg1);						\
+	    mac-grant-err, "int", arg0, arg1);				\
 	SDT_PROBE_DEFINE3(mac_framework, kernel, name, mac_grant_ok,	\
-	    "INT", arg0, arg1);
+	    mac-grant-ok, "INT", arg0, arg1);
 
 #define	MAC_GRANT_PROBE2(name, error, arg0, arg1)	do {		\
 	if (error) {							\
