@@ -91,6 +91,7 @@ static void schizo_intr_clear(void *);
 static int schizo_intr_register(struct schizo_softc *sc, u_int ino);
 static int schizo_get_intrmap(struct schizo_softc *, u_int,
     bus_addr_t *, bus_addr_t *);
+static bus_space_tag_t schizo_alloc_bus_tag(struct schizo_softc *, int);
 static timecounter_get_t schizo_get_timecount;
 
 /* Interrupt handlers */
@@ -112,7 +113,8 @@ static bus_read_ivar_t schizo_read_ivar;
 static bus_setup_intr_t schizo_setup_intr;
 static bus_alloc_resource_t schizo_alloc_resource;
 static bus_activate_resource_t schizo_activate_resource;
-static bus_adjust_resource_t schizo_adjust_resource;
+static bus_deactivate_resource_t schizo_deactivate_resource;
+static bus_release_resource_t schizo_release_resource;
 static bus_get_dma_tag_t schizo_get_dma_tag;
 static pcib_maxslots_t schizo_maxslots;
 static pcib_read_config_t schizo_read_config;
@@ -130,14 +132,14 @@ static device_method_t schizo_methods[] = {
 	DEVMETHOD(device_resume,	bus_generic_resume),
 
 	/* Bus interface */
+	DEVMETHOD(bus_print_child,	bus_generic_print_child),
 	DEVMETHOD(bus_read_ivar,	schizo_read_ivar),
 	DEVMETHOD(bus_setup_intr,	schizo_setup_intr),
 	DEVMETHOD(bus_teardown_intr,	bus_generic_teardown_intr),
 	DEVMETHOD(bus_alloc_resource,	schizo_alloc_resource),
-	DEVMETHOD(bus_activate_resource, schizo_activate_resource),
-	DEVMETHOD(bus_deactivate_resource, bus_generic_deactivate_resource),
-	DEVMETHOD(bus_adjust_resource,	schizo_adjust_resource),
-	DEVMETHOD(bus_release_resource,	bus_generic_release_resource),
+	DEVMETHOD(bus_activate_resource,	schizo_activate_resource),
+	DEVMETHOD(bus_deactivate_resource,	schizo_deactivate_resource),
+	DEVMETHOD(bus_release_resource,	schizo_release_resource),
 	DEVMETHOD(bus_get_dma_tag,	schizo_get_dma_tag),
 
 	/* pcib interface */
@@ -149,10 +151,10 @@ static device_method_t schizo_methods[] = {
 	/* ofw_bus interface */
 	DEVMETHOD(ofw_bus_get_node,	schizo_get_node),
 
-	/* ofw_pci interface */
+	/* ofw_pci interface */ 	 
 	DEVMETHOD(ofw_pci_setup_device,	schizo_setup_device),
 
-	DEVMETHOD_END
+	KOBJMETHOD_END
 };
 
 static devclass_t schizo_devclass;
@@ -178,8 +180,6 @@ struct schizo_icarg {
 	bus_addr_t		sica_clr;
 };
 
-#define	SCHIZO_CDMA_TIMEOUT	1	/* 1 second per try */
-#define	SCHIZO_CDMA_TRIES	15
 #define	SCHIZO_PERF_CNT_QLTY	100
 
 #define	SCHIZO_SPC_BARRIER(spc, sc, offs, len, flags)			\
@@ -501,8 +501,7 @@ schizo_attach(device_t dev)
 	 * Set up the IOMMU.  Schizo, Tomatillo and XMITS all have
 	 * one per PBM.  Schizo and XMITS additionally have a streaming
 	 * buffer, in Schizo version < 5 (i.e. revision < 2.3) it's
-	 * affected by several errata though.  However, except for context
-	 * flushes, taking advantage of it should be okay even with those.
+	 * affected by several errata and basically unusable though.
 	 */
 	memcpy(&sc->sc_dma_methods, &iommu_dma_methods,
 	    sizeof(sc->sc_dma_methods));
@@ -510,7 +509,8 @@ schizo_attach(device_t dev)
 	sc->sc_is.sis_is.is_flags = IOMMU_PRESERVE_PROM;
 	sc->sc_is.sis_is.is_pmaxaddr = IOMMU_MAXADDR(STX_IOMMU_BITS);
 	sc->sc_is.sis_is.is_sb[0] = sc->sc_is.sis_is.is_sb[1] = 0;
-	if (OF_getproplen(node, "no-streaming-cache") < 0)
+	if (OF_getproplen(node, "no-streaming-cache") < 0 &&
+	    !(sc->sc_mode == SCHIZO_MODE_SCZ && sc->sc_ver < 5))
 		sc->sc_is.sis_is.is_sb[0] = STX_PCI_STRBUF;
 
 #define	TSBCASE(x)							\
@@ -580,20 +580,14 @@ schizo_attach(device_t dev)
 	SLIST_INSERT_HEAD(&schizo_softcs, sc, sc_link);
 
 	/* Allocate our tags. */
-	sc->sc_pci_iot = sparc64_alloc_bus_tag(NULL, rman_get_bustag(
-	    sc->sc_mem_res[STX_PCI]), PCI_IO_BUS_SPACE, NULL);
-	if (sc->sc_pci_iot == NULL)
-		panic("%s: could not allocate PCI I/O tag", __func__);
-	sc->sc_pci_cfgt = sparc64_alloc_bus_tag(NULL, rman_get_bustag(
-	    sc->sc_mem_res[STX_PCI]), PCI_CONFIG_BUS_SPACE, NULL);
-	if (sc->sc_pci_cfgt == NULL)
-		panic("%s: could not allocate PCI configuration space tag",
-		    __func__);
+	sc->sc_pci_memt = schizo_alloc_bus_tag(sc, PCI_MEMORY_BUS_SPACE);
+	sc->sc_pci_iot = schizo_alloc_bus_tag(sc, PCI_IO_BUS_SPACE);
+	sc->sc_pci_cfgt = schizo_alloc_bus_tag(sc, PCI_CONFIG_BUS_SPACE);
 	if (bus_dma_tag_create(bus_get_dma_tag(dev), 8, 0,
 	    sc->sc_is.sis_is.is_pmaxaddr, ~0, NULL, NULL,
 	    sc->sc_is.sis_is.is_pmaxaddr, 0xff, 0xffffffff, 0, NULL, NULL,
 	    &sc->sc_pci_dmat) != 0)
-		panic("%s: could not create PCI DMA tag", __func__);
+		panic("%s: bus_dma_tag_create failed", __func__);
 	/* Customize the tag. */
 	sc->sc_pci_dmat->dt_cookie = &sc->sc_is;
 	sc->sc_pci_dmat->dt_mt = &sc->sc_dma_methods;
@@ -708,15 +702,13 @@ schizo_attach(device_t dev)
 			i = INTINO(bus_get_resource_start(dev, SYS_RES_IRQ,
 			    4));
 			if (i == STX_CDMA_A_INO || i == STX_CDMA_B_INO) {
-				sc->sc_cdma_vec = INTMAP_VEC(sc->sc_ign, i);
-				(void)schizo_get_intrmap(sc, i,
-				   &sc->sc_cdma_map, &sc->sc_cdma_clr);
+				(void)schizo_get_intrmap(sc, i, NULL,
+				   &sc->sc_cdma_clr);
 				schizo_set_intr(sc, 4, i, schizo_cdma);
 			} else {
 				i = STX_CDMA_A_INO + sc->sc_half;
-				sc->sc_cdma_vec = INTMAP_VEC(sc->sc_ign, i);
 				if (bus_set_resource(dev, SYS_RES_IRQ, 5,
-				    sc->sc_cdma_vec, 1) != 0)
+				    INTMAP_VEC(sc->sc_ign, i), 1) != 0)
 					panic("%s: failed to add CDMA "
 					    "interrupt", __func__);
 				j = schizo_intr_register(sc, i);
@@ -724,8 +716,8 @@ schizo_attach(device_t dev)
 					panic("%s: could not register "
 					    "interrupt controller for CDMA "
 					    "(%d)", __func__, j);
-				(void)schizo_get_intrmap(sc, i,
-				   &sc->sc_cdma_map, &sc->sc_cdma_clr);
+				(void)schizo_get_intrmap(sc, i, NULL,
+				   &sc->sc_cdma_clr);
 				schizo_set_intr(sc, 5, i, schizo_cdma);
 			}
 		} else {
@@ -992,8 +984,7 @@ schizo_cdma(void *arg)
 {
 	struct schizo_softc *sc = arg;
 
-	atomic_cmpset_32(&sc->sc_cdma_state, SCHIZO_CDMA_STATE_PENDING,
-	    SCHIZO_CDMA_STATE_RECEIVED);
+	atomic_store_rel_32(&sc->sc_cdma_state, SCHIZO_CDMA_STATE_RECEIVED);
 	return (FILTER_HANDLED);
 }
 
@@ -1127,7 +1118,7 @@ schizo_route_interrupt(device_t bridge, device_t dev, int pin)
 	pintr = pin;
 	if (ofw_bus_lookup_imap(ofw_bus_get_node(dev), &sc->sc_pci_iinfo,
 	    &reg, sizeof(reg), &pintr, sizeof(pintr), &mintr, sizeof(mintr),
-	    maskbuf))
+	    NULL, maskbuf))
 		return (mintr);
 
 	device_printf(bridge, "could not route pin %d for device %d.%d\n",
@@ -1158,10 +1149,7 @@ schizo_dmamap_sync(bus_dma_tag_t dt, bus_dmamap_t map, bus_dmasync_op_t op)
 	struct timeval cur, end;
 	struct schizo_iommu_state *sis = dt->dt_cookie;
 	struct schizo_softc *sc = sis->sis_sc;
-	int i, res;
-#ifdef INVARIANTS
-	register_t pil;
-#endif
+	int res;
 
 	if ((map->dm_flags & DMF_STREAMED) != 0) {
 		iommu_dma_methods.dm_dmamap_sync(dt, map, op);
@@ -1173,41 +1161,25 @@ schizo_dmamap_sync(bus_dma_tag_t dt, bus_dmamap_t map, bus_dmasync_op_t op)
 
 	if ((op & BUS_DMASYNC_POSTREAD) != 0) {
 		/*
-		 * Note that in order to allow this function to be called from
+	 	 * Note that in order to allow this function to be called from
 		 * filters we would need to use a spin mutex for serialization
 		 * but given that these disable interrupts we have to emulate
 		 * one.
 		 */
-		critical_enter();
-		KASSERT((rdpr(pstate) & PSTATE_IE) != 0,
-		    ("%s: interrupts disabled", __func__));
-		KASSERT((pil = rdpr(pil)) <= PIL_BRIDGE,
-		    ("%s: PIL too low (%ld)", __func__, pil));
 		for (; atomic_cmpset_acq_32(&sc->sc_cdma_state,
 		    SCHIZO_CDMA_STATE_IDLE, SCHIZO_CDMA_STATE_PENDING) == 0;)
 			;
-		SCHIZO_PCI_WRITE_8(sc, sc->sc_cdma_map,
-		    INTMAP_ENABLE(sc->sc_cdma_vec, PCPU_GET(mid)));
-		for (i = 0; i < SCHIZO_CDMA_TRIES; i++) {
-			if (i > 0)
-				printf("%s: try %d\n", __func__, i);
-			SCHIZO_PCI_WRITE_8(sc, sc->sc_cdma_clr,
-			    INTCLR_RECEIVED);
+		SCHIZO_PCI_WRITE_8(sc, sc->sc_cdma_clr, INTCLR_RECEIVED);
+		microuptime(&cur);
+		end.tv_sec = 15;
+		end.tv_usec = 0;
+		timevaladd(&end, &cur);
+		for (; (res = atomic_cmpset_rel_32(&sc->sc_cdma_state,
+		    SCHIZO_CDMA_STATE_RECEIVED, SCHIZO_CDMA_STATE_IDLE)) ==
+		    0 && timevalcmp(&cur, &end, <=);)
 			microuptime(&cur);
-			end.tv_sec = SCHIZO_CDMA_TIMEOUT;
-			end.tv_usec = 0;
-			timevaladd(&end, &cur);
-			for (; (res = atomic_cmpset_rel_32(&sc->sc_cdma_state,
-			    SCHIZO_CDMA_STATE_RECEIVED,
-			    SCHIZO_CDMA_STATE_IDLE)) == 0 &&
-			    timevalcmp(&cur, &end, <=);)
-				microuptime(&cur);
-			if (res != 0)
-				break;
-		}
 		if (res == 0)
 			panic("%s: DMA does not sync", __func__);
-		critical_exit();
 	}
 
 	if ((op & BUS_DMASYNC_PREWRITE) != 0)
@@ -1361,10 +1333,14 @@ schizo_alloc_resource(device_t bus, device_t child, int type, int *rid,
 	struct schizo_softc *sc;
 	struct resource *rv;
 	struct rman *rm;
+	bus_space_tag_t bt;
+	bus_space_handle_t bh;
+	int needactivate = flags & RF_ACTIVE;
+
+	flags &= ~RF_ACTIVE;
 
 	sc = device_get_softc(bus);
-	switch (type) {
-	case SYS_RES_IRQ:
+	if (type == SYS_RES_IRQ) {
 		/*
 		 * XXX: Don't accept blank ranges for now, only single
 		 * interrupts.  The other case should not happen with
@@ -1375,28 +1351,38 @@ schizo_alloc_resource(device_t bus, device_t child, int type, int *rid,
 		if (start != end)
 			panic("%s: XXX: interrupt range", __func__);
 		start = end = INTMAP_VEC(sc->sc_ign, end);
-		return (bus_generic_alloc_resource(bus, child, type, rid,
-		    start, end, count, flags));
+		return (BUS_ALLOC_RESOURCE(device_get_parent(bus), child,
+		    type, rid, start, end, count, flags));
+	}
+	switch (type) {
 	case SYS_RES_MEMORY:
 		rm = &sc->sc_pci_mem_rman;
+		bt = sc->sc_pci_memt;
+		bh = sc->sc_pci_bh[OFW_PCI_CS_MEM32];
 		break;
 	case SYS_RES_IOPORT:
 		rm = &sc->sc_pci_io_rman;
+		bt = sc->sc_pci_iot;
+		bh = sc->sc_pci_bh[OFW_PCI_CS_IO];
 		break;
 	default:
 		return (NULL);
+		/* NOTREACHED */
 	}
 
-	rv = rman_reserve_resource(rm, start, end, count, flags & ~RF_ACTIVE,
-	    child);
+	rv = rman_reserve_resource(rm, start, end, count, flags, child);
 	if (rv == NULL)
 		return (NULL);
 	rman_set_rid(rv, *rid);
+	bh += rman_get_start(rv);
+	rman_set_bustag(rv, bt);
+	rman_set_bushandle(rv, bh);
 
-	if ((flags & RF_ACTIVE) != 0 && bus_activate_resource(child, type,
-	    *rid, rv) != 0) {
-		rman_release_resource(rv);
-		return (NULL);
+	if (needactivate) {
+		if (bus_activate_resource(child, type, *rid, rv)) {
+			rman_release_resource(rv);
+			return (NULL);
+		}
 	}
 	return (rv);
 }
@@ -1405,56 +1391,56 @@ static int
 schizo_activate_resource(device_t bus, device_t child, int type, int rid,
     struct resource *r)
 {
-	struct schizo_softc *sc;
-	struct bus_space_tag *tag;
+	void *p;
+	int error;
 
-	sc = device_get_softc(bus);
-	switch (type) {
-	case SYS_RES_IRQ:
-		return (bus_generic_activate_resource(bus, child, type, rid,
-		    r));
-	case SYS_RES_MEMORY:
-		tag = sparc64_alloc_bus_tag(r, rman_get_bustag(
-		    sc->sc_mem_res[STX_PCI]), PCI_MEMORY_BUS_SPACE, NULL);
-		if (tag == NULL)
-			return (ENOMEM);
-		rman_set_bustag(r, tag);
-		rman_set_bushandle(r, sc->sc_pci_bh[OFW_PCI_CS_MEM32] +
-		    rman_get_start(r));
-		break;
-	case SYS_RES_IOPORT:
-		rman_set_bustag(r, sc->sc_pci_iot);
-		rman_set_bushandle(r, sc->sc_pci_bh[OFW_PCI_CS_IO] +
-		    rman_get_start(r));
-		break;
+	if (type == SYS_RES_IRQ)
+		return (BUS_ACTIVATE_RESOURCE(device_get_parent(bus), child,
+		    type, rid, r));
+	if (type == SYS_RES_MEMORY) {
+		/*
+		 * Need to memory-map the device space, as some drivers
+		 * depend on the virtual address being set and usable.
+		 */
+		error = sparc64_bus_mem_map(rman_get_bustag(r),
+		    rman_get_bushandle(r), rman_get_size(r), 0, 0, &p);
+		if (error != 0)
+			return (error);
+		rman_set_virtual(r, p);
 	}
 	return (rman_activate_resource(r));
 }
 
 static int
-schizo_adjust_resource(device_t bus, device_t child, int type,
-    struct resource *r, u_long start, u_long end)
+schizo_deactivate_resource(device_t bus, device_t child, int type, int rid,
+    struct resource *r)
 {
-	struct schizo_softc *sc;
-	struct rman *rm;
 
-	sc = device_get_softc(bus);
-	switch (type) {
-	case SYS_RES_IRQ:
-		return (bus_generic_adjust_resource(bus, child, type, r,
-		    start, end));
-	case SYS_RES_MEMORY:
-		rm = &sc->sc_pci_mem_rman;
-		break;
-	case SYS_RES_IOPORT:
-		rm = &sc->sc_pci_io_rman;
-		break;
-	default:
-		return (EINVAL);
+	if (type == SYS_RES_IRQ)
+		return (BUS_DEACTIVATE_RESOURCE(device_get_parent(bus), child,
+		    type, rid, r));
+	if (type == SYS_RES_MEMORY) {
+		sparc64_bus_mem_unmap(rman_get_virtual(r), rman_get_size(r));
+		rman_set_virtual(r, NULL);
 	}
-	if (rman_is_region_manager(r, rm) == 0)
-		return (EINVAL);
-	return (rman_adjust_resource(r, start, end));
+	return (rman_deactivate_resource(r));
+}
+
+static int
+schizo_release_resource(device_t bus, device_t child, int type, int rid,
+    struct resource *r)
+{
+	int error;
+
+	if (type == SYS_RES_IRQ)
+		return (BUS_RELEASE_RESOURCE(device_get_parent(bus), child,
+		    type, rid, r));
+	if (rman_get_flags(r) & RF_ACTIVE) {
+		error = bus_deactivate_resource(child, type, rid, r);
+		if (error)
+			return (error);
+	}
+	return (rman_release_resource(r));
 }
 
 static bus_dma_tag_t
@@ -1487,7 +1473,7 @@ schizo_setup_device(device_t bus, device_t child)
 	/*
 	 * Disable bus parking in order to work around a bus hang caused by
 	 * Casinni/Skyhawk combinations.
-	 */
+	 */ 
 	if (OF_getproplen(ofw_bus_get_node(child), "pci-req-removal") >= 0)
 		SCHIZO_PCI_SET(sc, STX_PCI_CTRL, SCHIZO_PCI_READ_8(sc,
 		    STX_PCI_CTRL) & ~STX_PCI_CTRL_ARB_PARK);
@@ -1512,6 +1498,22 @@ schizo_setup_device(device_t bus, device_t child)
 	}
 }
 
+static bus_space_tag_t
+schizo_alloc_bus_tag(struct schizo_softc *sc, int type)
+{
+	bus_space_tag_t bt;
+
+	bt = malloc(sizeof(struct bus_space_tag), M_DEVBUF,
+	    M_NOWAIT | M_ZERO);
+	if (bt == NULL)
+		panic("%s: out of memory", __func__);
+
+	bt->bst_cookie = sc;
+	bt->bst_parent = rman_get_bustag(sc->sc_mem_res[STX_PCI]);
+	bt->bst_type = type;
+	return (bt);
+}
+
 static u_int
 schizo_get_timecount(struct timecounter *tc)
 {
