@@ -14,13 +14,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the NetBSD
- *	Foundation, Inc. and its contributors.
- * 4. Neither the name of The NetBSD Foundation nor the names of its
- *    contributors may be used to endorse or promote products derived
- *    from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
@@ -93,7 +86,7 @@
  * SUCH DAMAGE.
  *
  *	from: @(#)machdep.c	8.6 (Berkeley) 1/14/94
- *	from: NetBSD: machdep.c,v 1.111 2001/09/15 07:13:40 eeh Exp
+ *	from: NetBSD: machdep.c,v 1.221 2008/04/28 20:23:37 martin Exp
  *	and
  *	from: FreeBSD: src/sys/i386/i386/busdma_machdep.c,v 1.24 2001/08/15
  */
@@ -652,7 +645,7 @@ nexus_dmamem_alloc(bus_dma_tag_t dmat, void **vaddr, int flags,
 	}
 	if (*vaddr == NULL)
 		return (ENOMEM);
-	if ((uintptr_t)*vaddr % dmat->dt_alignment)
+	if (vtophys(*vaddr) % dmat->dt_alignment)
 		printf("%s: failed to align memory properly.\n", __func__);
 	return (0);
 }
