@@ -67,6 +67,7 @@ __FBSDID("$FreeBSD$");
 #include <sys/reboot.h>
 #include <sys/signalvar.h>
 #include <sys/smp.h>
+#include <sys/syscallsubr.h>
 #include <sys/sysent.h>
 #include <sys/sysproto.h>
 #include <sys/timetc.h>
@@ -87,6 +88,7 @@ __FBSDID("$FreeBSD$");
 
 #include <machine/bus.h>
 #include <machine/cache.h>
+#include <machine/clock.h>
 #include <machine/cmt.h>
 #include <machine/cpu.h>
 #include <machine/fireplane.h>
@@ -374,7 +376,7 @@ sparc64_init(caddr_t mdp, u_long o1, u_long o2, u_long o3, ofw_vec_t *vec)
 
 	/*
 	 * Parse metadata if present and fetch parameters.  Must be before the
-	 * console is inited so cninit() gets the right value of boothowto.
+	 * console is inited so cninit gets the right value of boothowto.
 	 */
 	if (mdp != NULL) {
 		preload_metadata = mdp;
@@ -419,19 +421,37 @@ sparc64_init(caddr_t mdp, u_long o1, u_long o2, u_long o3, ofw_vec_t *vec)
 	root = OF_peer(0);
 	pc->pc_node = find_bsp(root, pc->pc_mid, cpu_impl);
 	if (pc->pc_node == 0)
-		OF_panic("%s: cannot find boot CPU node", __func__);
+		OF_exit();
 	if (OF_getprop(pc->pc_node, "clock-frequency", &pc->pc_clock,
 	    sizeof(pc->pc_clock)) <= 0)
-		OF_panic("%s: cannot determine boot CPU clock", __func__);
+		OF_exit();
+
+	/*
+	 * Provide a DELAY() that works before PCPU_REG is set.  We can't
+	 * set PCPU_REG without also taking over the trap table or the
+	 * firmware will overwrite it.  Unfortunately, it's way to early
+	 * to also take over the trap table at this point.
+	 */
+	clock_boot = pc->pc_clock;
+	delay_func = delay_boot;
+
+	/*
+	 * Initialize the console before printing anything.
+	 * NB: the low-level console drivers require a working DELAY() at
+	 * this point.
+	 */
+	cninit();
 
 	/*
 	 * Panic if there is no metadata.  Most likely the kernel was booted
 	 * directly, instead of through loader(8).
 	 */
 	if (mdp == NULL || kmdp == NULL || end == 0 ||
-	    kernel_tlb_slots == 0 || kernel_tlbs == NULL)
-		OF_panic("%s: missing loader metadata.\nThis probably means "
-		    "you are not using loader(8).", __func__);
+	    kernel_tlb_slots == 0 || kernel_tlbs == NULL) {
+		printf("sparc64_init: missing loader metadata.\n"
+		    "This probably means you are not using loader(8).\n");
+		panic("sparc64_init");
+	}
 
 	/*
 	 * Work around the broken loader behavior of not demapping no
@@ -441,7 +461,7 @@ sparc64_init(caddr_t mdp, u_long o1, u_long o2, u_long o3, ofw_vec_t *vec)
 	for (va = KERNBASE + (kernel_tlb_slots - 1) * PAGE_SIZE_4M;
 	    va >= roundup2(end, PAGE_SIZE_4M); va -= PAGE_SIZE_4M) {
 		if (bootverbose)
-			OF_printf("demapping unused kernel TLB slot "
+			printf("demapping unused kernel TLB slot "
 			    "(va %#lx - %#lx)\n", va, va + PAGE_SIZE_4M - 1);
 		stxa(TLB_DEMAP_VA(va) | TLB_DEMAP_PRIMARY | TLB_DEMAP_PAGE,
 		    ASI_DMMU_DEMAP, 0);
@@ -459,15 +479,13 @@ sparc64_init(caddr_t mdp, u_long o1, u_long o2, u_long o3, ofw_vec_t *vec)
 	 */
 	if (OF_getprop(pc->pc_node, "#dtlb-entries", &dtlb_slots,
 	    sizeof(dtlb_slots)) == -1)
-		OF_panic("%s: cannot determine number of dTLB slots",
-		    __func__);
+		panic("sparc64_init: cannot determine number of dTLB slots");
 	if (OF_getprop(pc->pc_node, "#itlb-entries", &itlb_slots,
 	    sizeof(itlb_slots)) == -1)
-		OF_panic("%s: cannot determine number of iTLB slots",
-		    __func__);
+		panic("sparc64_init: cannot determine number of iTLB slots");
 
 	/*
-	 * Initialize and enable the caches.  Note that this may include
+	 * Initialize and enable the caches.  Note that his may include
 	 * applying workarounds.
 	 */
 	cache_init(pc);
@@ -555,13 +573,9 @@ sparc64_init(caddr_t mdp, u_long o1, u_long o2, u_long o3, ofw_vec_t *vec)
 	sun4u_set_traptable(tl0_base);
 
 	/*
-	 * Initialize the console.
-	 * NB: the low-level console drivers require a working DELAY() and
-	 * some compiler optimizations may cause the curthread accesses of
-	 * mutex(9) to be factored out even if the latter aren't actually
-	 * called, both requiring PCPU_REG to be set.
+	 * It's now safe to use the real DELAY().
 	 */
-	cninit();
+	delay_func = delay_tick;
 
 	/*
 	 * Initialize the dynamic per-CPU area for the BSP and the message
@@ -707,7 +721,7 @@ struct sigreturn_args {
  * MPSAFE
  */
 int
-sigreturn(struct thread *td, struct sigreturn_args *uap)
+sys_sigreturn(struct thread *td, struct sigreturn_args *uap)
 {
 	struct proc *p;
 	mcontext_t *mc;
@@ -976,7 +990,7 @@ ptrace_clear_single_step(struct thread *td)
 }
 
 void
-exec_setregs(struct thread *td, u_long entry, u_long stack, u_long ps_strings)
+exec_setregs(struct thread *td, struct image_params *imgp, u_long stack)
 {
 	struct trapframe *tf;
 	struct pcb *pcb;
@@ -999,12 +1013,8 @@ exec_setregs(struct thread *td, u_long entry, u_long stack, u_long ps_strings)
 	tf->tf_out[0] = stack;
 	tf->tf_out[3] = p->p_sysent->sv_psstrings;
 	tf->tf_out[6] = sp - SPOFF - sizeof(struct frame);
-	tf->tf_tnpc = entry + 4;
-	tf->tf_tpc = entry;
-	/*
-	 * While we could adhere to the memory model indicated in the ELF
-	 * header, it turns out that just always using TSO performs best.
-	 */
+	tf->tf_tnpc = imgp->entry_addr + 4;
+	tf->tf_tpc = imgp->entry_addr;
 	tf->tf_tstate = TSTATE_IE | TSTATE_PEF | TSTATE_MM_TSO;
 
 	td->td_retval[0] = tf->tf_out[0];
