@@ -38,6 +38,7 @@ __FBSDID("$FreeBSD$");
 #include <sys/sched.h>
 #include <sys/smp.h>
 #include <sys/sysctl.h>
+#include <sys/timeet.h>
 #include <sys/timetc.h>
 
 #include <dev/ofw/openfirm.h>
@@ -52,9 +53,6 @@ __FBSDID("$FreeBSD$");
 #include <machine/tick.h>
 #include <machine/ver.h>
 
-/* 10000 ticks proved okay for 500MHz. */
-#define	TICK_GRACE(clock)	((clock) / 1000000 * 2 * 10)
-
 #define	TICK_QUALITY_MP	10
 #define	TICK_QUALITY_UP	1000
 
@@ -76,29 +74,73 @@ static int adjust_ticks = 0;
 SYSCTL_INT(_machdep_tick, OID_AUTO, adjust_ticks, CTLFLAG_RD, &adjust_ticks,
     0, "total number of tick interrupts with adjustment");
 
-u_int hardclock_use_stick = 0;
-SYSCTL_INT(_machdep_tick, OID_AUTO, hardclock_use_stick, CTLFLAG_RD,
-    &hardclock_use_stick, 0, "hardclock uses STICK instead of TICK timer");
+u_int tick_et_use_stick = 0;
+SYSCTL_INT(_machdep_tick, OID_AUTO, tick_et_use_stick, CTLFLAG_RD,
+    &tick_et_use_stick, 0, "tick event timer uses STICK instead of TICK");
+
+typedef uint64_t rd_tick_t(void);
+static rd_tick_t *rd_tick;
+typedef void wr_tick_cmpr_t(uint64_t);
+static wr_tick_cmpr_t *wr_tick_cmpr;
 
 static struct timecounter stick_tc;
+static struct eventtimer tick_et;
 static struct timecounter tick_tc;
-static u_long tick_increment;
 
-static uint64_t tick_cputicks(void);
-static timecounter_get_t stick_get_timecount_up;
 #ifdef SMP
 static timecounter_get_t stick_get_timecount_mp;
 #endif
-static timecounter_get_t tick_get_timecount_up;
+static timecounter_get_t stick_get_timecount_up;
+static rd_tick_t stick_rd;
+static wr_tick_cmpr_t stick_wr_cmpr;
+static int tick_et_start(struct eventtimer *et, struct bintime *first,
+    struct bintime *period);
+static int tick_et_stop(struct eventtimer *et);
 #ifdef SMP
 static timecounter_get_t tick_get_timecount_mp;
 #endif
-static void tick_hardclock(struct trapframe *tf);
-static void tick_hardclock_bbwar(struct trapframe *tf);
-static inline void tick_hardclock_common(struct trapframe *tf, u_long tick,
-    u_long adj);
+static timecounter_get_t tick_get_timecount_up;
+static void tick_intr(struct trapframe *tf);
 static inline void tick_process(struct trapframe *tf);
-static void stick_hardclock(struct trapframe *tf);
+static rd_tick_t tick_rd;
+static wr_tick_cmpr_t tick_wr_cmpr;
+static wr_tick_cmpr_t tick_wr_cmpr_bbwar;
+static uint64_t tick_cputicks(void);
+
+static uint64_t
+stick_rd(void)
+{
+
+	return (rdstick());
+}
+
+static void
+stick_wr_cmpr(uint64_t tick)
+{
+
+	wrstickcmpr(tick, 0);
+}
+
+static uint64_t
+tick_rd(void)
+{
+
+	return (rd(tick));
+}
+
+static void
+tick_wr_cmpr(uint64_t tick_cmpr)
+{
+
+	wrtickcmpr(tick_cmpr, 0);
+}
+
+static void
+tick_wr_cmpr_bbwar(uint64_t tick_cmpr)
+{
+
+	wrtickcmpr_bbwar(tick_cmpr, 0);
+}
 
 static uint64_t
 tick_cputicks(void)
@@ -112,8 +154,6 @@ cpu_initclocks(void)
 {
 	uint32_t clock, sclock;
 
-	stathz = hz;
-
 	clock = PCPU_GET(clock);
 	sclock = 0;
 	if (PCPU_GET(impl) == CPU_IMPL_SPARC64V ||
@@ -128,27 +168,23 @@ cpu_initclocks(void)
 	 * Given that the STICK timers typically are driven at rather low
 	 * frequencies they shouldn't be used except when really necessary.
 	 */
-	if (hardclock_use_stick != 0) {
-		intr_setup(PIL_TICK, stick_hardclock, -1, NULL, NULL);
+	if (tick_et_use_stick != 0) {
+		rd_tick = stick_rd;
+		wr_tick_cmpr = stick_wr_cmpr;
 		/*
 		 * We don't provide a CPU ticker as long as the frequency
 		 * supplied isn't actually used per-CPU.
 		 */
 	} else {
-		intr_setup(PIL_TICK, PCPU_GET(impl) >= CPU_IMPL_ULTRASPARCI &&
-		    PCPU_GET(impl) < CPU_IMPL_ULTRASPARCIII ?
-		    tick_hardclock_bbwar : tick_hardclock, -1, NULL, NULL);
+		rd_tick = tick_rd;
+		if (PCPU_GET(impl) >= CPU_IMPL_ULTRASPARCI &&
+		    PCPU_GET(impl) < CPU_IMPL_ULTRASPARCIII)
+			wr_tick_cmpr = tick_wr_cmpr_bbwar;
+		else
+			wr_tick_cmpr = tick_wr_cmpr;
 		set_cputicker(tick_cputicks, clock, 0);
 	}
-	tick_increment = clock / hz;
-	/*
-	 * Avoid stopping of hardclock in terms of a lost (S)TICK interrupt
-	 * by ensuring that the (S)TICK period is at least TICK_GRACE ticks.
-	 */
-	if (tick_increment < TICK_GRACE(clock))
-		panic("%s: HZ too high, decrease to at least %d",
-		    __func__, clock / TICK_GRACE(clock));
-	tick_start();
+	intr_setup(PIL_TICK, tick_intr, -1, NULL, NULL);
 
 	/*
 	 * Initialize the (S)TICK-based timecounter(s).
@@ -186,108 +222,90 @@ cpu_initclocks(void)
 #endif
 		tc_init(&stick_tc);
 	}
+	tick_et.et_name = tick_et_use_stick ? "stick" : "tick";
+	tick_et.et_flags = ET_FLAGS_PERIODIC | ET_FLAGS_ONESHOT |
+	    ET_FLAGS_PERCPU;
+	tick_et.et_quality = 1000;
+	tick_et.et_frequency = tick_et_use_stick ? sclock : clock;
+	tick_et.et_min_period.sec = 0;
+	tick_et.et_min_period.frac = 0x00010000LLU << 32; /* To be safe. */
+	tick_et.et_max_period.sec = 3600 * 24; /* No practical limit. */
+	tick_et.et_max_period.frac = 0;
+	tick_et.et_start = tick_et_start;
+	tick_et.et_stop = tick_et_stop;
+	tick_et.et_priv = NULL;
+	et_register(&tick_et);
+
+	cpu_initclocks_bsp();
 }
 
 static inline void
 tick_process(struct trapframe *tf)
 {
-
-	if (curcpu == 0)
-		hardclock(TRAPF_USERMODE(tf), TRAPF_PC(tf));
-	else
-		hardclock_cpu(TRAPF_USERMODE(tf));
-	if (profprocs != 0)
-		profclock(TRAPF_USERMODE(tf), TRAPF_PC(tf));
-	statclock(TRAPF_USERMODE(tf));
-}
-
-/*
- * NB: the sequence of reading the (S)TICK register, calculating the value
- * of the next tick and writing it to the (S)TICK_COMPARE register must not
- * be interrupted, not even by an IPI, otherwise a value that is in the past
- * could be written in the worst case, causing hardclock to stop.
- */
-
-static void
-tick_hardclock(struct trapframe *tf)
-{
-	u_long adj, tick;
-	register_t s;
+	struct trapframe *oldframe;
+	struct thread *td;
 
 	critical_enter();
-	adj = PCPU_GET(tickadj);
-	s = intr_disable();
-	tick = rd(tick);
-	wrtickcmpr(tick + tick_increment - adj, 0);
-	intr_restore(s);
-	tick_hardclock_common(tf, tick, adj);
+	if (tick_et.et_active) {
+		td = curthread;
+		oldframe = td->td_intr_frame;
+		td->td_intr_frame = tf;
+		tick_et.et_event_cb(&tick_et, tick_et.et_arg);
+		td->td_intr_frame = oldframe;
+	}
 	critical_exit();
 }
 
 static void
-tick_hardclock_bbwar(struct trapframe *tf)
+tick_intr(struct trapframe *tf)
 {
-	u_long adj, tick;
-	register_t s;
-
-	critical_enter();
-	adj = PCPU_GET(tickadj);
-	s = intr_disable();
-	tick = rd(tick);
-	wrtickcmpr_bbwar(tick + tick_increment - adj, 0);
-	intr_restore(s);
-	tick_hardclock_common(tf, tick, adj);
-	critical_exit();
-}
-
-static void
-stick_hardclock(struct trapframe *tf)
-{
-	u_long adj, stick;
-	register_t s;
-
-	critical_enter();
-	adj = PCPU_GET(tickadj);
-	s = intr_disable();
-	stick = rdstick();
-	wrstickcmpr(stick + tick_increment - adj, 0);
-	intr_restore(s);
-	tick_hardclock_common(tf, stick, adj);
-	critical_exit();
-}
-
-static inline void
-tick_hardclock_common(struct trapframe *tf, u_long tick, u_long adj)
-{
-	u_long ref;
+	u_long adj, ref, tick, tick_increment;
 	long delta;
+	register_t s;
 	int count;
 
-	ref = PCPU_GET(tickref);
-	delta = tick - ref;
-	count = 0;
-	while (delta >= tick_increment) {
-		tick_process(tf);
-		delta -= tick_increment;
-		ref += tick_increment;
-		if (adj != 0)
-			adjust_ticks++;
-		count++;
-	}
-	if (count > 0) {
-		adjust_missed += count - 1;
-		if (delta > (tick_increment >> 3)) {
-			if (adj == 0)
-				adjust_edges++;
-			adj = tick_increment >> 4;
-		} else
+	tick_increment = PCPU_GET(tickincrement);
+	if (tick_increment != 0) {
+		/*
+		 * NB: the sequence of reading the (S)TICK register,
+		 * calculating the value of the next tick and writing it to
+		 * the (S)TICK_COMPARE register must not be interrupted, not
+		 * even by an IPI, otherwise a value that is in the past could
+		 * be written in the worst case and thus causing the periodic
+		 * timer to stop.
+		 */
+		s = intr_disable();
+		adj = PCPU_GET(tickadj);
+		tick = rd_tick();
+		wr_tick_cmpr(tick + tick_increment - adj);
+		intr_restore(s);
+		ref = PCPU_GET(tickref);
+		delta = tick - ref;
+		count = 0;
+		while (delta >= tick_increment) {
+			tick_process(tf);
+			delta -= tick_increment;
+			ref += tick_increment;
+			if (adj != 0)
+				adjust_ticks++;
+			count++;
+		}
+		if (count > 0) {
+			adjust_missed += count - 1;
+			if (delta > (tick_increment >> 3)) {
+				if (adj == 0)
+					adjust_edges++;
+				adj = tick_increment >> 4;
+			} else
+				adj = 0;
+		} else {
 			adj = 0;
-	} else {
-		adj = 0;
-		adjust_excess++;
-	}
-	PCPU_SET(tickref, ref);
-	PCPU_SET(tickadj, adj);
+			adjust_excess++;
+		}
+		PCPU_SET(tickref, ref);
+		PCPU_SET(tickadj, adj);
+	} else
+		tick_process(tf);
 }
 
 static u_int
@@ -334,32 +352,51 @@ tick_get_timecount_mp(struct timecounter *tc)
 }
 #endif
 
-void
-tick_start(void)
+static int
+tick_et_start(struct eventtimer *et, struct bintime *first,
+    struct bintime *period)
 {
-	u_long base;
+	u_long base, div, fdiv;
 	register_t s;
 
+	if (period != NULL) {
+		div = (tick_et.et_frequency * (period->frac >> 32)) >> 32;
+		if (period->sec != 0)
+			div += tick_et.et_frequency * period->sec;
+	} else
+		div = 0;
+	if (first != NULL) {
+		fdiv = (tick_et.et_frequency * (first->frac >> 32)) >> 32;
+		if (first->sec != 0)
+			fdiv += tick_et.et_frequency * first->sec;
+	} else
+		fdiv = div;
+	PCPU_SET(tickincrement, div);
+
 	/*
 	 * Try to make the (S)TICK interrupts as synchronously as possible
 	 * on all CPUs to avoid inaccuracies for migrating processes.  Leave
 	 * out one tick to make sure that it is not missed.
 	 */
-	critical_enter();
-	PCPU_SET(tickadj, 0);
 	s = intr_disable();
-	if (hardclock_use_stick != 0)
-		base = rdstick();
-	else
-		base = rd(tick);
-	base = roundup(base, tick_increment);
+	base = rd_tick();
+	if (div != 0) {
+		PCPU_SET(tickadj, 0);
+		base = roundup(base, div);
+	}
 	PCPU_SET(tickref, base);
-	if (hardclock_use_stick != 0)
-		wrstickcmpr(base + tick_increment, 0);
-	else
-		wrtickcmpr(base + tick_increment, 0);
+	wr_tick_cmpr(base + fdiv);
 	intr_restore(s);
-	critical_exit();
+	return (0);
+}
+
+static int
+tick_et_stop(struct eventtimer *et)
+{
+
+	PCPU_SET(tickincrement, 0);
+	tick_stop(PCPU_GET(impl));
+	return (0);
 }
 
 void
