@@ -62,8 +62,6 @@
 #define __GNUCLIKE___OFFSETOF 1
 #define __GNUCLIKE___SECTION 1
 
-#define __GNUCLIKE_ATTRIBUTE_MODE_DI 1
-
 #ifndef __INTEL_COMPILER
 # define __GNUCLIKE_CTOR_SECTION_HANDLING 1
 #endif
@@ -234,12 +232,6 @@
 #define	__always_inline
 #endif
 
-#if __GNUC_PREREQ__(4, 2) /* actually 4.1.3 */
-#define	__gnu89_inline	__attribute__((__gnu_inline__)) __inline
-#else
-#define	__gnu89_inline
-#endif
-
 #if __GNUC_PREREQ__(3, 1)
 #define	__noinline	__attribute__ ((__noinline__))
 #else
@@ -396,6 +388,9 @@
 #endif	/* __STDC__ */
 #endif	/* __GNUC__ || __INTEL_COMPILER */
 
+#define	__GLOBL1(sym)	__asm__(".globl " #sym)
+#define	__GLOBL(sym)	__GLOBL1(sym)
+
 #if defined(__GNUC__) || defined(__INTEL_COMPILER)
 #define	__IDSTRING(name,string)	__asm__(".ident\t\"" string "\"")
 #else
