@@ -88,7 +88,7 @@ int	__elfN(brand_inuse)(Elf_Brandinfo *entry);
 int	__elfN(insert_brand_entry)(Elf_Brandinfo *entry);
 int	__elfN(remove_brand_entry)(Elf_Brandinfo *entry);
 int	__elfN(freebsd_fixup)(register_t **, struct image_params *);
-int	__elfN(coredump)(struct thread *, struct vnode *, off_t);
+int	__elfN(coredump)(struct thread *, struct vnode *, off_t, int);
 
 /* Machine specific function to dump per-thread information. */
 void	__elfN(dump_thread)(struct thread *, void *, size_t *);
