@@ -83,10 +83,11 @@
 	__PMC_CPU(INTEL_CORE,	0x87,	"Intel Core Solo/Duo")	\
 	__PMC_CPU(INTEL_CORE2,	0x88,	"Intel Core2")		\
 	__PMC_CPU(INTEL_CORE2EXTREME,	0x89,	"Intel Core2 Extreme")	\
-	__PMC_CPU(INTEL_ATOM,	0x8A,	"Intel Atom")		\
-	__PMC_CPU(INTEL_COREI7, 0x8B,   "Intel Core i7")	\
-	__PMC_CPU(INTEL_WESTMERE, 0x8C,   "Intel Westmere")	\
- 	__PMC_CPU(MIPS_24K,     0x200,  "MIPS 24K") 
+	__PMC_CPU(INTEL_ATOM,	0x8A,	"Intel Atom") \
+	__PMC_CPU(INTEL_COREI7, 0x8B,   "Intel Core i7") \
+	__PMC_CPU(INTEL_WESTMERE, 0x8C,   "Intel Westmere") \
+	__PMC_CPU(INTEL_XSCALE,	0x100,	"Intel XScale") \
+	__PMC_CPU(MIPS_24K,     0x200,  "MIPS 24K") 
 
 enum pmc_cputype {
 #undef	__PMC_CPU
@@ -109,9 +110,10 @@ enum pmc_cputype {
 	__PMC_CLASS(P6)		/* Intel Pentium Pro counters */	\
 	__PMC_CLASS(P4)		/* Intel Pentium-IV counters */		\
 	__PMC_CLASS(IAF)	/* Intel Core2/Atom, fixed function */	\
-	__PMC_CLASS(IAP)	/* Intel Core...Atom, programmable */   \
+	__PMC_CLASS(IAP)	/* Intel Core...Atom, programmable */	\
 	__PMC_CLASS(UCF)	/* Intel Uncore fixed function */	\
 	__PMC_CLASS(UCP)	/* Intel Uncore programmable */		\
+	__PMC_CLASS(XSCALE)	/* Intel XScale counters */		\
 	__PMC_CLASS(MIPS24K)    /* MIPS 24K */
 
 enum pmc_class {
@@ -299,7 +301,7 @@ enum pmc_event {
 	__PMC_OP(PMCRW, "Read/Set a PMC")				\
 	__PMC_OP(PMCSETCOUNT, "Set initial count/sampling rate")	\
 	__PMC_OP(PMCSTART, "Start a PMC")				\
-	__PMC_OP(PMCSTOP, "Start a PMC")				\
+	__PMC_OP(PMCSTOP, "Stop a PMC")					\
 	__PMC_OP(WRITELOG, "Write a cookie to the log file")
 
 
