@@ -96,9 +96,11 @@ vm_contig_launder_page(vm_page_t m, vm_page_t *next)
 	int vfslocked;
 
 	mtx_assert(&vm_page_queue_mtx, MA_OWNED);
+	vm_page_lock_assert(m, MA_OWNED);
 	object = m->object;
 	if (!VM_OBJECT_TRYLOCK(object) &&
-	    !vm_pageout_fallback_object_lock(m, next)) {
+	    (!vm_pageout_fallback_object_lock(m, next) || m->hold_count != 0)) {
+		vm_page_unlock(m);
 		VM_OBJECT_UNLOCK(object);
 		return (EAGAIN);
 	}
@@ -108,9 +110,10 @@ vm_contig_launder_page(vm_page_t m, vm_page_t *next)
 		return (EBUSY);
 	}
 	vm_page_test_dirty(m);
-	if (m->dirty == 0 && m->hold_count == 0)
+	if (m->dirty == 0)
 		pmap_remove_all(m);
-	if (m->dirty) {
+	if (m->dirty != 0) {
+		vm_page_unlock(m);
 		if ((object->flags & OBJ_DEAD) != 0) {
 			VM_OBJECT_UNLOCK(object);
 			return (EAGAIN);
@@ -134,13 +137,17 @@ vm_contig_launder_page(vm_page_t m, vm_page_t *next)
 			return (0);
 		} else if (object->type == OBJT_SWAP ||
 			   object->type == OBJT_DEFAULT) {
+			vm_page_unlock_queues();
 			m_tmp = m;
 			vm_pageout_flush(&m_tmp, 1, VM_PAGER_PUT_SYNC, 0, NULL);
 			VM_OBJECT_UNLOCK(object);
+			vm_page_lock_queues();
 			return (0);
 		}
-	} else if (m->hold_count == 0)
+	} else {
 		vm_page_cache(m);
+		vm_page_unlock(m);
+	}
 	VM_OBJECT_UNLOCK(object);
 	return (0);
 }
@@ -162,9 +169,14 @@ vm_contig_launder(int queue, vm_paddr_t low, vm_paddr_t high)
 		if (pa < low || pa + PAGE_SIZE > high)
 			continue;
 
-		KASSERT(VM_PAGE_INQUEUE2(m, queue),
+		if (!vm_pageout_page_lock(m, &next) || m->hold_count != 0) {
+			vm_page_unlock(m);
+			continue;
+		}
+		KASSERT(m->queue == queue,
 		    ("vm_contig_launder: page %p's queue is not %d", m, queue));
 		error = vm_contig_launder_page(m, &next);
+		vm_page_lock_assert(m, MA_NOTOWNED);
 		if (error == 0)
 			return (TRUE);
 		if (error == EBUSY)
@@ -260,9 +272,7 @@ retry:
 				i -= PAGE_SIZE;
 				m = vm_page_lookup(object, OFF_TO_IDX(offset +
 				    i));
-				vm_page_lock_queues();
 				vm_page_free(m);
-				vm_page_unlock_queues();
 			}
 			VM_OBJECT_UNLOCK(object);
 			vm_map_delete(map, addr, addr + size);
