@@ -53,16 +53,11 @@ __FBSDID("$FreeBSD$");
 #include <machine/segments.h>
 
 #include <dev/ic/i8259.h>
-#ifdef __amd64__
-#include <amd64/isa/icu.h>
-#include <amd64/isa/isa.h>
-#else
-#include <i386/isa/icu.h>
+#include <x86/isa/icu.h>
 #ifdef PC98
 #include <pc98/cbus/cbus.h>
 #else
-#include <i386/isa/isa.h>
-#endif
+#include <x86/isa/isa.h>
 #endif
 #include <isa/isavar.h>
 
