@@ -49,17 +49,18 @@ __FBSDID("$FreeBSD$");
 #include <sys/proc.h>
 #include <sys/sched.h>
 #include <sys/smp.h>
+#include <sys/timeet.h>
 
 #include <vm/vm.h>
 #include <vm/pmap.h>
 
-#include <machine/apicreg.h>
+#include <x86/apicreg.h>
 #include <machine/cpu.h>
 #include <machine/cputypes.h>
 #include <machine/frame.h>
 #include <machine/intr_machdep.h>
 #include <machine/apicvar.h>
-#include <machine/mca.h>
+#include <x86/mca.h>
 #include <machine/md_var.h>
 #include <machine/smp.h>
 #include <machine/specialreg.h>
@@ -79,11 +80,6 @@ __FBSDID("$FreeBSD$");
 #define	GSEL_APIC	GSEL(GCODE_SEL, SEL_KPL)
 #endif
 
-#ifdef KDTRACE_HOOKS
-#include <sys/dtrace_bsd.h>
-cyclic_clock_func_t	cyclic_clock_func[MAXCPU];
-#endif
-
 /* Sanity checks on IDT vectors. */
 CTASSERT(APIC_IO_INTS + APIC_NUM_IOINTS == APIC_TIMER_INT);
 CTASSERT(APIC_TIMER_INT < APIC_LOCAL_INTS);
@@ -93,6 +89,7 @@ CTASSERT(IPI_STOP < APIC_SPURIOUS_INT);
 /* Magic IRQ values for the timer and syscalls. */
 #define	IRQ_TIMER	(NUM_IO_INTS + 1)
 #define	IRQ_SYSCALL	(NUM_IO_INTS + 2)
+#define	IRQ_DTRACE_RET	(NUM_IO_INTS + 3)
 
 /*
  * Support for local APICs.  Local APICs manage interrupts on each
@@ -119,9 +116,8 @@ struct lapic {
 	u_int la_cluster_id:2;
 	u_int la_present:1;
 	u_long *la_timer_count;
-	u_long la_hard_ticks;
-	u_long la_stat_ticks;
-	u_long la_prof_ticks;
+	u_long la_timer_period;
+	u_int la_timer_mode;
 	/* Include IDT_SYSCALL to make indexing easier. */
 	int la_ioint_irqs[APIC_NUM_IOINTS + 1];
 } static lapics[MAX_APIC_ID + 1];
@@ -158,16 +154,19 @@ extern inthand_t IDTVEC(rsvd);
 
 volatile lapic_t *lapic;
 vm_paddr_t lapic_paddr;
-static u_long lapic_timer_divisor, lapic_timer_period, lapic_timer_hz;
-static enum lapic_clock clockcoverage;
+static u_long lapic_timer_divisor;
+static struct eventtimer lapic_et;
 
 static void	lapic_enable(void);
 static void	lapic_resume(struct pic *pic);
-static void	lapic_timer_enable_intr(void);
-static void	lapic_timer_oneshot(u_int count);
-static void	lapic_timer_periodic(u_int count);
+static void	lapic_timer_oneshot(u_int count, int enable_int);
+static void	lapic_timer_periodic(u_int count, int enable_int);
+static void	lapic_timer_stop(void);
 static void	lapic_timer_set_divisor(u_int divisor);
 static uint32_t	lvt_mode(struct lapic *la, u_int pin, uint32_t value);
+static int	lapic_et_start(struct eventtimer *et,
+    struct bintime *first, struct bintime *period);
+static int	lapic_et_stop(struct eventtimer *et);
 
 struct pic lapic_pic = { .pic_resume = lapic_resume };
 
@@ -218,6 +217,8 @@ lvt_mode(struct lapic *la, u_int pin, uint32_t value)
 void
 lapic_init(vm_paddr_t addr)
 {
+	u_int regs[4];
+	int i, arat;
 
 	/* Map the local APIC and setup the spurious interrupt handler. */
 	KASSERT(trunc_page(addr) == addr,
@@ -242,8 +243,36 @@ lapic_init(vm_paddr_t addr)
 	/* XXX: Thermal interrupt */
 
 	/* Local APIC CMCI. */
-	setidt(APIC_CMC_INT, IDTVEC(cmcint), SDT_APICT, SEL_KPL,
-	    GSEL(GCODE_SEL, SEL_KPL));
+	setidt(APIC_CMC_INT, IDTVEC(cmcint), SDT_APICT, SEL_KPL, GSEL_APIC);
+
+	if ((resource_int_value("apic", 0, "clock", &i) != 0 || i != 0)) {
+		arat = 0;
+		/* Intel CPUID 0x06 EAX[2] set if APIC timer runs in C3. */
+		if (cpu_vendor_id == CPU_VENDOR_INTEL && cpu_high >= 6) {
+			do_cpuid(0x06, regs);
+			if ((regs[0] & CPUTPM1_ARAT) != 0)
+				arat = 1;
+		}
+		bzero(&lapic_et, sizeof(lapic_et));
+		lapic_et.et_name = "LAPIC";
+		lapic_et.et_flags = ET_FLAGS_PERIODIC | ET_FLAGS_ONESHOT |
+		    ET_FLAGS_PERCPU;
+		lapic_et.et_quality = 600;
+		if (!arat) {
+			lapic_et.et_flags |= ET_FLAGS_C3STOP;
+			lapic_et.et_quality -= 200;
+		}
+		lapic_et.et_frequency = 0;
+		/* We don't know frequency yet, so trying to guess. */
+		lapic_et.et_min_period.sec = 0;
+		lapic_et.et_min_period.frac = 0x00001000LL << 32;
+		lapic_et.et_max_period.sec = 1;
+		lapic_et.et_max_period.frac = 0;
+		lapic_et.et_start = lapic_et_start;
+		lapic_et.et_stop = lapic_et_stop;
+		lapic_et.et_priv = NULL;
+		et_register(&lapic_et);
+	}
 }
 
 /*
@@ -278,6 +307,10 @@ lapic_create(u_int apic_id, int boot_cpu)
 	lapics[apic_id].la_ioint_irqs[IDT_SYSCALL - APIC_IO_INTS] = IRQ_SYSCALL;
 	lapics[apic_id].la_ioint_irqs[APIC_TIMER_INT - APIC_IO_INTS] =
 	    IRQ_TIMER;
+#ifdef KDTRACE_HOOKS
+	lapics[apic_id].la_ioint_irqs[IDT_DTRACE_RET - APIC_IO_INTS] = IRQ_DTRACE_RET;
+#endif
+
 
 #ifdef SMP
 	cpu_add(apic_id, boot_cpu);
@@ -337,17 +370,19 @@ lapic_setup(int boot)
 	/* Program timer LVT and setup handler. */
 	lapic->lvt_timer = lvt_mode(la, LVT_TIMER, lapic->lvt_timer);
 	if (boot) {
-		snprintf(buf, sizeof(buf), "cpu%d: timer", PCPU_GET(cpuid));
+		snprintf(buf, sizeof(buf), "cpu%d:timer", PCPU_GET(cpuid));
 		intrcnt_add(buf, &la->la_timer_count);
 	}
 
-	/* We don't setup the timer during boot on the BSP until later. */
-	if (!(boot && PCPU_GET(cpuid) == 0) && lapic_timer_hz != 0) {
-		KASSERT(lapic_timer_period != 0, ("lapic%u: zero divisor",
+	/* Setup the timer if configured. */
+	if (la->la_timer_mode != 0) {
+		KASSERT(la->la_timer_period != 0, ("lapic%u: zero divisor",
 		    lapic_id()));
 		lapic_timer_set_divisor(lapic_timer_divisor);
-		lapic_timer_periodic(lapic_timer_period);
-		lapic_timer_enable_intr();
+		if (la->la_timer_mode == 1)
+			lapic_timer_periodic(la->la_timer_period, 1);
+		else
+			lapic_timer_oneshot(la->la_timer_period, 1);
 	}
 
 	/* Program error LVT and clear any existing errors. */
@@ -445,86 +480,67 @@ lapic_disable_pmc(void)
 #endif
 }
 
-/*
- * Called by cpu_initclocks() on the BSP to setup the local APIC timer so
- * that it can drive hardclock, statclock, and profclock. 
- */
-enum lapic_clock
-lapic_setup_clock(enum lapic_clock srcsdes)
+static int
+lapic_et_start(struct eventtimer *et,
+    struct bintime *first, struct bintime *period)
 {
+	struct lapic *la;
 	u_long value;
-	int i;
 
-	/* lapic_setup_clock() should not be called with LAPIC_CLOCK_NONE. */
-	MPASS(srcsdes != LAPIC_CLOCK_NONE);
-
-	/* Can't drive the timer without a local APIC. */
-	if (lapic == NULL ||
-	    (resource_int_value("apic", 0, "clock", &i) == 0 && i == 0)) {
-		clockcoverage = LAPIC_CLOCK_NONE;
-		return (clockcoverage);
+	if (et->et_frequency == 0) {
+		/* Start off with a divisor of 2 (power on reset default). */
+		lapic_timer_divisor = 2;
+		/* Try to calibrate the local APIC timer. */
+		do {
+			lapic_timer_set_divisor(lapic_timer_divisor);
+			lapic_timer_oneshot(APIC_TIMER_MAX_COUNT, 0);
+			DELAY(1000000);
+			value = APIC_TIMER_MAX_COUNT - lapic->ccr_timer;
+			if (value != APIC_TIMER_MAX_COUNT)
+				break;
+			lapic_timer_divisor <<= 1;
+		} while (lapic_timer_divisor <= 128);
+		if (lapic_timer_divisor > 128)
+			panic("lapic: Divisor too big");
+		if (bootverbose)
+			printf("lapic: Divisor %lu, Frequency %lu Hz\n",
+			    lapic_timer_divisor, value);
+		et->et_frequency = value;
+		et->et_min_period.sec = 0;
+		et->et_min_period.frac =
+		    ((0x00000002LLU << 32) / et->et_frequency) << 32;
+		et->et_max_period.sec = 0xfffffffeLLU / et->et_frequency;
+		et->et_max_period.frac =
+		    ((0xfffffffeLLU << 32) / et->et_frequency) << 32;
 	}
-
-	/* Start off with a divisor of 2 (power on reset default). */
-	lapic_timer_divisor = 2;
-
-	/* Try to calibrate the local APIC timer. */
-	do {
-		lapic_timer_set_divisor(lapic_timer_divisor);
-		lapic_timer_oneshot(APIC_TIMER_MAX_COUNT);
-		DELAY(2000000);
-		value = APIC_TIMER_MAX_COUNT - lapic->ccr_timer;
-		if (value != APIC_TIMER_MAX_COUNT)
-			break;
-		lapic_timer_divisor <<= 1;
-	} while (lapic_timer_divisor <= 128);
-	if (lapic_timer_divisor > 128)
-		panic("lapic: Divisor too big");
-	value /= 2;
-	if (bootverbose)
-		printf("lapic: Divisor %lu, Frequency %lu Hz\n",
-		    lapic_timer_divisor, value);
-
-	/*
-	 * We want to run stathz in the neighborhood of 128hz.  We would
-	 * like profhz to run as often as possible, so we let it run on
-	 * each clock tick.  We try to honor the requested 'hz' value as
-	 * much as possible.
-	 *
-	 * If 'hz' is above 1500, then we just let the lapic timer
-	 * (and profhz) run at hz.  If 'hz' is below 1500 but above
-	 * 750, then we let the lapic timer run at 2 * 'hz'.  If 'hz'
-	 * is below 750 then we let the lapic timer run at 4 * 'hz'.
-	 *
-	 * Please note that stathz and profhz are set only if all the
-	 * clocks are handled through the local APIC.
-	 */
-	if (srcsdes == LAPIC_CLOCK_ALL) {
-		if (hz >= 1500)
-			lapic_timer_hz = hz;
-		else if (hz >= 750)
-			lapic_timer_hz = hz * 2;
-		else
-			lapic_timer_hz = hz * 4;
-	} else
-		lapic_timer_hz = hz;
-	lapic_timer_period = value / lapic_timer_hz;
-	if (srcsdes == LAPIC_CLOCK_ALL) {
-		if (lapic_timer_hz < 128)
-			stathz = lapic_timer_hz;
-		else
-			stathz = lapic_timer_hz / (lapic_timer_hz / 128);
-		profhz = lapic_timer_hz;
+	lapic_timer_set_divisor(lapic_timer_divisor);
+	la = &lapics[lapic_id()];
+	if (period != NULL) {
+		la->la_timer_mode = 1;
+		la->la_timer_period =
+		    (et->et_frequency * (period->frac >> 32)) >> 32;
+		if (period->sec != 0)
+			la->la_timer_period += et->et_frequency * period->sec;
+		lapic_timer_periodic(la->la_timer_period, 1);
+	} else {
+		la->la_timer_mode = 2;
+		la->la_timer_period =
+		    (et->et_frequency * (first->frac >> 32)) >> 32;
+		if (first->sec != 0)
+			la->la_timer_period += et->et_frequency * first->sec;
+		lapic_timer_oneshot(la->la_timer_period, 1);
 	}
+	return (0);
+}
 
-	/*
-	 * Start up the timer on the BSP.  The APs will kick off their
-	 * timer during lapic_setup().
-	 */
-	lapic_timer_periodic(lapic_timer_period);
-	lapic_timer_enable_intr();
-	clockcoverage = srcsdes;
-	return (srcsdes);
+static int
+lapic_et_stop(struct eventtimer *et)
+{
+	struct lapic *la = &lapics[lapic_id()];
+
+	la->la_timer_mode = 0;
+	lapic_timer_stop();
+	return (0);
 }
 
 void
@@ -759,8 +775,6 @@ lapic_handle_intr(int vector, struct trapframe *frame)
 {
 	struct intsrc *isrc;
 
-	if (vector == -1)
-		panic("Couldn't get vector from ISR!");
 	isrc = intr_lookup_source(apic_idt_to_irq(PCPU_GET(apic_id),
 	    vector));
 	intr_execute_handlers(isrc, frame);
@@ -770,6 +784,8 @@ void
 lapic_handle_timer(struct trapframe *frame)
 {
 	struct lapic *la;
+	struct trapframe *oldframe;
+	struct thread *td;
 
 	/* Send EOI first thing. */
 	lapic_eoi();
@@ -786,7 +802,7 @@ lapic_handle_timer(struct trapframe *frame)
 	 * and unlike other schedulers it actually schedules threads to
 	 * those CPUs.
 	 */
-	if ((hlt_cpus_mask & (1 << PCPU_GET(cpuid))) != 0)
+	if (CPU_ISSET(PCPU_GET(cpuid), &hlt_cpus_mask))
 		return;
 #endif
 
@@ -794,44 +810,14 @@ lapic_handle_timer(struct trapframe *frame)
 	la = &lapics[PCPU_GET(apic_id)];
 	(*la->la_timer_count)++;
 	critical_enter();
-
-#ifdef KDTRACE_HOOKS
-	/*
-	 * If the DTrace hooks are configured and a callback function
-	 * has been registered, then call it to process the high speed
-	 * timers.
-	 */
-	int cpu = PCPU_GET(cpuid);
-	if (cyclic_clock_func[cpu] != NULL)
-		(*cyclic_clock_func[cpu])(frame);
-#endif
-
-	/* Fire hardclock at hz. */
-	la->la_hard_ticks += hz;
-	if (la->la_hard_ticks >= lapic_timer_hz) {
-		la->la_hard_ticks -= lapic_timer_hz;
-		if (PCPU_GET(cpuid) == 0)
-			hardclock(TRAPF_USERMODE(frame), TRAPF_PC(frame));
-		else
-			hardclock_cpu(TRAPF_USERMODE(frame));
-	}
-	if (clockcoverage == LAPIC_CLOCK_ALL) {
-
-		/* Fire statclock at stathz. */
-		la->la_stat_ticks += stathz;
-		if (la->la_stat_ticks >= lapic_timer_hz) {
-			la->la_stat_ticks -= lapic_timer_hz;
-			statclock(TRAPF_USERMODE(frame));
-		}
-
-		/* Fire profclock at profhz, but only when needed. */
-		la->la_prof_ticks += profhz;
-		if (la->la_prof_ticks >= lapic_timer_hz) {
-			la->la_prof_ticks -= lapic_timer_hz;
-			if (profprocs != 0)
-				profclock(TRAPF_USERMODE(frame),
-				    TRAPF_PC(frame));
-		}
+	if (lapic_et.et_active) {
+		td = curthread;
+		td->td_intr_nesting_level++;
+		oldframe = td->td_intr_frame;
+		td->td_intr_frame = frame;
+		lapic_et.et_event_cb(&lapic_et, lapic_et.et_arg);
+		td->td_intr_frame = oldframe;
+		td->td_intr_nesting_level--;
 	}
 	critical_exit();
 }
@@ -847,36 +833,41 @@ lapic_timer_set_divisor(u_int divisor)
 }
 
 static void
-lapic_timer_oneshot(u_int count)
+lapic_timer_oneshot(u_int count, int enable_int)
 {
 	u_int32_t value;
 
 	value = lapic->lvt_timer;
 	value &= ~APIC_LVTT_TM;
 	value |= APIC_LVTT_TM_ONE_SHOT;
+	if (enable_int)
+		value &= ~APIC_LVT_M;
 	lapic->lvt_timer = value;
 	lapic->icr_timer = count;
 }
 
 static void
-lapic_timer_periodic(u_int count)
+lapic_timer_periodic(u_int count, int enable_int)
 {
 	u_int32_t value;
 
 	value = lapic->lvt_timer;
 	value &= ~APIC_LVTT_TM;
 	value |= APIC_LVTT_TM_PERIODIC;
+	if (enable_int)
+		value &= ~APIC_LVT_M;
 	lapic->lvt_timer = value;
 	lapic->icr_timer = count;
 }
 
 static void
-lapic_timer_enable_intr(void)
+lapic_timer_stop(void)
 {
 	u_int32_t value;
 
 	value = lapic->lvt_timer;
-	value &= ~APIC_LVT_M;
+	value &= ~APIC_LVTT_TM;
+	value |= APIC_LVT_M;
 	lapic->lvt_timer = value;
 }
 
@@ -1033,6 +1024,10 @@ apic_enable_vector(u_int apic_id, u_int vector)
 	KASSERT(vector != IDT_SYSCALL, ("Attempt to overwrite syscall entry"));
 	KASSERT(ioint_handlers[vector / 32] != NULL,
 	    ("No ISR handler for vector %u", vector));
+#ifdef KDTRACE_HOOKS
+	KASSERT(vector != IDT_DTRACE_RET,
+	    ("Attempt to overwrite DTrace entry"));
+#endif
 	setidt(vector, ioint_handlers[vector / 32], SDT_APIC, SEL_KPL,
 	    GSEL_APIC);
 }
@@ -1042,6 +1037,10 @@ apic_disable_vector(u_int apic_id, u_int vector)
 {
 
 	KASSERT(vector != IDT_SYSCALL, ("Attempt to overwrite syscall entry"));
+#ifdef KDTRACE_HOOKS
+	KASSERT(vector != IDT_DTRACE_RET,
+	    ("Attempt to overwrite DTrace entry"));
+#endif
 	KASSERT(ioint_handlers[vector / 32] != NULL,
 	    ("No ISR handler for vector %u", vector));
 #ifdef notyet
@@ -1065,6 +1064,10 @@ apic_free_vector(u_int apic_id, u_int vector, u_int irq)
 	KASSERT(irq < NUM_IO_INTS, ("Invalid IRQ %u", irq));
 	KASSERT(lapics[apic_id].la_ioint_irqs[vector - APIC_IO_INTS] ==
 	    irq, ("IRQ mismatch"));
+#ifdef KDTRACE_HOOKS
+	KASSERT(vector != IDT_DTRACE_RET,
+	    ("Attempt to overwrite DTrace entry"));
+#endif
 
 	/*
 	 * Bind us to the cpu that owned the vector before freeing it so
@@ -1097,6 +1100,10 @@ apic_idt_to_irq(u_int apic_id, u_int vector)
 	KASSERT(vector >= APIC_IO_INTS && vector != IDT_SYSCALL &&
 	    vector <= APIC_IO_INTS + APIC_NUM_IOINTS,
 	    ("Vector %u does not map to an IRQ line", vector));
+#ifdef KDTRACE_HOOKS
+	KASSERT(vector != IDT_DTRACE_RET,
+	    ("Attempt to overwrite DTrace entry"));
+#endif
 	irq = lapics[apic_id].la_ioint_irqs[vector - APIC_IO_INTS];
 	if (irq < 0)
 		irq = 0;
@@ -1128,6 +1135,10 @@ DB_SHOW_COMMAND(apic, db_show_apic)
 			irq = lapics[apic_id].la_ioint_irqs[i];
 			if (irq == -1 || irq == IRQ_SYSCALL)
 				continue;
+#ifdef KDTRACE_HOOKS
+			if (irq == IRQ_DTRACE_RET)
+				continue;
+#endif
 			db_printf("vec 0x%2x -> ", i + APIC_IO_INTS);
 			if (irq == IRQ_TIMER)
 				db_printf("lapic timer\n");
@@ -1260,7 +1271,7 @@ apic_init(void *dummy __unused)
 	if (resource_disabled("apic", 0))
 		return;
 
-	/* First, probe all the enumerators to find the best match. */
+	/* Probe all the enumerators to find the best match. */
 	best_enum = NULL;
 	best = 0;
 	SLIST_FOREACH(enumerator, &enumerators, apic_next) {
@@ -1296,13 +1307,12 @@ apic_init(void *dummy __unused)
 	}
 #endif
 
-	/* Second, probe the CPU's in the system. */
+	/* Probe the CPU's in the system. */
 	retval = best_enum->apic_probe_cpus();
 	if (retval != 0)
 		printf("%s: Failed to probe CPUs: returned %d\n",
 		    best_enum->apic_name, retval);
 
-#ifdef __amd64__
 }
 SYSINIT(apic_init, SI_SUB_TUNABLES - 1, SI_ORDER_SECOND, apic_init, NULL);
 
@@ -1317,19 +1327,14 @@ apic_setup_local(void *dummy __unused)
  
 	if (best_enum == NULL)
 		return;
-#endif
-	/* Third, initialize the local APIC. */
+
+	/* Initialize the local APIC. */
 	retval = best_enum->apic_setup_local();
 	if (retval != 0)
 		printf("%s: Failed to setup the local APIC: returned %d\n",
 		    best_enum->apic_name, retval);
 }
-#ifdef __amd64__
-SYSINIT(apic_setup_local, SI_SUB_CPU, SI_ORDER_SECOND, apic_setup_local,
-    NULL);
-#else
-SYSINIT(apic_init, SI_SUB_CPU, SI_ORDER_SECOND, apic_init, NULL);
-#endif
+SYSINIT(apic_setup_local, SI_SUB_CPU, SI_ORDER_SECOND, apic_setup_local, NULL);
 
 /*
  * Setup the I/O APICs.
